﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace Analysis_Part
{
    [Serializable]
    public class BDDModel : IModel
    {
        [field: NonSerialized]
        public event ModelHandler0<BDDModel> load_obj;
        [field: NonSerialized]
        public event ModelHandler1<BDDModel> changed;

        private static BDDModel instance;

        public List<String> names;
        public Dictionary<string, string> bdds;


        public BDDModel()
        {
            instance = this;
            bdds = new Dictionary<string, string>();
            names = new List<String>();

        }

        public static void setInstance(BDDModel bdd)
        {
            instance = bdd;
        }

        public static BDDModel getInstance
        {

            get
            {
                return instance;
            }
        }

        public List<string> getNames(int matrix_form)
        {
            return names;
        }

        public void attach(IModelObserver imo)
        {
            load_obj += new ModelHandler0<BDDModel>(imo.ShowSelectedMatrix);
            changed += new ModelHandler1<BDDModel>(imo.ViewListChanged);
        }




        // to create the BDD variables and feautures
        public void create(string t, string[] input, string[] notused, string name, int r, int c, bool viz, int matrix_form)
        {
            if (!names.Contains(name))
            {
                LatticeBasedOnBDD.initialiseVariables(input[0]);
                LatticeBasedOnBDD.initialiseUpgradeFeatures(input[1]);
                names.Add(name);
                // create or just saving part:
                bdds.Add(name, input[0] + "-" + input[1]);
                // update gui
                updateViewList();
                // update ASR file
                updateConfigFiles(name);// represents a semiring

            }
            else MessageBox.Show("Name of BDD already exists!");

        }

        // init the corresponding preorder as active one

        public void loadBDD(string bdd)
        {
            if (names.Contains(bdd))
            {
                string[] input = ((string)bdds[bdd]).Split('-');
                LatticeBasedOnBDD.initialiseVariables(input[0]);
                LatticeBasedOnBDD.initialiseUpgradeFeatures(input[1]);
            }
            else MessageBox.Show("Name of BDD does not exist any more!");
        }


        // visualization of elements:
        public void LoadAndCompute(string bdd, string s,int matrix_form, bool c, bool opt, bool cts = false, bool upto = false, bool threshold = false, uint t = 0)
        {
            loadBDD(bdd);
            List<string> resultList = new List<string>();
            // resultList contains variables and logic operators (or, and, false and true)
            string[] input = ((string)bdds[bdd]).Split('-');
            resultList.Add(input[0]);
            resultList.Add("and" + " " + "or" + " " + "false" + " " + "true");
            load_obj.Invoke(this, new ModelEventArgs(resultList));
        }


        public void updateViewList()
        {
            changed.Invoke(this, new ModelEventArgs1("LatticeBasedOnBDD"));
        }

        private void updateConfigFiles(string name)
        {
            string sourcePath = Directory.GetCurrentDirectory();
            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString()).ToString() + "\\Semiring_Generator\\bin\\Debug\\";
            string structure = System.IO.File.ReadAllText("config_ASR.txt");
            int index = structure.LastIndexOf("#" + name + "-" + "LatticeBasedOnBDD" + "#");
            if (index < 0)
            {
                structure += "\n" + "#" + name + "-" + "LatticeBasedOnBDD" + "#";
                File.WriteAllText("config_ASR.txt", structure);
                string sourceFile = System.IO.Path.Combine(sourcePath, "config_ASR.txt");
                string destFile = System.IO.Path.Combine(targetPath, "config_ASR.txt");
                System.IO.File.Copy(sourceFile, destFile, true);
            }
        }



    }
}
