﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.Windows.Forms;

namespace Analysis_Part
{
    /// <summary>
    /// the controller class of the MVC
    /// </summary>
    public interface IController
    {
        void update_image(IView sender, ViewEvent_Selection e);
        void LoadAndCompute(IView sender, ViewEvent_Algorithms e);
        void remove_Matrix(IView sender, ViewEvent_Selection e);
        void load_Preorder(IView sender, ViewEvent_Selection e);

        IModel getTypeModel(String t);

        void SaveData(string filename);
        void setModels(Dictionary<String, IModel> models);
        Dictionary<String, IModel> getModels();
        List<String> models_forwards(int matrix_form);
    }
    public class IncController : IController
    {
        IView view;
        Dictionary<String, IModel> models;
        // The controller which implements the IController interface ties the view and model 
        // together and attaches the view via the IModelInterface and addes the event
        // handler to the view_changed function. The view ties the controller to itself.
        public IncController(IView v, Dictionary<String, IModel> models)
        {
            view = v;
            this.models = models;
            view.SetController(this);

            foreach (string s in models.Keys)
            {
                models[s].attach((IModelObserver)view);
            }

            // init the components of the view for event handling:  
            view.Create += new ViewHandler_Creation<IView>(this.view_create);
            view.NewImage += new ViewHandler_Selection<IView>(this.update_image);

            view.LoadAndCompute += new ViewHandler_Algorithms<IView>(this.LoadAndCompute);
            view.Remove += new ViewHandler_Selection<IView>(this.remove_Matrix);
            view.LoadPreorder += new ViewHandler_Selection<IView>(this.load_Preorder);
            view.StopComputation += new ViewHandler_Selection<IView>(this.stop_Computation);
        }

        public void stop_Computation(IView sender, ViewEvent_Selection e)
        {
            try
            {
                ((IModel_Extension)getTypeModel(e.semi)).stop_Computation();
            }
            catch
            {
                MessageBox.Show("Exception in AbortComputation");
            }

        }

        public void remove_Matrix(IView sender, ViewEvent_Selection e)
        {
            try
            {
                ((IModel_Extension)getTypeModel(e.semi)).removeMatrix(e.name, e.semi, e.matrix_form);
            }
            catch
            {
                MessageBox.Show("Exception in RemoveMatrix!");
            }

        }


        public void update_image(IView sender, ViewEvent_Selection e)
        {
            try
            {
                ((IModel_Extension)getTypeModel(e.semi)).updateImage(e.name, e.semi, e.viz, e.matrix_form);
            }
            catch
            {
                MessageBox.Show("Exception in UpdateImage!");
            }

        }

        // connect view to datamodel for event-interaction
        public void setModels(Dictionary<String, IModel> models)
        {
            this.models = models;
            foreach (string s in models.Keys)
            {
                models[s].attach((IModelObserver)view);
            }
        }

        public Dictionary<String, IModel>  getModels()
        {
            return models;
        }

        public void LoadAndCompute(IView sender, ViewEvent_Algorithms e)
        {
            try
            {
                getTypeModel(e.semi).LoadAndCompute(e.name, e.semi, e.matrix_form, e.complete, e.optimized, e.cts, e.upto, e.threshold, e.t);
            }
            catch
            {
                MessageBox.Show("Semiring or matrix not available! Or some exception happened during computation!");
            }
    
        }

        // only activated by FiniteLattice-Selection and BDD-Selection
        public void load_Preorder(IView sender, ViewEvent_Selection e)
        {
            try
            {
                getTypeModel(e.semi).LoadAndCompute(e.name, e.semi,0,false, false, false, false, false, 0);
            }
            catch
            {
                MessageBox.Show("Object not available!");
                view.DisableMatrixInputBox();
                view.RemoveSelection();
            }

        }

        //create matrix or preorder, preorders are saved to be consistent with config-files
        public void view_create(IView v, ViewEvent_CreateMatrix e)
        {
            try
            {

                getTypeModel(e.type).create(e.type, e.input, e.alphabet, e.name, e.r, e.c, e.viz,e.matrix_form);
                if (e.type == "PreorderModel" || e.type== "BDDModel")
                {
                    SaveData("autosave");
                }

            }
            catch(Exception ex)
            {
                MessageBox.Show("Semiring not available! (in view_create)");
                MessageBox.Show(ex.ToString());
            }
  
        }

        /// <summary>
        /// to save current state of data-models
        /// based on serialize
        /// </summary>
        /// <param name="filename"></param>
        public void SaveData(string filename)
        {
            // load autosave and produce savecopy


            // avoid deleting semirings, which had been used by last session
            // otherwise user is not able to use all his stuff including preorders!
            List<string> delete = new List<string>();
            if (filename =="autosave")
            {
                string nodelete = "";
                foreach (string s in models.Keys)
                {
                    if (models[s].getNames(-1).Count > 0 || s=="PreorderModel")
                    {
                        nodelete += s + "\n";
                        //Basetypes: do not allow to delete if in use
                        // TODO Testing
                        Type t = Type.GetType("Analysis_Part." + s);
                        GetAllBaseTypesUsed(t, ref nodelete);
                    }
                    else
                    {
                        delete.Add(s);
                    }
                }
                // file and informaton is than available for the semiring generator!
                File.WriteAllText("config_lastsession_save.txt", nodelete);
            }

            // only save used datamodels->TODO
            //for (int i = 0; i < delete.Count; i++)
            //{
            //    models.Remove(delete[i]);
            //}

                // serializable
                if (filename.Contains(".bin"))
                {
                    filename = filename.Substring(0, filename.Length - 4);
                }

            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream(filename + ".bin",
                                     FileMode.Create,
                                     FileAccess.Write, FileShare.None);

            try
            {
                formatter.Serialize(stream, models);
            }
            catch (Exception s) {
                //old copy

                MessageBox.Show(s.ToString());

            }
            stream.Close();
      
            //TODO serialize only preorder-model, for separat loading!

        }

        /// <summary>
        /// Get all Semiring-types currently used by some matrices! 
        /// Do not allow to delete this semirings, otherwise serialization would fail.
        /// </summary>
        /// <param name="t"></param>
        /// <param name="nodelete"></param>
        private void GetAllBaseTypesUsed(Type t, ref string nodelete)
        {
            FieldInfo[] info = t.GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
            for (int i = 0; i < info.Length; i++)
            {
                // ignore int, float etc. only consider Semiring-types
                string name = info[i].FieldType.FullName.Replace("Analysis_Part.", String.Empty);
                if (info[i].FieldType.FullName.Contains("Analysis_Part."))
                  {
                    nodelete += name + "\n";
                    GetAllBaseTypesUsed(info[i].FieldType, ref nodelete);
                }
            }
        }

        /// <summary>
        /// to update the view of matrix list
        /// </summary>
        /// <returns></returns>
        public List<String> models_forwards(int matrix_form)
        {

            List<String> show = new List<String>();
            foreach (string s in models.Keys)
            {
                for (int i = 0; i < models[s].getNames(matrix_form).Count(); i++)
                {
                    if (s != "PreorderModel" && s != "BDDModel") show.Add(models[s].getNames(matrix_form)[i]);
                }
            }


            return show;
        }


        /// <summary>
        /// get the datamodel by semiring-name
        /// </summary>
        /// <param name="t"></param>
        /// <returns></returns>
        public IModel getTypeModel(String t)
        {
 
                return models[t];       
        }
    }

}