
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using System.Reflection;
using GraphVizWrapper;
using GraphVizWrapper.Queries;
using GraphVizWrapper.Commands;
using System.Diagnostics;
using System.Threading;


namespace Analysis_Part
{
    /// <summary>
    /// this file contains the generic matrix-datamodel and interfaces
    /// </summary>
    /// <typeparam name="IModel"></typeparam>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void ModelHandler0<IModel>(IModel sender, ModelEventArgs e);
    public delegate void ModelHandler1<IModel>(IModel sender, ModelEventArgs1 e);
    public delegate void ModelHandler2<IModel>();

    public delegate void UpdateDimension<IModel>(IModel sender, ModelEvent_updateDimension e);

    // The ModelEventArgs class which is derived from th EventArgs class to 
    // be passed on to the controller when the value is changed
    public class ModelEventArgs : EventArgs
    {
        public List<String> newList_collection;
   
        public ModelEventArgs(List<String> list)
        {
            newList_collection = list;
        }
    }

    public class ModelEventArgs1 : EventArgs
    {
        public string newList;

        public ModelEventArgs1(  string list)
        {
            newList = list;
        }
    }

    public class ModelEvent_updateDimension : EventArgs
    {
        public int r;
        public int c;
        public string[] alphabet;
        public string[,] matrix;

        public ModelEvent_updateDimension(int r, int c, String []alphabet, String [,] matrix)
        {
            this.r=r;
            this.c=c;
            this.alphabet = alphabet;
            this.matrix = matrix;
        }
    }

    // The interface which the form/view must implement so that, the event will be
    // fired when a value is changed in the model.
    public interface IModelObserver
    {
        void ViewListChanged(IModel model, ModelEventArgs1 e);
        void ShowSelectedMatrix(IModel model, ModelEventArgs e);
        void UpdateImage();
        void UpdateDimension(IModel model, ModelEvent_updateDimension e);
    }


    public interface IModel
    {
        List<String> getNames(int matrix_form);
        void attach(IModelObserver imo);
        void updateViewList();

        void create(String t, string[] input, string[] alphabet, string name, int r, int c, bool viz, int matrix_form
            );
        void LoadAndCompute(string p, string s, int matrix_form, bool complete,bool opt, bool cts=false, bool upto=false, bool threshold=false, uint t=0);
    }

    public interface IModel_Extension
    {
        void removeMatrix(string p, string s, int matrix_form);
        void updateImage(string p, string s, bool viz, int matrix_form);
        void stop_Computation();
    }


    // Datamanagement:
    [Serializable]
    public class DataManager<T> : IModel, IModel_Extension where T : class
    {
        // events
        [field: NonSerialized]
        public event ModelHandler0<DataManager<T>> load_obj;
        [field: NonSerialized]
        public event ModelHandler1<DataManager<T>> changed;
        [field: NonSerialized]
        public event ModelHandler2<DataManager<T>> newImage;
        [field: NonSerialized]
        public event UpdateDimension<DataManager<T>> update_Dimension;

        Hashtable matrices_formversion_wa;
        Hashtable matrices_formversion_cts;
        public List<String> names;

        [field: NonSerialized]
        Thread computationThread;

        Matrix<T> currentMatrix;
        public volatile bool computeUntilEnd = true;
  

        public DataManager()
        {
            // datamanagment update: alllows loading, saving 
            matrices_formversion_wa = new Hashtable();
            matrices_formversion_cts = new Hashtable(); ;
            names = new List<String>();

        }

        public List<String> getNames(int matrix_form)
        {
            List<String> name_list =names;
            if (matrix_form == 0)
            {
                name_list= addSemiring_types(matrices_formversion_wa);

            }
            if(matrix_form==1) 
            {
                name_list = addSemiring_types(matrices_formversion_cts);
            }

            return name_list;
        }

        // update list to semiring-types
        private List<String> addSemiring_types(Hashtable matrices)
        {
            List<String> name_list = new List<string>();
            foreach (DictionaryEntry entry in matrices)
            {
                name_list.Add((String)entry.Key);
                int index = names.FindIndex(delegate (String str)
                {
                    return str== (String)entry.Key;
                }
            ); 
                name_list.Add(names.ElementAt(index+1));
            }

            return name_list;
        }

        /// <summary>
        /// if the user selects a matrix and wants to see the automata,
        /// image will be updated
        /// </summary>
        /// <param name="matrix"></param>
        /// <param name="semi"></param>
        /// <param name="viz"></param>
        public void updateImage(string matrix, string semi, bool viz, int matrix_form)
        {

            if (names.Contains(matrix))
            {

                Matrix<T> mat;
                if (matrix_form == 0) mat = (Matrix<T>)matrices_formversion_wa[matrix];
                else mat = (Matrix<T>)matrices_formversion_cts[matrix];



                //get the alphabet
                string[] alphabet;
                try
                {
                    alphabet = mat.getAlphabet();
                } catch
                {
                    // no alphabet specified by the user
                    alphabet = new string[0];
                }

                // dimension information
                updateDimension(mat.rows(), mat.columns(), alphabet, returnStringMatrix(mat));

                if (viz)
                {
                    visualizeGraph(mat, alphabet,matrix_form);
                    updateImage();
                }//  default png
                else
                {
                    System.IO.File.Copy("no_viz.png", "test_gviz.png", true);
                    updateImage();
                }

            }
        }


        public string[,] returnStringMatrix(Matrix<T> matrix)
        {
            string[,] ma = new string[matrix.rows(), matrix.columns()];
            for (int i=0; i< matrix.rows(); i++)
            {
                for (int j = 0; j< matrix.columns(); j++)
                {
                    ma[i, j] = matrix.get(i, j).ToString();
                }
            }

            return ma;
        }

        /// <summary>
        /// to delete a matrix<T>
        /// </summary>
        /// <param name="matrix"></param>
        /// <param name="semi"></param>
        public void removeMatrix(string matrix, string semi, int matrix_form)
        {
            if (names.Contains(matrix))
            {
                if (matrix_form ==0) matrices_formversion_wa.Remove(matrix);
                else matrices_formversion_cts.Remove(matrix);



                int i = names.IndexOf(matrix);
                names.RemoveAt(i + 1);
                names.Remove(matrix);
                updateViewList();
            }
        }


        /// <summary>
        /// this method is loading the selected matrix by gui-list
        /// and starts AlgorithmB
        /// complete-only result or optimized
        /// </summary>
        /// <param name="matrix"></param>
        /// <param name="semi"></param>
        /// <param name="complete"></param>
        /// <param name="optimzized"></param>
        public void LoadAndCompute(string matrix, string semi, int matrix_form,bool complete, bool optimzized, bool cts=false, bool upto=false, bool threshold=false, uint t=0)
        {
            if (names.Contains(matrix))
            {
                // during testing and developing necessary because of wrong serialization
                computeUntilEnd = true;

                Matrix<T> mat;
                if (matrix_form ==0) mat = (Matrix<T>)matrices_formversion_wa[matrix];
                else mat = (Matrix<T>)matrices_formversion_cts[matrix];
                currentMatrix = mat;
                
                List<string> resultList = resultList = new List<string>();
                resultList.Add("\nInput matrix :  \n");
                resultList.Add(mat.ToString()+"\n");

                // Thread testing for AlgorithmB-versions:
                if (optimzized)
                {
                    computationThread = new Thread(() => startComputationAlgorithmB(mat, false, true, resultList));
                }
                else if(!cts&&!upto&&!threshold)
                {
                    computationThread = new Thread(() => startComputationAlgorithmB(mat, complete, false, resultList));                                                                
                }
                if (threshold) {
                    computationThread = new Thread(() => startbonchiThreshold(mat, t));
                }
                if (upto)
                {
                    computationThread = new Thread(() => startUpTo(mat));
                }
                if (cts)
                {
                    computationThread = new Thread(() => startCTS(mat, resultList));
                }

                computationThread.Start();
                Thread.Sleep(1000);
            }
        }
    

        /// <summary>
        /// the user could interrupt the computation aborting the thread
        /// (no file-writing, no local varibales used anymore)
        /// </summary>
        public void stop_Computation()
        {
            Thread.Sleep(100);
            if (computationThread != null && computationThread.IsAlive)
            {
                try
                {
                    computeUntilEnd = false; //for resultList check, before passing to view
                    List<string> abortComputation = new List<string>();
                    abortComputation.Add("\nComputation has been interrupted for :"+"\n");
                    abortComputation.Add(currentMatrix.ToString() + "\n");
           
                    computationThread.Abort();
                    //Console.WriteLine("abort");
                    computationThread.Join(); // This makes the code here pause until the Thread exits.
                    computationThread = null;

                    computeUntilEnd = true; // set the computationUntilEnd-Flag to true again
                    load_obj.Invoke(this, new ModelEventArgs(abortComputation));

                }
                catch (Exception e) { Console.WriteLine(e.ToString()); }

            }
        }

        // computation in separat thread startet:
        public void startComputationAlgorithmB(Matrix<T> mat, bool complete, bool opt, List<string> resultList)
        {

          try {
                string info = "";
                List<Matrix<T>> result = new List<Matrix<T>>();
                if (!opt)
                {                               
                    result = mat.AlgorithmB(complete);// the computation itself!!
                    if (result.Count() > 0) info = getEquiStates(result[result.Count() - 1]);
                    foreach (Matrix<T> inside in result)
                    {
                        resultList.Add(inside.ToString());
                    }
                    resultList.Add(info);
                }



                if (opt)
                {
                    Matrix<T>[] m = mat.OptimizedAlgorithmB();
        
                    resultList.Add(m[0].ToString());
     
                    info = getEquiStates(m[0]);
                    resultList.Add(info);
                }

                
                // only passs the results, if computation has been finished without interruption!   
                if (computeUntilEnd)// volatile bool should be save enough to enable the data passing
                {
                    load_obj.Invoke(this, new ModelEventArgs(resultList));
                }
                            
            }
            catch (ThreadAbortException abortException)
            {
                Console.WriteLine("User interrupted the computation."); 
            }
            catch (Exception)
            {
                Console.WriteLine("Some error occurs while computing or passing the data to the view!");
            }
    
    
        }

        private void startbonchiThreshold(Matrix<T> mat, uint t)
        {
            if (typeof(T) == typeof(TropicalSemiringInt))
            {
                try
                {
                    //only allow for tropical-semiring-int
                    bool res = mat.bonchiThreshold(Activator.CreateInstance(typeof(T), t) as T);
                    if (computeUntilEnd == true)// volatile bool 
                    {
                        if (res) MessageBox.Show("Threshold respected.");
                        else MessageBox.Show("Threshold not respected.");
                    }
                }
                catch (ThreadAbortException)
                {
                    Console.WriteLine("User interrupted the computation.");
                }
                catch (Exception)
                {
                    Console.WriteLine("Some error occurs while computing or passing the data to the view!");
                }

            }
            else
            {
                MessageBox.Show("The invocation of the algorithm is initially only designed for the tropical SemiringInt.");
            }
        }

        private void startUpTo(Matrix<T> mat)
        {
            try
            {
                bool res = mat.bonchiVectors();
                if (res) MessageBox.Show("The vectors are language equivalent.");
                else MessageBox.Show("The vectors are not language equivalent.");
            }
            catch (ThreadAbortException)
            {
                Console.WriteLine("User interrupted the computation.");
            }
            catch (Exception)
            {
                Console.WriteLine("Some error occurs while computing or passing the data to the view!");
            }
        }

        private void startCTS(Matrix<T> mat, List<string> resultList)
        {
            try
            {
                //start CTS
                Matrix<T> bisimMatrix = Matrix<T>.ctsBisim(mat, mat);
                resultList.Add(bisimMatrix.ToString());
                if (computeUntilEnd)// volatile bool should be save enough to enable the data passing
                {
                    load_obj.Invoke(this, new ModelEventArgs(resultList));
                }
            }
            catch (ThreadAbortException)
            {
                Console.WriteLine("User interrupted the computation.");
            }
            catch (Exception)
            {
                Console.WriteLine("Some error occurs while computing or passing the data to the view!");
            }
        }

        /// <summary>
        /// all states, which are equivalent, will
        /// be write out in one line
        /// </summary>
        /// <param name="m"></param>
        /// <returns></returns>
        public string getEquiStates(Matrix<T> m)
        {
            string info = "\nEqual states are:\n";
            Dictionary<int, string> list = new Dictionary<int, string>();
            for (int i = 0; i < m.columns(); i++)
            {
                if (!list.Keys.Contains(i))
                {
                    string temp_info = "";
                    for (int j = i + 1; j < m.columns(); j++)
                    {
                        if (!list.Keys.Contains(j))
                        {
                            if (new Matrix<T>(m.column(i)) == new Matrix<T>(m.column(j)))
                            {
                                if (temp_info == "")
                                {
                                    temp_info += (j + 1).ToString();
                                }
                                else
                                {
                                    temp_info += "," + (j + 1).ToString();
                                    list.Add(j, "");
                                }
                            }
                        }
                    }
                    list.Add(i, temp_info);
                }

            }

            foreach (int str in list.Keys)
            {
                if (list[str] != "")
                {
                    info += (str + 1).ToString() + "," + list[str] + "\n";
                }
            }

            return info;
        }

        /// <summary>
        /// this method creates a matrix<T> -instanz
        /// </summary>
        /// <param name="t"></param>//type of semiring
        /// <param name="input"></param>// the matrix 
        /// <param name="alphabet"></param>
        /// <param name="name"></param>
        /// <param name="r"></param>// row
        /// <param name="c"></param>// column
        /// <param name="viz"></param>// visualization flag
        public void create(String t, String[] input, String[] alphabet, String name, int r, int c, bool viz, int matrix_form)
        {
            int a = name.IndexOf("-");
            string n = name;
            string type_info = "";
            if (a > 0)
            {
                n = name.Substring(0, a);
                type_info = name.Substring(a + 1, name.Length - a - 1);
                // update modulo or preorder depending !!!
            }

            if (!names.Contains(name))
            {
                Matrix<T> mat= new Matrix<T>(r, c);
                mat.MatrixName = name;
                if(alphabet.Length>0) mat.setAlphabet(alphabet);

                for (int i = 0; i < r; i++)
                {
                    for (int j = 0; j < c; j++)
                    {
                        Object obj = Activator.CreateInstance(typeof(T), null);// as T;
                        Type type = Type.GetType("Analysis_Part." + t);


                        try
                        {
                            MethodInfo method = type.GetMethod("allowedStrings");
                            Boolean str = (bool)method.Invoke(obj, new Object[] { input[(i * c) + j] });
                            if (str)
                            {
                                try
                                {
                                    T oItem = Activator.CreateInstance(typeof(T), input[(i * c) + j]) as T;
                                    mat.set(i, j, oItem);

                                }
                                catch (Exception e)
                                {
                                    if(input[(i * c) + j]!=null) MessageBox.Show("Check your input. Do you set all seperators: [],#  or , correctly?");
                                    return;                                 
                                }

                            }
                            else
                            {
                                MessageBox.Show("No " + input[(i * c) + j] + " allowed for this semiring!");
                                return;
                            }
                        }
                        // if the user implemented the constructor, in this case no alloed string is generated
                        catch
                        {
                            // allowedStrings method not implemented, verification should than
                            // be implemented by the user in the string-constructor
                            try
                            {
                                T oItem = Activator.CreateInstance(typeof(T), input[(i * c) + j]) as T;
                                mat.set(i, j, oItem);

                            }
                            catch
                            {
                                MessageBox.Show("Check your input. Do you set all seperators: [] and , correctly?");
                                return;
                            }

                        }
                    }
                }


                // visualize graph testing:
                if (viz) visualizeGraph(mat, alphabet,matrix_form);
                else {
                    System.IO.File.Copy("no_viz.png", "test_gviz.png", true);
                }

     
                // add name and Semiring info
                names.Add(n);
                if (a > 0) t = t + "-" + type_info;
                names.Add(t);
                // save in corresponding hashtables
                if (matrix_form == 0)
                {
                    matrices_formversion_wa.Add(n, mat);
                }
                if (matrix_form == 1)
                {
                    matrices_formversion_cts.Add(n, mat);
                }
                updateViewList();
                updateDimension(r, c, alphabet, returnStringMatrix(mat));
            }
            else MessageBox.Show("Name of Matrix already exists!");


        }

        /// <summary>
        /// this method is constructing an automate
        /// using graph-viz
        /// </summary>
        /// <param name="matrix"></param>
        /// <param name="alpha"></param>
        public void visualizeGraph(Matrix<T> matrix, String[] alpha, int matrix_form)
        {

            Type t = typeof(T);

            int form = 1;
            if (matrix_form == 1) form = 0;

            int alpabetsize = (matrix.rows() - form) / matrix.columns();
            int statesize = matrix.columns();

            Dictionary<int, string> alphabet = new Dictionary<int, string>();

            if (alpha.Length > 0)
            {
                for (int i = 0; i < alpha.Length; i++)
                {
                    alphabet.Add(i, alpha[i]);
                }

            }
            else
            {
                for (int i = 0; i < alpabetsize; i++)
                {
                    alphabet.Add(i, "a" + i);
                }
            }



            string digraph = "{ node [margin=0  width=0.5 shape=circle ] h0 [color=white fontcolor=white ]";
            for (int i = 1; i < statesize; i++)
            {
                digraph += " h" + i + " [color=white fontcolor=white ]";
            }
            digraph += "}";
            // { node [margin=0 fontcolor=blue fontsize=32 width=0.5 shape=circle style=filled] b [fillcolor=yellow fixedsize=true label="a very long label"] d [fixedsize=shape label="an even longer label"] }

            for (int i = 0; i < statesize; i++)
            {
                int state = 0;
                int count = 0;
                for (int j = 0; j < matrix.rows(); j++)
                {
                    //[label="0.2",weight="0.2"]
                    if (matrix.get(j, i) != (dynamic)t.GetMethod("Zero").Invoke(null, null))
                    {
                        if (j == matrix.rows() - 1)
                        {
                            // b [fillcolor=yellow fixedsize=true label="a very long label"]
                            // no label for final state
                            digraph += i + "->" + "h" + i + "[label=\"" + matrix.get(j, i).ToString() + "\"];";
                        }
                        else digraph += i + "->" + state + "[label=\"" + alphabet[j % alpabetsize] + ": " + matrix.get(j, i).ToString() + "\"];";

                    }
                    count++;
                    if (count == alpabetsize)
                    {
                        state++;
                        count = 0;
                    }
                }

            }


            var getStartProcessQuery = new GetStartProcessQuery();
            var getProcessStartInfoQuery = new GetProcessStartInfoQuery();
            var registerLayoutPluginCommand = new RegisterLayoutPluginCommand(getProcessStartInfoQuery, getStartProcessQuery);

            // GraphGeneration can be injected via the IGraphGeneration interface

            var wrapper = new GraphGeneration(getStartProcessQuery,
                                              getProcessStartInfoQuery,
                                              registerLayoutPluginCommand);

            byte[] output = wrapper.GenerateGraph("digraph{" + digraph + "}", Enums.GraphReturnType.Png);


            try
            {
                System.IO.FileStream _FileStream =
                   new System.IO.FileStream("test_gviz.png", System.IO.FileMode.Create,
                                            System.IO.FileAccess.Write);
                _FileStream.Write(output, 0, output.Length);
                _FileStream.Close();


            }
            catch (Exception _Exception)
            {
                // Error
                Console.WriteLine("Exception caught in process: {0}",
                                  _Exception.ToString());
            }

        }



        /// <summary>
        /// MVC event based c# elements
        /// </summary>
        public void updateViewList()
        {
            changed.Invoke(this, new ModelEventArgs1("matrix_managment"));
        }

        public void updateImage()
        {
            newImage.Invoke();
        }

        // update also matrix.inforamtion, if wanted
        public void updateDimension(int row, int col, String[] alphabet, String [,] matrix)
        {
            update_Dimension.Invoke(this, new ModelEvent_updateDimension(row,col, alphabet,matrix));
        }


        // Attach the function which is implementing the IModelObserver so that it can be fired by the form
        public void attach(IModelObserver imo)
        {
            load_obj += new ModelHandler0<DataManager<T>>(imo.ShowSelectedMatrix);
            changed += new ModelHandler1<DataManager<T>>(imo.ViewListChanged);
            newImage += new ModelHandler2<DataManager<T>>(imo.UpdateImage);
            update_Dimension += new UpdateDimension<DataManager<T>>(imo.UpdateDimension);
        }
    }
}



