﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections;
using System.Windows.Forms;
using Microsoft.VisualBasic;

namespace Analysis_Part
{
    [Serializable]
    public class FiniteLattice
    {
        public static Preorder preorder;
        private BitArray downset;
        public FiniteLattice(BitArray downset, Preorder preorder)
        {
            FiniteLattice.preorder = preorder;
            this.downset = downset;
        }
        public FiniteLattice(BitArray downset)
        {
            this.downset = downset;
        }
        public FiniteLattice(Preorder preorder)
        {
            FiniteLattice.preorder = preorder;
            this.downset = new BitArray(preorder.n);
        }

        public FiniteLattice(FiniteLattice x)
        {
            this.downset = new BitArray(preorder.n);
            this.downset = x.downset;
        }
        public FiniteLattice()
        {
            this.downset = new BitArray(preorder.n);

        }

        public FiniteLattice(string input)
        {
            this.downset = new BitArray(preorder.n);

            if (input == "+" || input.Contains("+")) this.downset = new BitArray(preorder.n).Not();
            else if(!input.Contains('-'))
            {
                string[] split = input.Split(',');
                for (int i = 0; i < split.Count(); i++)
                {
                    for (int j = 0; j < preorder.names.Count(); j++)
                    {
                        if (preorder.names[j] == split[i]) putIntoSet(j);
                    }
                }
            }
            callCheckingInput(input);
        }

        public void putIntoSet(int i)
        {
            downset[i] = true;
        }
        public void takeOutOfSet(int i)
        {
            downset[i] = false;
        }
        public void flipBit(int i){
            downset[i]=!downset[i];
        }

        /// <summary>
        /// Yields the zero element of the lattice, i.e. the bottom element.
        /// </summary>
        /// <returns>The bottom-element Elements.bot.</returns>
        public static FiniteLattice Zero()
        {
            return new FiniteLattice(new BitArray(preorder.n));
        }
        public static FiniteLattice One()
        {
            return new FiniteLattice((new BitArray(preorder.n)).Not());
        }
        /// <summary>
        /// The +-operation of the lattice, i.e. the supremum.
        /// </summary>
        /// <param name="c1">The first summand.</param>
        /// <param name="c2">The second summand.</param>
        /// <returns>The least element that is greater or equal to c1 and greater or equal to c2.</returns>
        public static FiniteLattice operator +(FiniteLattice c1, FiniteLattice c2)
        {
            return new FiniteLattice(new BitArray(c1.downset).Or(c2.downset));
        }
        /// <summary>
        /// The +-operation of the lattice, i.e. the supremum.
        /// </summary>
        /// <param name="c1">The first summand.</param>
        /// <param name="c2">The second summand.</param>
        /// <returns>The least element that is less or equal to c1 and less or equal to c2.</returns>
        public static FiniteLattice operator *(FiniteLattice c1, FiniteLattice c2)
        {
            return new FiniteLattice(new BitArray(c1.downset).And(c2.downset));
        }

        private bool recursiveApproximationCheck(BitArray checkFlag, int i)
        {
            foreach (int j in preorder.predecessors[i]) {
                if (!downset[j]) return false;
                if (!checkFlag[j] && !recursiveApproximationCheck(checkFlag, j)) {
                    checkFlag[j] = true;
                    return false;
                }
                checkFlag[j] = true;
            }
            return true;
        }
        public void approximate(){
            BitArray checkFlag = new BitArray(preorder.n);
            for(int i=0; i<preorder.n;++i)
                if(!checkFlag[i]){
                    if(!recursiveApproximationCheck(checkFlag, i)) downset[i]=false;
                    checkFlag[i] = true;
                }
            
        }


        private void recursiveDownClosure(BitArray checkFlag, int i)
        {
            foreach (int j in preorder.predecessors[i])
            {
                if (!checkFlag[j]) recursiveDownClosure(checkFlag, j);
                checkFlag[j] = true;
            }
            downset[i] = true;
        }
        public void closeDown()
        {
            BitArray checkFlag = new BitArray(preorder.n);
            for (int i = 0; i < preorder.n; ++i)
                if (!checkFlag[i])
                {
                    if (downset[i]) recursiveDownClosure(checkFlag, i);
                }
        }

        /// <summary>
        /// Computes the pseudocomplement c1/c2 usually denoted c2->c1
        /// </summary>
        /// <param name="c1">The dividend.</param>
        /// <param name="c2">The divisor.</param>
        /// <returns>c2->c1</returns>
        public static FiniteLattice operator /(FiniteLattice c1, FiniteLattice c2)
        {
            FiniteLattice result = (new FiniteLattice((new BitArray(c2.downset)).Not().Or(c1.downset)));
            result.approximate();
            return result;
        }

        public static bool operator !=(FiniteLattice c1, FiniteLattice c2)
        {
            return !(c1 == c2);
        }
        public static bool operator ==(FiniteLattice c1, FiniteLattice c2)
        {

            for (int i = 0; i < preorder.n; ++i)
                if (c1.downset[i] != c2.downset[i]) return false;
            return true;
        }
        public static bool operator <=(FiniteLattice c1, FiniteLattice c2)
        {

            for (int i = 0; i < preorder.n; ++i)
                if (c1.downset[i] && !c2.downset[i]) return false;
            return true;
        }
        public static bool operator >=(FiniteLattice c1, FiniteLattice c2)
        {

            for (int i = 0; i < preorder.n; ++i)
                if (!c1.downset[i] && c2.downset[i]) return false;
            return true;
        }
        /* Define how Field-elements should be printed, as well as the one- and the zero-element of the Field:*/
        public override string ToString()
        {
            string result = "{";
            int i = 0;
            do
            {
                if (downset[i])
                {
                    result += preorder.names[i++];
                    break;
                }
            } while (++i < preorder.n);
            while (i < preorder.n)
            {
                if (downset[i]) result += ','+preorder.names[i];
                ++i;
            }
                
            return result +"}";
        }






        /// <summary>
        /// Check whether a linear combination exists, using the the algorithm from Zhou / Li: Largest Solution of Linear Equation over the complete heyting algebra.
        /// </summary>
        /// <param name="set">The set of rows the vector shall be combined from.</param>
        /// <param name="vec">The vector (represented as an array) that may be combined linearly.</param>
        /// <returns>A LinearCombination object that indicates in the field CombinationExists, whether a linear combination was found, and, provided a combination was found, in its field Multiplicands the multiplicands that form the linear combination.</returns>
        public static LinearCombination<FiniteLattice> LinearCombination(List<FiniteLattice[]> set, FiniteLattice[] vec)
        {

            LinearCombination<FiniteLattice> lc;
            lc.CombinationExists = true;
            lc.Multiplicands = new FiniteLattice[set.Count()];
            for (int j = 0; j < lc.Multiplicands.Length; ++j)
            {
                lc.Multiplicands[j] = One();
                for (int i = 0; i < vec.Length; ++i)
                {
                    //i\in M_j(A,b)?
                    if (!(set[j][i] <= vec[i]))
                        lc.Multiplicands[j] *= vec[i] / set[j][i];
                }
            }
            //Check, if linear combination was found:
            for (int i = 0; i < vec.Length; ++i)
            {
                if (vec[i] != Matrix<FiniteLattice>.multiplyVectors(Matrix<FiniteLattice>.getColumn(set, i), lc.Multiplicands)) lc.CombinationExists = false;
            }
            return lc;
        }

        private void callCheckingInput(string str){
            
            for (int i=0; i<downset.Length;i++){

                if (downset[i])
                {
                    for (int j = 0; j < preorder.predecessors[i].Count; j++)
                    {
                        if (!downset[preorder.predecessors[i][j]])
                        {

                            DialogResult dialogResult = MessageBox.Show("Please select the next top(yes) or bottom(no) element\n of your unvalid input:" + str, "Invalid Input!", MessageBoxButtons.YesNo);
                            if (dialogResult == DialogResult.Yes)
                            {
                                this.closeDown();
                            }
                            else if (dialogResult == DialogResult.No)
                            {
                                this.approximate();
                            }
                            j = preorder.predecessors[i].Count;
                            i = downset.Length-1;
                        }

                    }

                }

        
            }
     
        }


        //update method
        public static bool allowedStrings(String s)
        {

            return true;
        }

    }
}
