﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;


namespace Analysis_Part
{
    /// <summary>
    /// this class is the view of the MVC AlgorithmB programm (LanguageEquivalenceChecking)
    /// </summary>
    public partial class Form2 : Form, IView, IModelObserver
    {
        private const string MOD = "Modulo";
        private const string FL = "FiniteLattice";
        private const string PM = "PreorderModel";
        private const string BDDString = "LatticeBasedOnBDD";
        private const string BM = "BDDModel";

        private const int RECGEN = 5; // constant for recently generated semirings

        // event handling
        IController Controller;
        // loads matrix and starts AlgorithmB
        public event ViewHandler_Algorithms<IView> LoadAndCompute;
        // to init the image for the currently selected matrix
        public event ViewHandler_Selection<IView> NewImage;
        // to Create a matrix or a preorder
        public event ViewHandler_Creation<IView> Create;
        // to remove a matrix
        public event ViewHandler_Selection<IView> Remove;
        // to load a preorder
        public event ViewHandler_Selection<IView> LoadPreorder;
        // stop computation
        public event ViewHandler_Selection<IView> StopComputation;
        // update to the ui thread if ComputationThread finished his computation
        delegate void SetTextCallback(IModel model, ModelEventArgs e);

        // View will set the associated controller, this is how view is linked to the controller.



        // test alpha-matrix:
        private readonly TextBox txt = new TextBox { BorderStyle = BorderStyle.FixedSingle, Visible = false };


        public void SetController(IController cont)
        {
            Controller = cont;
        }

        //init all the components
        public Form2()
        {
            LatticeBasedOnBDD.initialise(3/*,new bool[]{false, true, true}*/);
            LatticeBasedOnBDD l = new LatticeBasedOnBDD("a or (b and c)");
            Console.WriteLine(l);

            InitializeComponent();
            // initialization of all available semirings:
            InitComboboxes("config_main.txt");
            InitComboboxes("config_ASR.txt");
            InitComboboxes("config_DP.txt");
            InitComboboxes("config_field.txt");
            // default selection
            this.SelectMatrixform.SetItemChecked(0, true);
        }

        /// <summary>
        /// update to the current available semirings
        /// </summary>
        /// <param name="filename"></param>
        private void InitComboboxes(string filename)
        {
            string[] configSR = System.IO.File.ReadAllLines(filename);
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {
                    if (!SemiringList.Items.Contains(line.Substring(1, line.Length - 2)))
                    {
                        SemiringList.Items.Add(line.Substring(1, line.Length - 2));
                    }
                }
            }

            if (RecentlyGeneratedSR.Items.Count > RECGEN)
            {
                for (int i = 0; i > RECGEN - 1; i++)
                {
                    if (RecentlyGeneratedSR.Items.Count > RECGEN) RecentlyGeneratedSR.Items.RemoveAt(i);
                }
            }
        }

        /// <summary>
        /// If some missing input has been found in the user-input the gui
        /// is informing the user about his fault and offers the user the possibility to move the input to the front and fill the rest with zeros
        /// automaticly.
        /// </summary>
        /// <param name="matrixForm"></param>
        /// <param name="inputMissing"></param>
        /// <param name="alphabetSize"></param>
        /// <param name="row"></param>
        /// <param name="col"></param>
        /// <returns></returns>

        private bool CheckMatrixform(int matrixForm, string inputMissing, int alphabetSize, int row, int col)
        {
            bool inputRequirement = true;

            if (!String.IsNullOrWhiteSpace(inputMissing))
            {
                Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.MsgBoxResult.Yes;
                response = Microsoft.VisualBasic.Interaction.MsgBox("In the line(s) " + inputMissing.Substring(0, inputMissing.Length - 1) + " input is missing.\n" +
                "Do you want continue?\nOtherwise Zeros are used at the end to fill the gaps.", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                if (response != Microsoft.VisualBasic.MsgBoxResult.Yes) return false;
            }

            int formVariable = 1;
            if (matrixForm == 1) formVariable = 0;


                if (alphabetSize > 0 && inputRequirement)
                {
                    if ((row - formVariable) % alphabetSize != 0 || !((row - formVariable) % col == 0) || ((row - formVariable) / col) != alphabetSize)
                    {
                        inputRequirement = false;
                        MessageBox.Show("No valid n x m matrix or alphebt-size wrong!");
                    }
                }

                // check the n*m+1 x n form
                if (alphabetSize == 0 && (!((row - formVariable) % col == 0)) && inputRequirement)
                {
                    MessageBox.Show("No valid n x m matrix!");
                    inputRequirement = false;
                }
            
            return inputRequirement;
        }


        public string noMultipleChar(string str, string c)
        {
            while (str.Contains(c + c))
            {
                str=str.Replace(c + c, c);
            }
            // adapt the alphabetInput:
            if (str.Length == 1 && str[0].ToString().Equals(c)) return "";

            if (str[0].ToString().Equals(c)) str = str.Substring(1, str.Length-1);
            if (str[str.Length-1].ToString().Equals(c)) str = str.Substring(0, str.Length-1);
            return str;
        }

        /// <summary>
        /// this method takes the input of the matrixInput box and is checking, if
        /// the input is correct, means the dimension of the  matrix is of: (n*|alphabet|+1) X n
        /// or to distinguish between the two different matrix-forms
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void generateMatrix_Click_1(object sender, EventArgs e)
        {
            int matrixForm = SelectMatrixform.CheckedIndices[0];

            if (namingBox.Text != "" && CheckNaming(namingBox.Text))
            {
                // first check if type is selected:
                if ((SemiringList.SelectedIndex > -1 && RecentlyGeneratedSR.SelectedIndex < 1) || (RecentlyGeneratedSR.SelectedIndex > -1 && SemiringList.SelectedIndex < 1))
                {
                    String type = SemiringList.GetItemText(SemiringList.SelectedItem);
                    if (type == "") type = RecentlyGeneratedSR.GetItemText(RecentlyGeneratedSR.SelectedItem);
                    string type_info = "";
                    // update to Mod, FiniteLattice or BDD
                    if (type.Contains("Modulo") || type.Contains("FiniteLattice") || type.Contains("LatticeBasedOnBDD"))
                    {
                        int i = type.IndexOf("-");
                        if (i > -1)
                        {
                            type = type.Substring(i + 1, type.Length - i - 1);
                            type_info = SemiringList.GetItemText(SemiringList.SelectedItem).Substring(0, i);
                        }
                        if (type == MOD)
                        {
                            type_info = Modulo.modulo.ToString();
                        }
                    }

                    // Update
                    int col = dataGridView1.ColumnCount;
                    int row;
                    // if Alphabet defined, extract row-count
                    //TODO: do not allow multiple ,, anf Text != ,

                    if (alphabetBox.Text.Length>0 )alphabetBox.Text = noMultipleChar(alphabetBox.Text, ",");

                    string[] alphabet = alphabetBox.Text.Split(',');
                    if (alphabet.Length !=0 && alphabet[0]!="")
                    {
                        //row = (alphabet.Length * col) + 1;
                        // the same if alphabet is given
                        row = dataGridView1.RowCount-1;
                    }
                    else
                    {
                        row = dataGridView1.RowCount - 1;
                        // check if correct dimension of matrix depending on matrixform
                        if (matrixForm == 0)
                        {
                            alphabet = new String[(dataGridView1.RowCount - 2) / col];
                        }
                        else
                        {
                            alphabet = new String[(dataGridView1.RowCount - 1) / col];//CTS-form
                        }
                        for ( int l=0; l< alphabet.Length; l++)
                        {
                            alphabet[l] = "a" + l;
                            if(l==0) alphabetBox.Text="a0";
                            else alphabetBox.Text = alphabetBox.Text + "," + "a" + l;
                        }

                    }

                    int index = col * row;

                    string input_missing = "";
                    // check if some input is missing:
                    String[] matrixInput = new String[index];
                    int index2 = 0;
                    for ( int i=0; i < row; i++)
                    {
                        for (int j = 0; j < col; j++)
                        {
                            // null reference, if nothing typed
                            if(dataGridView1[j, i].Value != null)
                            {
                                if (String.IsNullOrWhiteSpace(dataGridView1[j, i].Value.ToString())) input_missing += "[" + (i+1) + "," + (j+1) + "]" + ",";
                                matrixInput[index2] = dataGridView1[j, i].Value.ToString();
                            }else
                            {
                                input_missing += "[" + (i + 1) + "," + (j + 1) + "]" + ",";
                            }
         
                     
                            index2++;
                        }
                    }


                    bool inputRequirement = CheckMatrixform(matrixForm, input_missing, alphabet.Length, row, col);


                    /*
                    String input = matrixInputInformBox.Text;

                    //  check matrix input
                    // get the row number
                    string[] inputArray = input.Split('\n');
                    List<string[]> real_input = new List<string[]>();
                    int col = 0;
                    string input_missing = "";
                    // get the col-number
                    for (int i = 0; i < inputArray.Length; i++)
                    {
                        if (!String.IsNullOrEmpty(inputArray[i]))
                        {
                            string[] temp1 = Regex.Split(inputArray[i], @"\s+");

                            //update temp
                            int count = 0;
                            for (int k = 0; k < temp1.Length; k++)
                            {
                                if (String.IsNullOrWhiteSpace(temp1[k]))
                                {
                                    count++;
                                }
                            }
                            string[] temp = new string[temp1.Length - count];
                            int index2 = 0;
                            for (int k = 0; k < temp1.Length; k++)
                            {
                                if (!String.IsNullOrWhiteSpace(temp1[k]))
                                {
                                    temp[index2] = temp1[k];
                                    index2++;
                                }
                            }


                            if (col < temp.Length) col = temp.Length;
                            // if a line with a < amount of cols occurs, something is wrong with the input, some matrix-elements where not specified by the user 
                            if (temp.Length < col) input_missing += (i + 1) + ",";

                            real_input.Add(temp);

                        }
                    }

                    // check if maybe the first line is missing some input
                    string[] firstLine = Regex.Split(inputArray[0], @"\s+");
                    if (firstLine.Length < col) input_missing += 1 + ",";
                    int row = real_input.Count;
                    int index = col * row;

   

                    // if Alphabet defined, check if consistent with the size of the matrix
                    string[] alphabet = new string[0];
                    if (!String.IsNullOrWhiteSpace(alphabetBox.Text))
                    {
                        alphabet = alphabetBox.Text.Split(',');
                    }

                    bool inputRequirement = CheckMatrixform(matrixForm, input_missing, alphabet.Length, row, col);


                    // if everthing is fine, go on
                    */
                    if (inputRequirement)
                    {
                        // prepare the semiring type:
                        String name = namingBox.Text;
                        if (type_info != "") name += "-" + type_info;

                        try
                        {
                            // call the Create-event
                            Create.Invoke(this, new ViewEvent_CreateMatrix(type, row, col, name, matrixInput, alphabet, image.Checked, matrixForm));
                        }
                        catch (Exception e1)
                        {
                            //All expected exceptions are handled directly in the Controller or the data-model.
                            MessageBox.Show( e1.ToString());
                        }

                    }                    
                }
                else
                {
                    MessageBox.Show("Please select only one structure!");
                }
                   
                }
                else MessageBox.Show("You maybe need to (re)name the matrix or remove the \"-\"."+ "\nYou maybe do not type any input?");
              

           }

        /// <summary>
        /// program does not allow to use "-" in the naming, used for parsing.
        /// </summary>
        /// <param name="text">the name for the matrix or finite lattice given by the use</param>
        /// <returns></returns>
        private bool CheckNaming(string text)
        {
            if (text.Contains("-")) return false;
            if (text.Contains("BDDModel")) return false;
            if (text.Contains("PreorderModel")) return false;
            if (text.Contains("Modulo")) return false;
            if (text.Contains("FiniteLattice")) return false;
            if (String.IsNullOrEmpty(text)) return false;

            else return true;
        }

        public void DisableMatrixInputBox()
        {
            matrixInputInformBox.Enabled = false;
            dataGridView1.Enabled = false;
        }

        public void RemoveSelection()
        {
            SemiringList.SelectedItem = null;
            SemiringList.SelectedText = "";
        }

        /// <summary>
        /// if a semiring is selected, the user should see the details for the element-input
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (SemiringList.Text != "")
            {
                finiteLatticeInfo.Items.Clear();
                if(col_nrs.Text!="")
                {
                    matrixInputInformBox.Enabled = true;
                    dataGridView1.Enabled = true;

                    matrixInputBox.Clear();
                    if (SemiringList.Text.Contains("-") && !SemiringList.Text.Contains("Modulo"))
                    {
                        string model = PM;
                        string selected = SemiringList.Text;
                        int index = selected.IndexOf("-");
                        string selectedType = selected.Substring(index + 1, selected.Length - index - 1);
                        selected = selected.Substring(0, index);
                        // distinguish Model-BDD and PreoroderModel:
                        if (selectedType.Contains("BDD")) model = BM;
                        try
                        {
                            LoadPreorder.Invoke(this, new ViewEvent_Selection(selected, model));
                        }
                        catch (Exception e4)
                        {
                            MessageBox.Show(e4.ToString());
                        }
                    }
                    else
                    {
                        // get the type to invoke the input matrix Method
                        Type t = Type.GetType("Analysis_Part." + SemiringList.Text);
                        try
                        {
                            string str = (string)t.GetMethod("inputSchema").Invoke(null, null);
                            matrixInputBox.AppendText(str);
                        }
                        catch { }

                    }
                }
                else
                {
                    MessageBox.Show("You need to determine the number of states of the matrix! \n");
                    matrixInputInformBox.Enabled = false;
                    dataGridView1.Enabled = false;
                    //SemiringList.Text = "";
                    SemiringList.SelectedText = "";
                }
           
           

            }
            else matrixInputInformBox.Enabled = false;


        }

        /// <summary>
        /// start AlgorithmB with the option to collect all results.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            if (ListMatrixSelection.SelectedItems.Count > -1)
            {
                try
                {
                    String matrix = ListMatrixSelection.SelectedItems[0].Text;
                    int index = ListMatrixSelection.SelectedItems[0].Index;
                    String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                    // update to selection name type and type_info
                    int a = semi.IndexOf("-");
                    string n = semi;
                    if (a > 0) n = semi.Substring(0, a);
                    string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                    if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                    if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                    LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n, matrix_form, true, false, false));
                }
                catch (Exception ex)
                {
                    MessageBox.Show("No matirx exist. \n" + ex.ToString());
                }
            }
            else MessageBox.Show("Select a matrix");
        }



        /// <summary>
        /// just update the image to the current matrix selection
        /// </summary>
        public void UpdateImage()
        {
            var filePath = "test_gviz.png";
            Bitmap bitmap = null;
            using (var stream = File.OpenRead(filePath))
            {
                bitmap = (Bitmap)Bitmap.FromStream(stream);
            }
            Bitmap image = bitmap;
            pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
            pictureBox1.Image = image;

        }

        /// <summary>
        /// called by changes in the models
        /// 
        ///  a semiring is removed/added
        ///  a fin. lattice is added
        /// </summary>
        /// <param name="model"></param>
        /// <param name="e"></param>
        public void ViewListChanged(IModel model, ModelEventArgs1 e)
        {
            if (e.newList == "matrix_managment")
            {
                UpdateViewList();             
            }
            if (e.newList == FL && namingBox.Text != "")
            {
                SemiringList.Items.Add(namingBox.Text + "-FiniteLattice");
                RecentlyGeneratedSR.Items.Add(namingBox.Text + "-FiniteLattice");
            }
            if (e.newList == BDDString && namingBox.Text != "")
            {
                SemiringList.Items.Add(namingBox.Text + "-" + BDDString);
                RecentlyGeneratedSR.Items.Add(namingBox.Text + "-"+BDDString);
            }

        }

        public void UpdateDimension (IModel model, ModelEvent_updateDimension e)
        {
            
            Dimension_Label.Text = e.c + " x " + e.r;
            genAlphaLine(e.c,e.r, e.alphabet);

            dataGridView1.ColumnCount= e.c;
            dataGridView1.RowCount = e.r;
            // update matrix
            for (int i=0; i< e.r; i++)
            {
                for (int j = 0; j < e.c; j++)
                {
                    dataGridView1[j, i].Value = e.matrix[i, j];
                }
            }

        }

        /// <summary>
        /// Update of the LineNumbering next to the matrix-InputBox by the alphabet and states
        /// </summary>
        /// <param name="col"></param>
        /// <param name="row"></param>
        /// <param name="symbols"></param>
        private void genAlphaLine(int col, int row, string [] symbols)
        {
            List<string> alphabet = new List<string>();

            if (symbols != null)
            {

                for (int i = 0; i < col; i++)
                {
                    for (int j = 0; j < symbols.Length; j++)
                    {
                        alphabet.Add("(" + symbols[j] + "," + (i + 1) + ")");
                    }
                }
            }
            else
            {
                // if no alphabet was defined by the user
                // update to CTS TODO
                int alphabetSize = (row - 1) / col;

                for (int i = 0; i < col; i++)
                {
                    for (int j = 0; j < alphabetSize; j++)
                    {
                        alphabet.Add("(a"+j+"," + (i + 1) + ")");
                    }
                }

            }

            alphabet.Add("\u2022");
            updateLineNumbers(alphabet,col);
        }

        /// <summary>
        /// shows only the matrix corresponding to the matrix-form selection:
        /// </summary>
        private void UpdateViewList()
        {
            ListMatrixSelection.BeginUpdate();
            ListMatrixSelection.Items.Clear();

            //string[] arr = new string[e.newList.Count];
            string[] arr = new string[Controller.models_forwards(SelectMatrixform.CheckedIndices[0]).Count()];
            arr = Controller.models_forwards(SelectMatrixform.CheckedIndices[0]).ToArray();
            for (int i = 0; i < arr.Count(); i++)
            {
                ListViewItem itm = new ListViewItem(arr[i]);
                i++;
                itm.SubItems.Add(arr[i]);
                ListMatrixSelection.Items.Add(itm);
            }
            ListMatrixSelection.EndUpdate();
            ListMatrixSelection.Refresh();

            var filePath = "test_gviz.png";
            Bitmap bitmap = null;

            using (var stream = File.OpenRead(filePath))
            {
                bitmap = (Bitmap)Bitmap.FromStream(stream);
            }


            Bitmap image = bitmap;
            pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
            pictureBox1.Image = image;

            //update line-numbering after matrix-generation:
        }

        /// <summary>
        /// update gui to selection of a preorder
        /// 
        /// or the results of AlgorithmB
        /// 
        /// same interface method for two different things, depends on datamodel-type
        /// </summary>
        /// <param name="model"></param>
        /// <param name="e"></param>
        public void ShowSelectedMatrix(IModel model, ModelEventArgs e)
        {
            if (model.GetType() == typeof(PreorderModel))
            {
                // gives the user an overviewe of the finite lattice elements:
                finiteLatticeInfo.Items.Clear();
                finiteLatticeInfo.Items.Add("-");
                for (int i = 0; i < e.newList_collection.Count; i++)
                {
                    ListViewItem itm = new ListViewItem(e.newList_collection[i]);
                    // i++;
                    // itm.SubItems.Add(e.newList[i]);
                    finiteLatticeInfo.Items.Add(itm);
                }
                finiteLatticeInfo.Items.Add("+");

            }

            if (model.GetType() == typeof(BDDModel))
            {
                // gives the user an overviewe of the finite lattice elements:
                finiteLatticeInfo.Items.Clear();
                for (int i = 0; i < e.newList_collection.Count; i++)
                {
                    ListViewItem itm = new ListViewItem(e.newList_collection[i]);
                    // i++;
                    // itm.SubItems.Add(e.newList[i]);
                    finiteLatticeInfo.Items.Add(itm);
                }

            }


            // just an update of the matrixInputinformBox with the results of the computation
            // or if it was interrupted:
            else if(model.GetType() != typeof(PreorderModel) && model.GetType() != typeof(BDDModel))
            {

                if (this.matrixInputInformBox.InvokeRequired)
                {
                    SetTextCallback d = new SetTextCallback(ShowSelectedMatrix);
                    this.Invoke(d, new object[] { model,e});
                }
                else
                {
                    matrixInputInformBox.AppendText(e.newList_collection[0]);
                    matrixInputInformBox.AppendText(e.newList_collection[1]);
                    int j = 0;
                    for (int i = 2; i < e.newList_collection.Count() - 1; i++)
                    {
                        if (i % 3 == 2)
                        {
                            j++;
                            matrixInputInformBox.AppendText("\n d" + j + " : \n");
                            matrixInputInformBox.AppendText(e.newList_collection[i]);
                        }
                        if (i % 3 == 0)
                        {
                            matrixInputInformBox.AppendText("\n m" + j + " : \n");
                            matrixInputInformBox.AppendText(e.newList_collection[i]);
                        }
                        if (i % 3 == 1)
                        {
                            matrixInputInformBox.AppendText("\n e" + j + " : \n");
                            matrixInputInformBox.AppendText(e.newList_collection[i]);
                        }
                    }

                    matrixInputInformBox.AppendText(e.newList_collection[e.newList_collection.Count() - 1]);

                    EnableAllButtons();
                    DisableEnable_ListSelection(true);
                }


            }


        }

        /// <summary>
        /// save the current state -> method for user
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void speichernToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Title = "Save your current matrices and Preorders.";
                saveFileDialog1.ShowDialog();
                if (saveFileDialog1.FileName != "")
                {
                    Controller.SaveData(saveFileDialog1.FileName);
                }
            }

            catch (Exception)
            {
                MessageBox.Show("Some error occurs.");
            }

        }

 
        /// <summary>
        /// load saved file-> method for user
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ladenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();

            openFileDialog1.InitialDirectory = "c:\\";
            openFileDialog1.FilterIndex = 2;
            openFileDialog1.RestoreDirectory = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    if ((openFileDialog1.OpenFile()) != null)
                    {
                        string filename = openFileDialog1.FileName;
                        IFormatter formatter = new BinaryFormatter();

                        try
                        {
                            Stream fromstream = new FileStream(filename,
                                          FileMode.Open,
                                          FileAccess.Read,
                                          FileShare.Read);
                            try
                            {
                                Dictionary<String, IModel> models_copy = Controller.getModels();
                                Controller.setModels((Dictionary<string, IModel>)formatter.Deserialize(fromstream));
                                PreorderModel.setInstance((PreorderModel)Controller.getModels()["PreorderModel"]);
                                // always work with PreorderModel , which is consistent in one way to the config-files 
                                // (means, if the config file includes this preorder, the datamodel too)
                                // TODO more speicherplatz-effizient solution, means a design for a relation between saved preorders and configfiles.
                                // not needed for this state of implementation, because user could not remove a fintite-lattice-preorder
                                if (models_copy["PreorderModel"].getNames(-1).Count > Controller.getModels()["PreorderModel"].getNames(-1).Count)
                                {
                                    Controller.getModels().Remove("PreorderModel");
                                    Controller.getModels().Add("PreorderModel", models_copy["PreorderModel"]);
                                    PreorderModel.setInstance((PreorderModel)Controller.getModels()["PreorderModel"]);
                                }

                                //search for inconsistent to config-files:
                                foreach (KeyValuePair<string, IModel> k in models_copy)
                                {
                                    if (!Controller.getModels().Keys.Contains(k.Key))
                                    {
                                        Controller.getModels().Add(k.Key, k.Value);
                                    }
                                }
                                Controller.setModels(Controller.getModels());

                            }
                            catch (Exception e0) { MessageBox.Show("Error: Could not read file from disk. Original error: " + e0.Message); }

                            fromstream.Close();
                            updateAfterFileloaded();
                        }
                        catch { }
                    }

                }

                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
        }

        /// <summary>
        /// update gui to saved data of the last session, which is loaded into the program by default 
        /// </summary>
        public void updateAfterFileloaded()
        {
            // update Matrix List
            ListMatrixSelection.BeginUpdate();
            ListMatrixSelection.Items.Clear();

            //string[] arr = new string[e.newList.Count];
            string[] arr = new string[Controller.models_forwards(SelectMatrixform.CheckedIndices[0]).Count()];
            arr = Controller.models_forwards(SelectMatrixform.CheckedIndices[0]).ToArray();
            for (int i = 0; i < arr.Count(); i++)
            {
                ListViewItem itm = new ListViewItem(arr[i]);
                i++;
                itm.SubItems.Add(arr[i]);
                ListMatrixSelection.Items.Add(itm);
            }
            ListMatrixSelection.EndUpdate();
            ListMatrixSelection.Refresh();
        }


        /// <summary>
        /// switch to generator
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void generator_Click(object sender, EventArgs e)
        {
            // very important, otherwise the inputs by the user of the current session are lost
            Controller.SaveData("autosave");

            // start Generator
            String genExepath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString()).ToString();
            Directory.SetCurrentDirectory(genExepath + "\\Semiring_Generator\\bin\\Debug\\");
            System.Diagnostics.Process.Start(genExepath + "\\Semiring_Generator\\bin\\Debug\\Semiring_Generator.exe");
            // close AlgorithmB Application
            Application.Exit();
            //Process.GetCurrentProcess().Kill();
        }

        /// <summary>
        /// generate finite semirings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button4_Click(object sender, EventArgs e)
        {

            if (comboBox3.SelectedIndex > -1 && CheckNaming(namingBox.Text))
            {
                string input = "";
                // if the user wants to generate a bounded distributive lattice
                if (comboBox3.Text == FL && !String.IsNullOrEmpty(namingBox.Text))
                {
                    string name = namingBox.Text;
                    int size = 0;

                    try
                    {
                        string s = (Microsoft.VisualBasic.Interaction.InputBox("Please type in the number of join-irreducible lattice elements: ", "Input"));
                        if (s != "") size = Convert.ToInt32(s);
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Please check your Input. It must be a natural number! ");
                    }

                    if (size > 0)
                    {

                        String[] inputArray = new String[size];

                        Boolean wrongInput = true;
                        while (wrongInput)
                        {
                            wrongInput = false;
                            // some input info
                            input = Microsoft.VisualBasic.Interaction.InputBox("Please enter the join- irreducible elements and the corresponding preorder like this :\n" +
                                                                                 "g>b,c;d>b\n", "Input");

                            input = input.Replace(" ", String.Empty);
                            // if not canceled
                            inputArray = input.Split(';');

                            if (input != "")
                            {
                                for (int i = 0; i < inputArray.Count(); i++)
                                {
                                    if (!inputArray[i].Contains(">")) wrongInput = true;
                                }
                            }

                        }

                        if (input != "")
                        {
                            try
                            {
                                Create.Invoke(this, new ViewEvent_CreateMatrix(PM, size, 0, name, inputArray, new string[0], false, 0));

                            }
                            catch (Exception e4)
                            {
                                MessageBox.Show("Check your Input: \n" + e4.ToString());
                            }

                        }

                    }

                }
                if (comboBox3.Text.Equals("BDD based finite Lattice") && !String.IsNullOrEmpty(namingBox.Text) && CheckNaming(namingBox.Text))
                {
                    string name = namingBox.Text;
                    string variables = "";
                    bool variableSuccess = false;

                    try
                    {
                        variables = (Microsoft.VisualBasic.Interaction.InputBox("Please type in the features. Enter the names sparated by a ',', e.g. a,b,c: ", "Features"));
                        if (LatticeBasedOnBDD.initialiseVariables(variables)) variableSuccess = true;
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("Please check your Input. You cannot name a feature 'and', 'or', 'not' or 'implies', nor can a feature's name include a ','.");
                    }

                    if (variableSuccess)
                    {
                        Boolean wrongInput = true;
                        while (wrongInput)
                        {
                            // some input info
                            input = InputBox.Show("Please enter the names of the features like this :\n" +
                                                                          "a,c\n", "Input");
                            // register cancel button has been pressed:
                            if (input == "Cancel")
                            {
                                wrongInput = false;
                            }
                            else
                            {
                                wrongInput = !LatticeBasedOnBDD.initialiseUpgradeFeatures(input);
                            }
                        }

                        if (input != "" && input != "Cancel")
                        {
                            try
                            {
                                string[] BDDInput = new string[2] { variables, input };
                                Create.Invoke(this, new ViewEvent_CreateMatrix(BM, 0, 0, name, BDDInput, new string[0], false, 0));
                            }
                            catch (Exception e4)
                            {
                                MessageBox.Show("Check your Input: \n" + e4.ToString());
                            }

                        }

                    }
                    else MessageBox.Show("Please check your Input. You cannot name a feature 'and', 'or', 'not' or 'implies', nor can a feature's name include a ','.");

                }

                // to switch the current modulo value
                if (comboBox3.Text == MOD)
                {
                    uint mod;
                    string s = (Microsoft.VisualBasic.Interaction.InputBox("Please type in the modulo value: ", "Input"));
                    if (s != "")
                        try
                        {
                            mod = Convert.ToUInt32(s);
                            Modulo.modulo = (int)mod;

                        }
                        catch (Exception)
                        {
                            MessageBox.Show("Only positiv integers!");
                        }
                }

            }
            else
            {
                if (! CheckNaming(namingBox.Text)) MessageBox.Show("First please define some name!");
                else MessageBox.Show("Please select a finite structure!");
            }
        }

        /// <summary>
        /// only selection for recently generated semirings,
        /// not updated to current program
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (RecentlyGeneratedSR.SelectedIndex > -1)
            {
                if (RecentlyGeneratedSR.Text != "") matrixInputInformBox.Enabled = true;
                else matrixInputInformBox.Enabled = false;
                string selected = RecentlyGeneratedSR.Text;

                if (RecentlyGeneratedSR.Text.Contains(FL))
                {

                    int index = selected.IndexOf("-");
                    // string selectedType = selected.Substring(index + 1, selected.Length - index - 1);
                    selected = selected.Substring(0, index);



                    try
                    {
                        LoadPreorder.Invoke(this, new ViewEvent_Selection(selected, PM));
                    }
                    catch (Exception e4)
                    {
                        MessageBox.Show(e4.ToString());
                    }
                }

            }
            else matrixInputInformBox.Enabled = false;
        }

        /// <summary>
        /// some inputs are not allowed for some semirings
        /// this catched directly b keypress of matrixInputbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void matrixBox2_KeyPress(object sender, KeyPressEventArgs e)
        {


            string type = SemiringList.Text + RecentlyGeneratedSR.Text;
            if (type == "LatticeZ") e.Handled = !((char.IsDigit(e.KeyChar) || e.KeyChar == (char)Keys.Back) || (e.KeyChar == (char)Keys.Enter) || (e.KeyChar == (char)Keys.Space) || (e.KeyChar == '-') || (e.KeyChar == '+') || (ModifierKeys == Keys.Control));
            if (type == "Complex") e.Handled = !((char.IsDigit(e.KeyChar) || e.KeyChar == (char)Keys.Back) || (e.KeyChar == (char)Keys.Enter) || (e.KeyChar == (char)Keys.Space) || (e.KeyChar == 'i') || (e.KeyChar == '.') || (ModifierKeys == Keys.Control));
            if (type == "LMonoidMaxTimes") e.Handled = !((char.IsDigit(e.KeyChar) || e.KeyChar == (char)Keys.Back) || (e.KeyChar == (char)Keys.Enter) || (e.KeyChar == (char)Keys.Space) || (e.KeyChar == '.') || (ModifierKeys == Keys.Control));
            //if (type == "Rational") e.Handled = !((char.IsDigit(e.KeyChar) || e.KeyChar == (char)Keys.Back) || (e.KeyChar == (char)Keys.Enter) || (e.KeyChar == (char)Keys.Space) || (e.KeyChar == '/'));

            if (type.Contains(FL)) e.Handled = !((FiniteLattice.preorder.names.Contains(e.KeyChar.ToString()) || e.KeyChar == (char)Keys.Back) || (e.KeyChar == (char)Keys.Enter) || (e.KeyChar == (char)Keys.Space) || (e.KeyChar == ',') || (e.KeyChar == '+') || (e.KeyChar == '-') || (ModifierKeys == Keys.Control));

            if (e.Handled)
            {
                MessageBox.Show("Check your Input. Your choosen set of input is specified by : " + SemiringList.Text + RecentlyGeneratedSR.Text);
            }
        }

        /// <summary>
        /// do not allow two semiring selctions ( recently generated combobox)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox1_MouseClick(object sender, MouseEventArgs e)
        {
            if (RecentlyGeneratedSR.Text == "")
            {

            }
            else
            {
                SemiringList.SelectedIndex = -1;
                MessageBox.Show("Select only one structure. Check the box of your semirings!");

            }
        }

        /// <summary>
        /// do not allow two semiring selctions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox2_MouseClick(object sender, MouseEventArgs e)
        {
            if (SemiringList.Text == "")
            {

            }
            else
            {
                RecentlyGeneratedSR.SelectedIndex = -1;
                MessageBox.Show("Select only one structure. Check the above matrix generation!");

            }

        }

        // do not to allow any input by the user
        private void comboBox1_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(e.KeyChar == (char)Keys.Back);

        }

        // do not to allow any input by the user
        private void comboBox2_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.Handled = !(e.KeyChar == (char)Keys.Back);

        }

        // disable matrixinputbox
        private void comboBox1_KeyUp(object sender, KeyEventArgs e)
        {
            if (SemiringList.Text == "") matrixInputInformBox.Enabled = false;
        }

        // disable matrixinputbox
        private void comboBox2_KeyUp(object sender, KeyEventArgs e)
        {
            if (RecentlyGeneratedSR.Text == "") matrixInputInformBox.Enabled = false;
        }

        /// <summary>
        /// Col-Information of matrix
        /// </summary>
        /// <param name="alphabet"></param>
        private void updateLineNumbers(List<string> alphabet, int col)
        {
     
            // testing datagridView:
            dataGridView1.ColumnCount = col;
            for (int i = 0; i < col; i++)
            {
                dataGridView1.Columns[i].Name = (i + 1).ToString();
            }

            if (alphabet.Count > 0)
            {
                dataGridView1.RowCount = alphabet.Count;

                for (int i = 0; i < dataGridView1.Rows.Count; i++)
                {
                    dataGridView1.Rows[i].HeaderCell.Value = alphabet[i];
                }
            }

            // TODO Size of first column:
            if (col > 0) dataGridView1.RowHeadersWidth = TextRenderer.MeasureText("ww (w,w)", new Font("Arial", 12.0F)).Width;
            
        }

        /// <summary>
        /// count the columns
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void matrixBox2_TextChanged(object sender, EventArgs e)
        {
     
            if (matrixInputInformBox.Lines.Count() > 0)
            {
                int index = matrixInputInformBox.SelectionStart;
                int line = matrixInputInformBox.GetLineFromCharIndex(index);
                if (line < matrixInputInformBox.Lines.Count())
                {
                    string line_text = matrixInputInformBox.Lines[line];
                    if (line_text.Length > 0)
                    {
                        string empty = line_text.Substring(0, 1);
                        while (empty == " ")
                        {
                            if (line_text.Length > 1) line_text = line_text.Substring(1, line_text.Length - 1);

                            if (line_text.Length > 0) empty = line_text.Substring(0, 1);
                            else empty = "end";
                            if (line_text.Length == 1) line_text = "";
                        }
                    }


                    string[] ele = Regex.Split(line_text, @"\s+");

                    //if (ele[ele.Length - 1] == "") col_nrs.Text = (ele.Count() - 1).ToString();
                    //else col_nrs.Text = ele.Count().ToString();
                }

      


            }
        }


        /// <summary>
        /// load the corresponding matrix as image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView1_SelectedIndexChanged(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            bool update = true;

            if (ListMatrixSelection.SelectedIndices.Count==1)
            {
                DialogResult dialogResult = MessageBox.Show("Are you sure", "Do you really want to update your matrix-values?", MessageBoxButtons.YesNo);
                if (dialogResult == DialogResult.No)
                {
                    update = false;
                }
            }



            if (ListMatrixSelection.SelectedItems.Count > 0 && update)
            {
                try
                {
                    String matrix = ListMatrixSelection.SelectedItems[0].Text;
                    int index = ListMatrixSelection.SelectedItems[0].Index;
                    String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                    // update to selection name type and type_info
                    int a = semi.IndexOf("-");
                    string n = semi;
                    if (a > 0) n = semi.Substring(0, a);
                    string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                    if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                    if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);
                    if (n == BDDString) BDDModel.getInstance.loadBDD(type_info);

                    // update corresponding buttons:
                    if (SelectMatrixform.CheckedIndices[0] == 0)
                    {
                        startAlgorithmB_complete.Enabled = true;
                        startAlgorithmB_result.Enabled = true;
                        startAlgorithmB_opt.Enabled = true;
                        start_up_to_Algorihtm.Enabled = true;
                        computeTreshold.Enabled = true;
                    }
                    if (SelectMatrixform.CheckedIndices[0] == 1)
                    {
                        computeCTS_bisim.Enabled = true;
                    }


                        NewImage.Invoke(this, new ViewEvent_Selection(matrix, n, image.Checked, matrix_form));
                        
                }
                catch (Exception ex)
                {
                    MessageBox.Show("No entry´or serialization problems occur. \n" + ex.ToString());
                }
            }
            else
            {
                // disable all buttons
                startAlgorithmB_complete.Enabled = false;
                startAlgorithmB_result.Enabled = false;
                startAlgorithmB_opt.Enabled = false;
                start_up_to_Algorihtm.Enabled = false;
                computeTreshold.Enabled = false;
                computeCTS_bisim.Enabled = false;
            }
        }

        /// <summary>
        /// AlgorithmB generation only result
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button5_Click_1(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            if (ListMatrixSelection.SelectedItems.Count > 0)
            {
                try
                {
                    String matrix = ListMatrixSelection.SelectedItems[0].Text;
                    int index = ListMatrixSelection.SelectedItems[0].Index;
                    String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                    // update to selection name type and type_info
                    int a = semi.IndexOf("-");
                    string n = semi;
                    if (a > 0) n = semi.Substring(0, a);
                    string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                    if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                    if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                    DisableAllButtons();
                    DisableEnable_ListSelection(false);
                    LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n,matrix_form,false, false, image.Checked));

                }
                catch (Exception ex)
                {

                    MessageBox.Show("No matirx exist. \n" + ex.ToString());
                    // update old Text
                }
            }
            else MessageBox.Show("Select a matrix");
        }

        private void DisableAllButtons()
        {
            startAlgorithmB_complete.Enabled = false;
            startAlgorithmB_result.Enabled = false;
            startAlgorithmB_opt.Enabled = false;
            start_up_to_Algorihtm.Enabled = false;
            computeTreshold.Enabled = false;
            computeCTS_bisim.Enabled = false;
        }

        private void EnableAllButtons()
        {
            startAlgorithmB_complete.Enabled = true;
            startAlgorithmB_result.Enabled = true;
            startAlgorithmB_opt.Enabled = true;
            start_up_to_Algorihtm.Enabled = true;
            computeTreshold.Enabled = true;
            computeCTS_bisim.Enabled = true;
        }

        private void DisableEnable_ListSelection(bool enable)
        {
            ListMatrixSelection.Enabled = enable;
        }


        /// <summary>
        /// event of button for optimized AlgorithmB version
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button3_Click(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            if (ListMatrixSelection.SelectedItems.Count > 0)
            {
                try
                {
                    String matrix = ListMatrixSelection.SelectedItems[0].Text;
                    int index = ListMatrixSelection.SelectedItems[0].Index;
                    String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                    // update to selection name type and type_info
                    int a = semi.IndexOf("-");
                    string n = semi;
                    if (a > 0) n = semi.Substring(0, a);
                    string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                    if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                    if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);


                    LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n, matrix_form, false, true, false));
                }
                catch (Exception ex)
                {
                    MessageBox.Show("No matrix exists. \n" + ex.ToString());
                }
            }
            else MessageBox.Show("Select a matrix");
        }

        /// <summary>
        /// save the current results from matrixbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveResultsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (var sfd = new SaveFileDialog())
            {
                sfd.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*";
                sfd.FilterIndex = 2;

                if (sfd.ShowDialog() == DialogResult.OK)
                {
                    if (sfd.FileName.Contains(".txt")) sfd.FileName = sfd.FileName.Substring(0, sfd.FileName.Length - 4);
                    File.WriteAllText(sfd.FileName + ".txt", matrixInputInformBox.Text);
                }
            }
        }

        /// <summary>
        /// allow to delete a matrix
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                List<ListViewItem> remove = new List<ListViewItem>();
                for (int i = 0; i < ListMatrixSelection.SelectedItems.Count; i++)
                {
                    remove.Add(ListMatrixSelection.SelectedItems[i]);
                }
                try
                {
                    // Remove from data-model
                    foreach (ListViewItem entry in remove)
                    {
                        this.Remove.Invoke(this, new ViewEvent_Selection(entry.SubItems[0].Text, entry.SubItems[1].Text, false, SelectMatrixform.CheckedIndices[0]));
                        ListMatrixSelection.Items.Remove(entry);
                    }
                }
                catch { }
            }

        }

        private void button8_Click(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            //Hier Threshold.
            try
            {
                String matrix = ListMatrixSelection.SelectedItems[0].Text;
                uint threshold = UInt32.Parse(richTextBox1.Text);
                int index = ListMatrixSelection.SelectedItems[0].Index;
                String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                // update to selection name type and type_info
                int a = semi.IndexOf("-");
                string n = semi;
                if (a > 0) n = semi.Substring(0, a);
                string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n, matrix_form, false, false, image.Checked,false,false,true, threshold));
            }
            catch (FormatException)
            {
                MessageBox.Show("You miss the treshhold input´or the type is wrong!\n");
            }
            catch (Exception ex)
            {
                MessageBox.Show("No matirx exist. \n" + ex.ToString());
            }

        }

        // start up to Algorithm
        private void button7_Click(object sender, EventArgs e)
        {
            int matrixForm = SelectMatrixform.CheckedIndices[0];
            try
            {
                String matrix = ListMatrixSelection.SelectedItems[0].Text;
                int index = ListMatrixSelection.SelectedItems[0].Index;
                String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                // update to selection name type and type_info
                int a = semi.IndexOf("-");
                string n = semi;
                if (a > 0) n = semi.Substring(0, a);
                string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n, matrixForm, false, false, image.Checked, false, true));
            }
            catch (Exception ex)
            {
                MessageBox.Show( ex.ToString());
            }
        }

        /*
         *For matrix input selection:
         * distinguish between n x (n*|Σ|+1) and n x (n*|Σ|) matrices
         * 
         * enables the corresponding Algorithm buttons
         */
        private void checkedListBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            // first update the matrix entries:
            UpdateViewList();
        }

        // always only one selection active
        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.NewValue == CheckState.Checked && SelectMatrixform.CheckedItems.Count > 0)
            {
                SelectMatrixform.ItemCheck -= checkedListBox1_ItemCheck;
                SelectMatrixform.SetItemChecked(SelectMatrixform.CheckedIndices[0], false);
                SelectMatrixform.ItemCheck += checkedListBox1_ItemCheck;
            }
            if(e.NewValue != CheckState.Checked && SelectMatrixform.CheckedItems.Count== 1)
            {
                SelectMatrixform.SetItemChecked((e.Index+1) % 2, true);
            }

        }

        private void lineNumbers_For_RichTextBox1_Click(object sender, EventArgs e)
        {

        }

        private void AbortComputation_Click(object sender, EventArgs e)
        {
            int matrix_form = SelectMatrixform.CheckedIndices[0];
            try
            {
                String matrix = ListMatrixSelection.SelectedItems[0].Text;
                int index = ListMatrixSelection.SelectedItems[0].Index;
                String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                // update to selection name type and type_info
                int a = semi.IndexOf("-");
                string n = semi;
                if (a > 0) n = semi.Substring(0, a);
                string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                StopComputation.Invoke(this, new ViewEvent_Selection(matrix, semi, false, SelectMatrixform.CheckedIndices[0]));
            }
            catch (Exception ex)
            {
                MessageBox.Show("No matirx exists. \n" + ex.ToString());
            }
        }

        private void computeCTS_bisim_Click(object sender, EventArgs e)
        {
            int matrixForm = SelectMatrixform.CheckedIndices[0];
            try
            {
                String matrix = ListMatrixSelection.SelectedItems[0].Text;
                int index = ListMatrixSelection.SelectedItems[0].Index;
                String semi = ListMatrixSelection.Items[index].SubItems[1].Text;
                // update to selection name type and type_info
                int a = semi.IndexOf("-");
                string n = semi;
                if (a > 0) n = semi.Substring(0, a);
                string type_info = semi.Substring(a + 1, semi.Length - a - 1);
                if (n == MOD) Modulo.modulo = Convert.ToInt32(type_info);
                if (n == FL) PreorderModel.getInstance.loadPreorder(type_info);

                LoadAndCompute.Invoke(this, new ViewEvent_Algorithms(matrix, n, matrixForm, false, false, image.Checked, true, false));
            }
            catch (Exception ex)
            {
                MessageBox.Show("No matirx exists. \n" + ex.ToString());
            }
        }

        private void finiteLatticeInfo_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void label6_Click(object sender, EventArgs e)
        {

        }


        /// <summary>
        /// To update the visualization of the matrix
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void alphabetBox_TextChanged(object sender, EventArgs e)
        {
            if (alphabetBox.Text != "")
            {
                int col = 0;

       
                string[] symbols= alphabetBox.Text.Split(',');

                // update visualization:
                if (col_nrs.Text != "")
                {
                    try
                    {
                        col = Int32.Parse(col_nrs.Text);
                    }
                    catch (FormatException)
                    {
                        MessageBox.Show("For number of states only integer-values, please! \n");
                    }
                    List<string> alphabet = new List<string>();

                    for ( int i=0; i < col; i++)
                    {
                        for (int j = 0; j < symbols.Length; j++)
                        {
                            alphabet.Add("(" + symbols[j] + "," + (i+1) + ")");
                        }
                    }
                    alphabet.Add("\u2022");
                    updateLineNumbers(alphabet,col);

                }

            }
        }


        private void col_nrs_TextChanged(object sender, EventArgs e)
        {
            if (col_nrs.Text != "")
            {
                int col = 0;

                try
                {
                    col = Int32.Parse(col_nrs.Text);
                }
                catch (FormatException)
                {
                    MessageBox.Show("For number of states only integer-values, please! \n");
                }

                List<string> alphabet = new List<string>();
                // update visualization:
                if (alphabetBox.Text != "")
                {

                    string[] symbols = alphabetBox.Text.Split(',');

              

                    for (int i = 0; i < col; i++)
                    {
                        for (int j = 0; j < symbols.Length; j++)
                        {
                            alphabet.Add("(" + symbols[j] + "," + (i+1) + ")");
                        }
                    }
                    alphabet.Add("\u2022");
                }

                updateLineNumbers(alphabet,col);

            }

        }


        /// <summary>
        /// Clear the dataGridView1
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            DialogResult dialogResult = MessageBox.Show("Sure", "Do you really want to update your matrix-values?", MessageBoxButtons.YesNo);
            if (dialogResult == DialogResult.Yes)
            {
                dataGridView1.Rows.Clear();
                dataGridView1.Refresh();
            }
        }

        private void namingBox_TextChanged(object sender, EventArgs e)
        {

        }
    }// class end



    // c# base for events
    // https://msdn.microsoft.com/de-de/library/900fyy8e.aspx
    public delegate void ViewHandler_Selection<IView>(IView sender, ViewEvent_Selection e);
    public delegate void ViewHandler_Algorithms<IView>(IView sender, ViewEvent_Algorithms e);

    public delegate void ViewHandler_Creation<IView>(IView sender, ViewEvent_CreateMatrix e);

    /// <summary>
    /// MVC handling for user actions based on selection
    /// </summary>
    public class ViewEvent_Selection: EventArgs
    {
        public String name = " ";
        public String semi = " ";
        public bool viz = false;
        public int matrix_form;

        public ViewEvent_Selection()
        {

        }
        public ViewEvent_Selection(String m, String s)
        {
            this.name = m;
            this.semi = s;
        }
        public ViewEvent_Selection(String m, String s, bool viz, int matrix_form)
        {
            this.name = m;
            this.semi = s;
            this.viz = viz;
            this.matrix_form = matrix_form;
        }

    }


    /// <summary>
    /// MVC handling for user actions based on selection and starting some algorithm on the selected matrix
    /// </summary>
    public class ViewEvent_Algorithms : ViewEvent_Selection
    {

        public bool complete = false;
        public bool optimized = false;
        public bool cts = false;
        public bool upto = false;
        public bool threshold = false;
        public uint t = 0;
    

        public ViewEvent_Algorithms()
        {

        }
   
        public ViewEvent_Algorithms(String m, String s, int matrix_form,bool c, bool opt, bool viz, bool cts = false, bool upto = false, bool threshold = false, uint t = 0)
        {
            this.name = m;
            this.semi = s;
            this.complete = c;
            this.optimized = opt;
            this.viz = viz;
            this.cts = cts;
            this.upto = upto;
            this.threshold = threshold;
            this.t = t;
            this.matrix_form = matrix_form;
        }
    }




    /// <summary>
    /// to Create a matrix or a preorder
    /// </summary>
    public class ViewEvent_CreateMatrix : ViewEvent_Algorithms
    {
        public int r;
        public int c;


        public String[] input;
        public String type;
        public String[] alphabet;

        public ViewEvent_CreateMatrix(String type, int r, int c, string name, String[] input, String[] alphabet, bool viz, int matrix_form)
        {
            this.type = type;
            this.r = r;
            this.c = c;
            this.name = name;
            this.input = input;
            this.alphabet = alphabet;
            this.viz = viz;
            this.matrix_form = matrix_form;
        }

    }


    //interface for event handling, based on delegeates
    public interface IView
    {
        event ViewHandler_Algorithms<IView> LoadAndCompute;
        event ViewHandler_Selection<IView> Remove;
        event ViewHandler_Selection<IView> LoadPreorder;
        event ViewHandler_Selection<IView> NewImage;
        event ViewHandler_Selection<IView> StopComputation;
        event ViewHandler_Creation<IView> Create;

        void SetController(IController cont);
        void DisableMatrixInputBox();
        void RemoveSelection();
    }
}