﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    /// <summary>
    /// The simple lattice with top, bottom, and incomparable a and b.
    /// </summary>
    [Serializable]
    class LatticeRaute
    {
        /// <summary>
        /// The type for the elements of the lattice.
        /// </summary>
        public enum Elements { bot, a, b, top };
     
        /// <summary>
        /// The element represented by the LatticeRaute object.
        /// </summary>
        Elements z;

        
        public LatticeRaute()
        {
           
        }

        /// <summary>
        /// A copy constructor, creating a new LatticeRaute object with the same value as the provided one
        /// </summary>
        /// <param name="x">The LatticeRaute object to copy.</param>
        public LatticeRaute(LatticeRaute x)
        {
            z = x.z;
        }

        /// <summary>
        /// Creates a new LatticeRaute object.
        /// </summary>
        /// <param name="x">The value the new object gets.</param>
        public LatticeRaute(Elements x)
        {
            z = x;
        }

        /// <summary>
        /// Generates a lattice element from a given string. The "inverse" to ToString.
        /// </summary>
        /// <param name="x">The string representing based on which the lattice element may be initialized.</param>
        public LatticeRaute(String x)
        {
            if (x == "0") z = Elements.bot;
            if (x == "1") z = Elements.top;
            if (x == "a") z = Elements.a;
            if (x == "b") z = Elements.b;
        }

        /* Define the necessary operators:*/
        public static bool operator >=(LatticeRaute c1, LatticeRaute c2)
        {
            return c1.z==Elements.top || c2.z==Elements.bot ||c1.z==c2.z;
        }
        public static bool operator <=(LatticeRaute c1, LatticeRaute c2)
        {
            return c2.z == Elements.top || c1.z == Elements.bot || c1.z == c2.z;
        }
        public static bool operator >(LatticeRaute c1, LatticeRaute c2)
        {
            return c1 >= c2&&c1!=c2;
        }
        public static bool operator <(LatticeRaute c1, LatticeRaute c2)
        {
            return c1 <= c2&&c1!=c2;
        }
        /// <summary>
        /// The +-operation of the lattice, i.e. the supremum.
        /// </summary>
        /// <param name="c1">The first summand.</param>
        /// <param name="c2">The second summand.</param>
        /// <returns>The least element that is bigger or equal to c1 and bigger or equal to c2.</returns>
        public static LatticeRaute operator +(LatticeRaute c1, LatticeRaute c2)
        {
            return new LatticeRaute(c1 >= c2 ? c1 : (c2>=c1?c2:new LatticeRaute(Elements.top)));
        }
        /// <summary>
        /// The *-operation of the lattice, i.e. the infimum.
        /// </summary>
        /// <param name="c1">The first factor.</param>
        /// <param name="c2">The second factor.</param>
        /// <returns>The greatest element that is smaller or equal to c1 and smaller or equal to c2.</returns>
        public static LatticeRaute operator *(LatticeRaute c1, LatticeRaute c2)
        {
            return new LatticeRaute(c1 >= c2 ? c2 : (c2 >= c1 ? c1 : new LatticeRaute(Elements.bot)));
        }

        /// <summary>
        /// Computes the pseudocomplement c1/c2 usually denoted c2->c1
        /// </summary>
        /// <param name="c1">The dividend.</param>
        /// <param name="c2">The divisor.</param>
        /// <returns>c2->c1</returns>
        public static LatticeRaute operator /(LatticeRaute c1, LatticeRaute c2)
        {
            LatticeRaute y = c1;
            LatticeRaute x = c2;
            if (x <= y) return One();
            if (x >= y && y != new LatticeRaute(Elements.bot)) return y;
            if (y == new LatticeRaute(Elements.a) || y == new LatticeRaute(Elements.b)) return y;
            if (x == new LatticeRaute(Elements.a)) return new LatticeRaute(Elements.b);
            if (x == new LatticeRaute(Elements.b)) return new LatticeRaute(Elements.a);


            return y;
        }

        public static bool operator !=(LatticeRaute c1, LatticeRaute c2)
        {
            return c1.z != c2.z;
        }
        public static bool operator ==(LatticeRaute c1, LatticeRaute c2)
        {
            return c1.z == c2.z;
        }


        /* Define how Field-elements should be printed, as well as the one- and the zero-element of the Field:*/
        public override string ToString()
        {
            if (z == Elements.a) return "a";
            if (z == Elements.b) return "b"; 
            //if (z == Elements.bot) return ""+'\u22A5';  Nur in Fensterversion verwenden, bot und top funktionieren in Konsole nicht
            //return ""+'\u22A4';
            if (z == Elements.bot) return "0";
            return "1";
        }
        /// <summary>
        /// Yields the one element of the lattice, i.e. the top element.
        /// </summary>
        /// <returns>The top-element Elements.top.</returns>
        public static LatticeRaute One()
        {
            return new LatticeRaute(Elements.top);
        }

        /// <summary>
        /// Yields the zero element of the lattice, i.e. the bottom element.
        /// </summary>
        /// <returns>The bottom-element Elements.bot.</returns>
        public static LatticeRaute Zero()
        {
            return new LatticeRaute(Elements.bot);
        }


        /// <summary>
        /// Naive check whether a linear combination exists, using the fact that only finitely many vectors can be generated. However, many possible linear combinations are ignored this way, so obviously, optimization is recommended.
        /// </summary>
        /// <param name="set">The set of rows the vector shall be combined from.</param>
        /// <param name="vec">The vector (represented as an array) that may be combined linearly.</param>
        /// <returns>A LinearCombination object that indicates in the field CombinationExists, whether a linear combination was found, and, provided a combination was found, in its field Multiplicands the multiplicands that form the linear combination.</returns>
        public static LinearCombination<LatticeRaute> LinearCombinationOld(List<LatticeRaute[]> set, LatticeRaute[] vec)
        {
     
            LinearCombination<LatticeRaute> lc;
            lc.CombinationExists = false;
            lc.Multiplicands = new LatticeRaute[set.Count()];
            for (int i = 0; i < set.Count(); ++i)
            {
                lc.Multiplicands[i] = Zero();
                if (!lc.CombinationExists)
                    if (equals(set[i], vec))
                    {
                        lc.Multiplicands[i] = One();
                        lc.CombinationExists = true;
                    }
            }
            return lc;
        }


        /// <summary>
        /// Check whether a linear combination exists, using the the algorithm from Zhou / Li: Largest Solution of Linear Equation over the complete heyting algebra.
        /// </summary>
        /// <param name="set">The set of rows the vector shall be combined from.</param>
        /// <param name="vec">The vector (represented as an array) that may be combined linearly.</param>
        /// <returns>A LinearCombination object that indicates in the field CombinationExists, whether a linear combination was found, and, provided a combination was found, in its field Multiplicands the multiplicands that form the linear combination.</returns>
        public static LinearCombination<LatticeRaute> LinearCombination(List<LatticeRaute[]> set, LatticeRaute[] vec)
        {
            
            LinearCombination<LatticeRaute> lc;
            lc.CombinationExists = true;
            lc.Multiplicands = new LatticeRaute[set.Count()];
            for (int j = 0; j < lc.Multiplicands.Length; ++j)
            {
                lc.Multiplicands[j] = One();
                for (int i = 0; i < vec.Length; ++i)
                {
                    //i\in M_j(A,b)?
                    if (!(set[j][i] <= vec[i]))
                    {
                         lc.Multiplicands[j] *= vec[i] / set[j][i];
                        // Console.WriteLine(vec[i].ToString() + " " + set[j][i].ToString() + " : " + vec[i] / set[j][i]);
                    }
                     
                }
            }
            //Check, if linear combination was found:
            for (int i = 0; i < vec.Length; ++i)
            {
                if (vec[i] != Matrix<LatticeRaute>.multiplyVectors(Matrix<LatticeRaute>.getColumn(set,i), lc.Multiplicands)) lc.CombinationExists = false;
            }
            return lc;
        }

        /// <summary>
        /// Checks whether two arrays of type LatticeRaute are equal
        /// </summary>
        /// <param name="a">The first array to be compared.</param>
        /// <param name="b">The second array to be compared.</param>
        /// <returns>true if both arrays are the same length and in each position the have the same lattice element.</returns>
        private static bool equals(LatticeRaute[] a, LatticeRaute[] b)
        {
            if (b.Length != a.Length) return false;
            for (int i = 0; i < a.Length; ++i) if (a[i] != b[i]) return false;
            return true;
        }


        public bool allowedStrings(String s)
        {
            if (s == "0") return true;
            if (s == "1") return true;
            if (s == Elements.a.ToString()) return true;
            if (s == Elements.b.ToString()) return true;

            return false;

        }


        public static string inputSchema()
        {
            return "element next next...\nPress Enter for next row";
        }




    }
}
