﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    /// <summary>
    /// This class represents all finit rings Zq
    /// </summary>
    [Serializable]
    public class Modulo : TemplateForFields
    {

        public static int modulo = 3; //default value
        int z;

        public Modulo()
        {
            z = 0;
        }

        public Modulo(Modulo x)
        {
            z = x.z;
            z = z % modulo;
            while (z < 0)
            {
                z = z + modulo;
            }
        }

        public Modulo(String x)
        {
            z = Convert.ToInt32(x);
            z = z % modulo;
            while (z < 0)
            {
                z = z + modulo;
            }
        }


        public Modulo(int x)
        {
            z = x % modulo;
            while (z < 0)
            {
                z = z + modulo;
            }
        }



        /* Define the necessary operators:*/

        // here: == means congruence
        public static bool operator !=(Modulo c1, Modulo c2)
        {
            return c1.z != c2.z;
        }
        public static bool operator ==(Modulo c1, Modulo c2)
        {
            return c1.z == c2.z;
        }

        public static Modulo operator -(Modulo c1, Modulo c2)
        {
            return new Modulo(((c1.z - c2.z) + modulo) % modulo);
        }

        public static Modulo operator +(Modulo c1, Modulo c2)
        {
            return new Modulo((c1.z + c2.z) % modulo);
        }

        public static Modulo operator *(Modulo c1, Modulo c2)
        {
            return new Modulo((c1.z * c2.z) % modulo);
        }


        // only works for fields
        // optimized: satz von fermat or eukl. Algorithmus to calculate the inverse
        public static Modulo operator /(Modulo c1, Modulo c2)
        {
            /*int mod = 0;
            for (int i = 0; i < modulo; i++)
            {
                if ((c2.z * i) % modulo == c1.z) mod = i;

            }

            return new Modulo(mod);*/
            return new Modulo(c1 * getModInverse(c2));


        }

        /// <summary>
        /// Solving: b*m + modulo *n =1
        /// </summary>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Modulo getModInverse(Modulo b)
        {

            int x = modulo;
            int x0 = 0; int x1 = 1;
            int q, t;
            int a = b.z;
            while (a > 1)
            {
                q = a / x;
                t = x; x = a % x; a = t;
                t = x0; x0 = x1 - q * x0; x1 = t;
            }

            if (x1 > 0) return new Modulo(x1);
            else return new Modulo(x1 + modulo);
        }


        public override string ToString()
        {

            return z.ToString();
        }
        /// <summary>
        /// the one element
        /// </summary>
        /// <returns></returns>
        public static Modulo One()
        {
            return new Modulo(1);
        }

        /// <summary>
        /// the zero element
        /// </summary>
        /// <returns></returns>
        public static Modulo Zero()
        {
            return new Modulo(0);
        }


        /// <summary>
        /// naive implementation of primefactorization
        /// Future work: should be replaced by quadratic sieb or other algorithms
        /// </summary>
        /// <param name="n"></param>
        /// <returns></returns>
        public static Dictionary<int, int> getPrimeFactorization(int n)
        {
            int p = 0;
            List<int> factors = new List<int>();
            Dictionary<int, int> factros_exp = new Dictionary<int, int>();
            int z = n;
            int count = 0;
            while (z > 1)
            {
                int i = 2;
                bool found = false;
                while (i * i <= z && !found)
                {
                    if (z % i == 0)
                    {
                        found = true;
                        p = i;
                        count++;
                    }
                    else
                    {

                        i = i + 1;
                        count = 0;
                    }

                }
                if (!found)
                {
                    p = z;
                }

                factors.Add(p);

                z = z / p;
            }

            p = factors[0];
            count = 1;
            for (int i = 1; i < factors.Count; i++)
            {
                if (factors[i] == p) count++;
                else
                {
                    factros_exp.Add(p, count);
                    p = factors[i];
                    count = 1;
                }
                if (i == factors.Count - 1) factros_exp.Add(p, count);
            }
            return factros_exp;

        }




        /// <summary>
        /// Hensel-Lifting based on recursion
        /// </summary>
        /// <param name="set">current A</param>
        /// <param name="vec">current b</param>
        /// <param name="n">degree of free variables</param>
        /// <param name="start">current index of combination</param>
        /// <param name="p">prime number</param>
        /// <param name="exp">exponent</param>
        /// <param name="combination">current combination</param>
        /// <param name="solutions">collect all solutions</param>
        /// <param name="h">current exponent</param>
        public static void getSolutionsHL(List<Modulo[]> A, Modulo[] b, List<Modulo[]> set, Modulo[] vec, int n, int start, int p, int exp, ref LinearCombination<Modulo> combination, ref List<LinearCombination<Modulo>> solutions, int h, ref LinearCombination<Modulo> lc, ref Permutation per)
        {
            if (h < exp && solutions.Count == 0)
            {
                // get transformation for Ax =b mod p:

                if (n == 0)
                {
                    Modulo.modulo = p;

                    //copies:
                    List<Modulo[]> setT = DeepRowListCopy(set);
                    Modulo[] vecCopy = DeepArrayCopy(vec);
                    DownwardElimination(setT, vecCopy, ref per);
                    //check if some 0 0 0 | >0 exist -> no solution 
                    int nosolution = 0;

                    // check how many zero-rows we have:
                    n = 0;
                    for (int i = 0; i < setT.Count; i++)
                    {
                        nosolution = 0;
                        if (vecCopy[i].z == 0)
                        {
                            for (int j = 0; j < setT[i].Length; j++)
                            {

                                if (setT[i][j].z != 0) j = setT[i].Length;
                                if (j == setT[i].Length - 1 && setT[i][j].z == 0) n++;

                            }
                            // adapt system to nxn if n+1 rows are zero rows:
                            if (i >= setT[i].Length) n--;

                        }
                        else
                        {
                            for (int j = 0; j < setT[i].Length; j++)
                            {
                                if (setT[i][j].z != 0) nosolution = 1;
                            }
                            if (nosolution == 0)
                            {
                                lc.CombinationExists = false;
                                return;
                            }

                        }
                    }

                    if (n == 0)
                    {
                        // get solution of current lifting-step
                        LinearCombination<Modulo> x = UpwardElimination(setT, vecCopy);
                        per.ApplyTo<Modulo>(x.Multiplicands);
                        h++;
                        lc.CombinationExists = x.CombinationExists;
                        //x = x0 + x1*p^1 + x2*p^2 ...
                        // add
                        int mod = (int)Math.Pow(p, h - 1);
                        if (h - 1 > 0 && x.CombinationExists && h <= exp)
                        {
                            Modulo.modulo = (int)Math.Pow(p, h);
                            MultiplyArray(x.Multiplicands, new Modulo(mod));
                            AddArray(lc.Multiplicands, x.Multiplicands);
                        }
                        else if (h <= exp)
                        {
                            lc.Multiplicands = DeepArrayCopy(x.Multiplicands);
                        }
                        // push solution:
                        if (x.CombinationExists && h < exp)
                        {


                            // to p*p...
                            Modulo.modulo = (int)Math.Pow(p, h + 1);
                            // here we need the starting Ax=b:
                            Modulo[] Ax = MultiplyCopyList(A, lc.Multiplicands);
                            Modulo[] xi = DeepArrayCopy(b);

                            SubtractArray(xi, Ax);

                            mod = (int)Math.Pow(p, h);
                            Modulo[] check1 = DeepArrayCopy(xi);
                            //integer mode for Division
                            for (int j = 0; j < xi.Length; j++)
                            {
                                xi[j].z = xi[j].z / mod;
                            }

                            // check if xi is an integer
                            Modulo[] check2 = MultiplyCopyArray(xi, new Modulo(mod));
                            bool integer = equals(check1, check2);
                            if (integer)
                            {

                                // no permutation needed because of next pushing step
                                per = new Permutation();
                                // starting matrix A
                                // and manipulated vector b
                                getSolutionsHL(A, b, A, xi, n, Math.Min(A[0].Length - 1, b.Length - 1), p, exp, ref combination, ref solutions, h, ref lc, ref per);
                            }
                            else
                            {
                                lc.CombinationExists = false;
                                return;
                            }
                        }
                        else if (h < exp)
                        {
                            lc.CombinationExists = false;
                            return;
                        }


                    }// end if n==0
                    else
                    {   
                        // get next combination n!=0
                        getSolutionsHL(A, b, setT, vecCopy, n, Math.Min(A[0].Length - 1, b.Length - 1), p, exp, ref combination, ref solutions, h, ref lc, ref per);
                    }

                }
                else
                {
                    // more than one solution path:
                    if (start == Math.Min(set[0].Length - 1, b.Length - 1))
                    {
                        combination.Multiplicands = new Modulo[Math.Min(set[0].Length, vec.Length)];
                    }
                    if (start >= Math.Min(set[0].Count() - 1, b.Length - 1) - n && start > -1)
                    {
                        int value = 0;
                        while (value < p)
                        {
                            // deep copy of temporary state depending on modulo
                            Modulo.modulo = p;
                            if (h != 0) Modulo.modulo = (int)Math.Pow(p, h);
                            LinearCombination<Modulo> state = new LinearCombination<Modulo>();
                            LinearCombination<Modulo> stateCombi = new LinearCombination<Modulo>();
                            state.CombinationExists = lc.CombinationExists;
                            if (lc.Multiplicands != null) state.Multiplicands = DeepArrayCopy(lc.Multiplicands);
                            combination.Multiplicands[start] = new Modulo(value);
                            if (combination.Multiplicands != null) stateCombi.Multiplicands = DeepArrayCopyHL(combination.Multiplicands, start, n);

                            getSolutionsHL(A, b, set, vec, n, start - 1, p, exp, ref combination, ref solutions, h, ref lc, ref per);
                            if (solutions.Count > 0)
                            {
                                // stop searching through combinations, because solution is found
                                return;
                            }
                            //reset last pushing step for other path
                            if (stateCombi.Multiplicands != null) combination.Multiplicands = DeepArrayCopyHL(stateCombi.Multiplicands, start, n);
                            else
                            {
                                combination.Multiplicands = new Modulo[set[0].Count()];
                            }
                            if (state.Multiplicands != null) lc.Multiplicands = DeepArrayCopy(state.Multiplicands);
                            else
                            {
                                lc.Multiplicands = null;
                            }
                            lc.CombinationExists = state.CombinationExists;
                            value++;
                        }

                    }
                    else
                    {
                        // update A and b to solution combination:
                        for (int i = Math.Min(set[0].Length - 1, b.Length - 1); i >= Math.Min(set[0].Length, b.Length) - n; i--)
                        {
                            set[i][i] = new Modulo(1);
                            vec[i] = combination.Multiplicands[i];
                        }


                        // try out solution path with current permutation:
                        // manipulated A and b
                        getSolutionsHL(A, b, set, vec, 0, Math.Min(A[0].Length - 1, b.Length - 1), p, exp, ref combination, ref solutions, h, ref lc, ref per);
                    }
                }
            }

            // add solution, if pushing was succesful
            if (h == exp && lc.CombinationExists)
            {
                // deep copy
                LinearCombination<Modulo> add = new LinearCombination<Modulo>();
                add.CombinationExists = true;
                add.Multiplicands = DeepArrayCopy(lc.Multiplicands);
                solutions.Add(add);

            }
        }

        /// <summary>
        /// copy method for temporary state
        /// </summary>
        /// <param name="x"></param>
        /// <param name="start"></param>
        /// <param name="n"></param>
        /// <returns></returns>
        public static Modulo[] DeepArrayCopyHL(Modulo[] x, int start, int n)
        {
            Modulo[] result = new Modulo[x.Length];
            for (int i = x.Length - 1; i >= start; i--) result[i] = new Modulo(x[i]);
            return result;
        }

        /// <summary>
        /// chinese remainder
        /// </summary>
        /// <param name="m1"></param>
        /// <param name="M1"></param>
        /// <returns></returns>
        public static int crt_ggt(int m1, int M1)
        {

            int x = M1;
            int x0 = 0; int x1 = 1;
            int q = 0, t = 0;
            int a = m1;
            while (a > 1)
            {
                q = a / x;
                t = x; x = a % x; a = t;
                t = x0; x0 = x1 - q * x0; x1 = t;
            }

            return t;
        }


        /// <summary>
        /// Primfactorization; Hensel-Lifting; Chinese Remainder
        /// </summary>
        /// <param name="set">Transposed set from AlgorithmB</param>
        /// <param name="vec">Ax=b and b=vec</param>
        /// <returns></returns>
        public static LinearCombination<Modulo> startHenselLifting(List<Modulo[]> set, Modulo[] vec)
        {
            int n = modulo;
            LinearCombination<Modulo> ret = new LinearCombination<Modulo>();
            Dictionary<int, int> primefactors = getPrimeFactorization(modulo);
            Dictionary<int, List<LinearCombination<Modulo>>> solutions = new Dictionary<int, List<LinearCombination<Modulo>>>();

            foreach (int i in primefactors.Keys)
            {
                List<LinearCombination<Modulo>> solution_set = new List<LinearCombination<Modulo>>();
                LinearCombination<Modulo> combination = new LinearCombination<Modulo>();
                LinearCombination<Modulo> lc = new LinearCombination<Modulo>();
                Permutation per = new Permutation();
                // prepare set and vec for primepower^exp:
                Modulo.modulo = (int)Math.Pow(i, primefactors[i]);
                List<Modulo[]> setT = Transpose(DeepRowListCopy(set));
                Modulo[] vecCopy = DeepArrayCopy(vec);
                getSolutionsHL(setT, vecCopy, setT, vecCopy, 0, 0, i, primefactors[i], ref combination, ref solution_set, 0, ref lc, ref per);
                if (solution_set.Count() == 0)
                {
                    ret.CombinationExists = false;
                    //reset to modulo
                    Modulo.modulo = n;
                    return ret;
                }
                else solutions.Add(i, solution_set);
            }

            if (primefactors.Count() == 1)
            {
                foreach (int i in primefactors.Keys)
                {
                    return solutions[i][0];
                }

            }
            // reset to modulo 
            // start chinese Remainder Theorem:
            Modulo.modulo = n;
            int[] crt_coe = new int[primefactors.Count];
            int index = 0;
            foreach (int i in primefactors.Keys)
            {
                int M = n / (int)Math.Pow(i, primefactors[i]);
                crt_coe[index] = crt_ggt(M, (int)Math.Pow(i, primefactors[i])) * M;
                index++;
            }

            // take first solutions:
            index = 0;
            foreach (int i in primefactors.Keys)
            {
                if (ret.Multiplicands == null)
                {
                    ret.Multiplicands = MultiplyCopyArray(solutions[i][0].Multiplicands, new Modulo(crt_coe[index]));
                }
                else
                {
                    AddArray(ret.Multiplicands, MultiplyCopyArray(solutions[i][0].Multiplicands, new Modulo(crt_coe[index])));
                }
                index++;
            }
            ret.CombinationExists = true;
            return ret;
        }



        /// <summary>
        /// get the prime -factors of a natural number mod
        /// quadratic sieb
        /// </summary>
        /// <param name="mod">natural number</param>
        /// <returns>a list with primes and corresponding exponents</returns>
        public Dictionary<int, int> getPrimeFactors(int mod)
        {
            Dictionary<int, int> primefactors = new Dictionary<int, int>();

            return primefactors;
        }



        /// <summary>
        /// multiply and copy
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static Modulo[] MultiplyCopyList(List<Modulo[]> a, Modulo[] b)
        {
            Modulo[] result = new Modulo[a.Count];
            for (int i = 0; i < a.Count; i++)
            {
                result[i] = a[i][0] * b[0];
                for (int j = 1; j < a[i].Length; j++)
                {
                    result[i] += a[i][j] * b[j];
                }

            }
            return result;
        }



        /* Naive implementation of Gaußian elimination. Replace this method's body if
        you want to implement another algorithm for finding linear combinations. 
        Otherwise, leave intact. */
        public static LinearCombination<Modulo> LinearCombination(List<Modulo[]> set, Modulo[] vec)
        {

            List<Modulo[]> setT = Transpose(DeepRowListCopy(set));
            Permutation p = new Permutation();
            Modulo[] vecCopy = DeepArrayCopy(vec);
            LinearCombination<Modulo> lc;

            if (isPrime(modulo))
            {
                DownwardElimination(setT, vecCopy, ref p);
                lc = UpwardElimination(setT, vecCopy);
                p.ApplyTo<Modulo>(lc.Multiplicands);

            }
            else
            {
                // gauss-based algorithm:
                //lc = GaussEliminationWMI(setT, vecCopy, p);


                /*
                 * Prime-Factorization
                 * Hensel-Lifting 
                 * Chinise-remainder
                */
                lc = startHenselLifting(set, vec);

            }


            return lc;
        }

        /// <summary>
        ///  GaussMultiplication without multiplicative inverse (WMI) for every element
        ///  1. Step: adapted downward-elimination
        ///  2. Step call solving method for dapted upward-elimination
        /// </summary>
        /// <param name="set"></param>
        /// <param name="vec"></param>
        /// <param name="p"></param>
        /// <returns></returns>
        public static LinearCombination<Modulo> GaussEliminationWMI(List<Modulo[]> set, Modulo[] vec, Permutation p)
        {
            LinearCombination<Modulo> lc = new LinearCombination<Modulo>();
            lc.CombinationExists = true;

            List<Modulo[]> s = new List<Modulo[]>();
            for (int i = 0; i < set.Count(); i++)
            {
                s.Add(CopyArray(set[i]));
            }
            Modulo[] check = CopyArray(vec);



            if (set.Count() == 0)
            {
                lc.CombinationExists = false;
                return lc;
            }

            // multiply with coefficient, no matter if Nullteiler or not
            // ---------->
            for (int i = 0; (i < set[0].Length) && (i < set.Count()); ++i)
            {
                if (set[i][i] != Zero())
                {
                    Modulo[] v = new Modulo[set[0].Length];
                    Modulo[] sub = new Modulo[set[0].Length];
                    // go down
                    for (int j = i + 1; j < set.Count(); ++j)
                    {
                        if (set[j][i] != Zero())
                        {
                            for (int l = 0; l < set[0].Length; l++)
                            {
                                v[l] = set[j][i] * set[i][l];
                                sub[l] = set[i][i] * set[j][l];
                            }
                            vec[j] = vec[i] * set[j][i] - vec[j] * set[i][i];
                            set[j] = CopyArray(v);
                            SubtractArray(set[j], sub);
                        }

                    }
                }
                else
                {
                    int k = i;
                    int j = i;
                    for (j = i; (j < set.Count()) && (set[j][k] == Zero()); ++j)
                    {
                        for (k = i; (k < set[0].Length) && (set[j][k] == Zero()); ++k) ;
                        if (k == set[0].Length) k = i;
                        if (set[j][k] != Zero()) break;
                    }


                    if ((j < set.Count()) && (set[j][k] != Zero()))
                    {

                        swapListIndex(set, i, j);
                        swapArrayIndex(vec, i, j);
                        swapArrayIndexInList(set, i, k);
                        p.AddIndexPair(i--, k); // i needs to be explored again in the next step, therefore --.
                    }

                }

            }


            // 2 part adpated upward elimination


            lc.Multiplicands = new Modulo[set[0].Length];
            lc = trySolving(min(set[0].Length, vec.Length) - 1, lc, set, vec, p, s, check);

            return lc;
        }


        /// <summary>
        /// this method is searching trought all possible solutions-paths of downwarded A'x=b' one guilty solution for the 
        /// LGS Ax=b
        /// </summary>
        /// <param name="start"></param>
        /// <param name="lc"></param>
        /// <param name="set"></param>
        /// <param name="vec"></param>
        /// <param name="p"></param>
        /// <param name="s"></param>
        /// <param name="check"></param>
        /// <returns></returns>
        public static LinearCombination<Modulo> trySolving(int start, LinearCombination<Modulo> lc, List<Modulo[]> set, Modulo[] vec, Permutation p, List<Modulo[]> s, Modulo[] check)
        {

            if (start < 0 && lc.CombinationExists)
            {
                // check if solution works
                for (int i = set[0].Length; i < vec.Length; ++i) if (vec[i] != Zero()) lc.CombinationExists = false;
                for (int i = vec.Length; i < set[0].Length; ++i) lc.Multiplicands[i] = Zero();
                // copy state for wrong solution:
                Modulo[] copy_lc = new Modulo[lc.Multiplicands.Length];
                for (int l = 0; l < lc.Multiplicands.Length; l++) copy_lc[l] = new Modulo(lc.Multiplicands[l]);

                if (lc.CombinationExists)
                {
                    p.ApplyTo<Modulo>(lc.Multiplicands);
                    for (int c = 0; c < s.Count(); c++)
                    {
                        Modulo trial = new Modulo();
                        for (int l = 0; l < s[c].Count(); l++)
                        {
                            trial += s[c][l] * lc.Multiplicands[l];
                        }
                        if (trial != check[c]) lc.CombinationExists = false;
                        // if solution does not work for Ax=b, update to state before:
                        if (!lc.CombinationExists)
                        {
                            for (int l = 0; l < lc.Multiplicands.Length; l++) lc.Multiplicands[l] = new Modulo(copy_lc[l]);
                            c = s.Count();
                        }
                    }
                }

                // return
            }
            else if (start < set[0].Length && start > -1)
            {
                int i = start;

                if (vec[i] != Zero() && set[i][i] == Zero())
                {
                    lc.CombinationExists = false;
                    return lc;
                }
                else
                {
                    // search for possible solutions of the i. equation
                    List<Modulo> solutions = new List<Modulo>();
                    if (getGGT(modulo, set[i][i].z) == 1 && set[i][i].z != 0)
                    {
                        // if no Nullteiler, a multiplicative Inverse exists
                        Modulo x = vec[i] / set[i][i];
                        if ((x * set[i][i]) == vec[i])
                        {
                            lc.CombinationExists = true;
                            solutions.Add(new Modulo(x));
                        }
                    }
                    else
                    {   // otherwise we ge a solution-set, maybe containing a guilty component of the soluiton-vector
                        for (int j = 0; j < modulo; j++)
                        {
                            if (set[i][i] * new Modulo(j) == vec[i])
                            {
                                lc.CombinationExists = true;
                                solutions.Add(new Modulo(j));
                            }
                        }

                    }

                    if (solutions.Count() == 0)
                    {
                        lc.CombinationExists = false;
                        return lc;
                    }

                    // go throught all possible solution-paths
                    for (int j = 0; j < solutions.Count(); j++)
                    {
                        lc.CombinationExists = true;
                        lc.Multiplicands[start] = solutions[j];
                        // prepare depending on temporary solution 
                        if (i > 0)
                        {
                            // generate copys before recursion
                            List<Modulo[]> copy_set = new List<Modulo[]>();
                            for (int l = 0; l < set.Count(); l++)
                            {
                                copy_set.Add(CopyArray(set[l]));
                            }
                            Modulo[] copy_vec = CopyArray(vec);
                            for (int k = i - 1; k > -1; k--)
                            {
                                copy_set[k][i] *= solutions[j];
                                copy_vec[k] -= copy_set[k][i];
                                copy_set[k][i] = new Modulo(0);
                            }
                            lc = trySolving(i - 1, lc, copy_set, copy_vec, p, s, check);
                            if (lc.CombinationExists) return lc;
                        }
                        else
                        {
                            lc = trySolving(i - 1, lc, set, vec, p, s, check);
                            if (lc.CombinationExists) return lc;
                        }
                    }


                }

            }


            return lc;
        }

        /// <summary>
        /// Helping methods for calculation-steps
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static int euklid(int a, int b)
        {
            if (b == 0)
                return a;
            else
                return euklid(b, a % b);
        }
        public static int getGGT(int a, int b)
        {

            if (b > 0)
            {
                int r = a % b;//division with rest
                a = b;
                b = r;
                a = getGGT(a, b);
            }
            return a;

        }


        /* From here on out, all functions can remain unchanged, except for the obvious
         * necessity to exchange all "Templates" with the name of your field*/
        public static Modulo[] CopyArray(Modulo[] a)
        {
            Modulo[] result = new Modulo[a.Length];
            for (int i = 0; i < a.Length; ++i) result[i] = new Modulo(a[i]);
            return result;
        }

        public static List<Modulo[]> DeepRowListCopy(List<Modulo[]> set)
        {
            List<Modulo[]> result = new List<Modulo[]>();
            for (int i = 0; i < set.Count(); ++i)
            {
                Modulo[] row = new Modulo[set[i].Length];
                for (int j = 0; j < set[i].Length; ++j)
                {
                    row[j] = new Modulo(set[i][j]);
                }
                result.Add(row);
            }
            return result;
        }

        public static Modulo[] DeepArrayCopy(Modulo[] x)
        {
            Modulo[] result = new Modulo[x.Length];
            for (int i = 0; i < x.Length; ++i) result[i] = new Modulo(x[i]);
            return result;
        }

        public static Modulo[] MultiplyCopyArray(Modulo[] a, Modulo b)
        {
            Modulo[] result = CopyArray(a);
            MultiplyArray(result, b);
            return result;
        }
        public static void SubtractArray(Modulo[] a, Modulo[] b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] - b[i];
        }
        public static void AddArray(Modulo[] a, Modulo[] b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] + b[i];
        }
        public static void SubtractArray(Modulo[] a, Modulo b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] - b;
        }
        public static void AddArray(Modulo[] a, Modulo b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] + b;
        }

        public static void swapArrayIndex(Modulo[] a, int i, int j)
        {
            swapArrayIndex<Modulo>(a, i, j);
        }
        public static void swapListIndex(List<Modulo> l, int i, int j)
        {
            swapListIndex<Modulo>(l, i, j);
        }

        public static void swapArrayIndexInList(List<Modulo[]> l, int i, int j)
        {
            swapArrayIndexInList<Modulo>(l, i, j);
        }




        public static void DownwardElimination(List<Modulo[]> set, Modulo[] vec, ref Permutation p)
        {
            if (set.Count() == 0) return;
            for (int i = 0; (i < set[0].Length) && (i < set.Count()); ++i)
            {

                //Console.WriteLine("i: " + i.ToString() + ", vec.Length: " + vec.Length.ToString() + ", set[i].Length: " + set[i].Length.ToString());
                if (set[i][i] != Zero())
                {
                    vec[i] /= set[i][i];
                    DivideArray(set[i], set[i][i]);
                    for (int j = i + 1; j < set.Count(); ++j)
                    {
                        vec[j] -= vec[i] * set[j][i];
                        SubtractArray(set[j], MultiplyCopyArray(set[i], set[j][i]));
                    }
                }
                else
                {
                    int k = i;
                    int j = i;
                    for (j = i; (j < set.Count()) && (set[j][k] == Zero()); ++j)
                    {
                        for (k = i; (k < set[0].Length) && (set[j][k] == Zero()); ++k) ;
                        if (k == set[0].Length) k = i;
                        if (set[j][k] != Zero()) break;
                    }

                    if ((j < set.Count()) && (set[j][k] != Zero()))
                    {

                        swapListIndex(set, i, j);
                        swapArrayIndex(vec, i, j);
                        swapArrayIndexInList(set, i, k);
                        p.AddIndexPair(i--, k); // i needs to be explored again in the next step, therefore --.
                    }
                }
            }
        }

        public static LinearCombination<Modulo> UpwardElimination(List<Modulo[]> set, Modulo[] vec)
        {
            LinearCombination<Modulo> lc = new LinearCombination<Modulo>();
            lc.CombinationExists = true;
            if (set.Count() == 0)
            {
                lc.CombinationExists = false;
                return lc;
            }
            lc.Multiplicands = new Modulo[set[0].Count()];


            for (int i = min(set[0].Length, vec.Length) - 1; i > -1; --i)
            {
                if (vec[i] != Zero() && set[i][i] == Zero()) lc.CombinationExists = false;
                for (int j = i - 1; j > -1; --j)
                {
                    vec[j] -= vec[i] * set[j][i];
                    SubtractArray(set[j], MultiplyCopyArray(set[i], set[j][i]));
                }
                lc.Multiplicands[i] = vec[i];
            }

            // difference between unkowns and vec.length
            for (int i = set[0].Length; i < vec.Length; ++i) if (vec[i] != Zero()) lc.CombinationExists = false;
            for (int i = vec.Length; i < set[0].Length; ++i) lc.Multiplicands[i] = Zero();

            return lc;
        }





        protected static List<Modulo[]> Transpose<Modulo>(List<Modulo[]> set)
        {
            List<Modulo[]> result = new List<Modulo[]>();
            if (set.Count() == 0) return result;
            for (int i = 0; i < set[0].Length; ++i)
            {
                Modulo[] row = new Modulo[set.Count()];
                for (int j = 0; j < set.Count(); ++j) row[j] = set[j][i];
                result.Add(row);
            }
            return result;
        }

        public static void DivideArray(Modulo[] a, Modulo b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] / b;
        }
        public static void MultiplyArray(Modulo[] a, Modulo b)
        {
            for (int i = 0; i < a.Length; ++i) a[i] = a[i] * b;
        }


        /// <summary>
        /// this is the random-solution algorithms for prime-fields
        /// by Prasad Raghavendra
        /// and some example code, but not consistent with the probability of the paper:
        /// http://helikoid.si/cms/index.php?option=com_content&view=article&id=74:randomized-ls-clojure&catid=38:general&Itemid=58
        /// 
        /// </summary>
        /// <param name="set"></param>
        /// <param name="vec"></param>
        /// <returns></returns>
        public static LinearCombination<Modulo> RandomLinearCombination(List<Modulo[]> set, Modulo[] vec)
        {
            LinearCombination<Modulo> lc;
            lc.CombinationExists = false;
            lc.Multiplicands = new Modulo[set[0].Count()];
            int n = set[0].Count();

            // create random Set
            int ran = 0;
            while (ran < 145 && !lc.CombinationExists)
            {
                lc.CombinationExists = true;
                ran += 30;
                int N = (int)((ran * modulo * modulo * Math.Log(modulo * n)));

                List<Modulo[]> setT = new List<Modulo[]>();
                Random rand = new Random();
                for (int i = 0; i < N; i++)
                {
                    // column * lc = vec , which means lc.count= n
                    Modulo[] temp = new Modulo[n];
                    for (int j = 0; j < set[0].Count(); j++)
                    {
                        temp[j] = (new Modulo(rand.Next(modulo)));
                    }
                    setT.Add(temp);
                }

                // number of vectors in set = lenght of vec:
                int size = set.Count();// == vec.Count()


                // Pick out all vectors that not satisfies the i. equatation:
                for (int i = 0; i < vec.Count(); ++i)
                {
                    List<int> pickout = new List<int>();
                    for (int j = 0; j < N; j++)
                    {
                        // satisfieng the ith constraint ?
                        if (!checkVector(set[i], setT[j], vec[i].z))
                        {
                            pickout.Add(j);
                            if (pickout.Count() > N)
                            {
                                // pickout get greater than N, no solution vector available 
                                lc.CombinationExists = false;
                                // only return false, if ran > 145, based on paper
                                if (ran > 145) return lc;
                            }
                        }
                    }
                    List<Modulo[]> list = new List<Modulo[]>();
                    for (int p = 0; p < N; p++)
                    {
                        // get all possible solutions:
                        if (!pickout.Contains(p)) list.Add(setT[p]);
                    }
                    // Recombine after the i. equ.:
                    if (pickout.Count() < N && i < vec.Count()) setT = recombine(list, N, n);
                    else
                    {
                        i = vec.Count();
                        lc.CombinationExists = false;
                        if (ran > 145) return lc;
                    }
                }
                // choose randomly one of the solution vectors:
                if (setT.Count() > 0) lc.Multiplicands = setT[rand.Next(setT.Count())];
                else lc.CombinationExists = false;

            }// end random while

            return lc;
        }

        // 
        public static bool checkVector(Modulo[] vec1, Modulo[] vec2, int b)
        {
            int result = 0;
            for (int i = 0; i < vec1.Count(); i++)
            {
                result += (vec1[i] * vec2[i]).z;
            }



            if ((result % modulo) == b) return true;
            else return false;
        }

        /// <summary>
        /// recombine by algorithm:
        /// https://rjlipton.wordpress.com/2012/08/09/a-new-way-to-solve-linear-equations/
        /// </summary>
        /// <param name="list"></param>
        /// <param name="N"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        public static List<Modulo[]> recombine(List<Modulo[]> list, int N, int size)
        {
            List<Modulo[]> setT = list;

            List<Modulo[]> S = new List<Modulo[]>();
            Random rand = new Random();
            for (int i = 0; i < N; i++)
            {
                // init possible solution vector
                Modulo[] temp = new Modulo[size];
                for (int s = 0; s < size; s++) temp[s] = new Modulo();

                for (int j = 0; j < modulo + 1; j++)
                {
                    // pick randomly vector from solution set
                    int ran = rand.Next(setT.Count);
                    for (int l = 0; l < size; l++)
                    {
                        // add the picked vector:
                        temp[l] = (temp[l] + setT[ran][l]);
                    }
                }
                S.Add(temp);
            }



            return S;
        }

        // equal test for modulo-vectors
        private static bool equals(Modulo[] a, Modulo[] b)
        {
            if (b.Length != a.Length) return false;
            for (int i = 0; i < a.Length; ++i) if (a[i] != b[i]) return false;
            return true;
        }

        /// <summary>
        /// naive implemantation
        /// </summary>
        /// <param name="number"></param>
        /// <returns></returns>
        public static bool isPrime(int number)
        {
            int boundary = (int)Math.Floor(Math.Sqrt(number));

            if (number == 1) return false;
            if (number == 2) return true;

            for (int i = 2; i <= boundary; ++i)
            {
                if (number % i == 0) return false;
            }

            return true;
        }

        /// <summary>
        /// input-checking method
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        public static bool allowedStrings(String s)
        {
            try
            {
                Convert.ToInt32(s);
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        /// <summary>
        /// Input information method
        /// </summary>
        /// <returns></returns>
        public static string inputSchema()
        {
            return "int next next int...\nPress Enter for next row";
        }


    }
}
