﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    /// <summary>
    /// A structure to store permutations and apply permutations to lists.
    /// </summary>
    public struct Permutation
    {   
        /// <summary>
        /// A simple structure to store to int values, interpreted as indexes of a list.
        /// </summary>
        struct IndexPair
        {
            public int i;
            public int j;
            public IndexPair(int i, int j)
            {
                this.i = i;
                this.j = j;
            }
        }

        /// <summary>
        /// List of all pairs of indexes that may be swapped by the permutation.
        /// </summary>
        List<IndexPair> indexes;
        //public Permutation(){
        //    indexes = new List<IndexPair>();
        //}
        /// <summary>
        /// Adds one index pair that is being swapped by the permutation.
        /// </summary>
        /// <param name="i">The index of the first element to be swapped.</param>
        /// <param name="j">The index of the second element to be swapped.</param>
        public void AddIndexPair(int i, int j){
            if (indexes==null) indexes=new List<IndexPair>();
            indexes.Add(new IndexPair(i, j));
        }
        /// <summary>
        /// Swaps two elements in a list according to a given index pair.
        /// </summary>
        /// <typeparam name="T">The type of the list where the swapping may be performed.</typeparam>
        /// <param name="l">The list where the swapping may be performed.</param>
        /// <param name="p">The index pair that denotes which elements may be swapped. Note that both index pair values need to be valid indexes in the list.</param>
        private void Swap<T>(List<T> l, IndexPair p)
        {
            T x = l[p.i];
            l[p.i] = l[p.j];
            l[p.j] = x;
        }
        /// <summary>
        /// Applies the permutation to a list.
        /// </summary>
        /// <typeparam name="T">Type of the list elements.</typeparam>
        /// <param name="l">The list the permutation shall be applied to. Note that the list needs to have enough elements that each index pair only has valid indexes of the list.</param>
        public void ApplyTo<T>(List<T> l)
        {
            if (indexes == null) return;
            for (int i = 0; i < indexes.Count(); ++i)
            {
                Swap<T>(l, indexes[i]);
            }
        }
        /// <summary>
        /// Swaps two elements in an array according to a given index pair.
        /// </summary>
        /// <typeparam name="T">The type of the array where the swapping may be performed.</typeparam>
        /// <param name="l">The array where the swapping may be performed.</param>
        /// <param name="p">The index pair that denotes which elements may be swapped. Note that both index pair values need to be valid indexes in the array.</param>
        private void Swap<T>(T[] l, IndexPair p)
        {
            T x = l[p.i];
            l[p.i] = l[p.j];
            l[p.j] = x;
        }
        /// <summary>
        /// Applies the permutation to an array.
        /// </summary>
        /// <typeparam name="T">Type of the array elements.</typeparam>
        /// <param name="l">The array the permutation shall be applied to. Note that the array needs to have enough elements that each index pair only has valid indexes of the array.</param>
        public void ApplyTo<T>(T[] l)
        {
            if (indexes == null) return;
            for (int i = indexes.Count()-1; i > -1; --i)
            {
                Swap<T>(l, indexes[i]);
            }
        }
    }
}
