﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;

namespace Analysis_Part
{
    [Serializable]
    public class Polynom<T> where T :  new()
    {

        private T[] mem0;


        public Polynom()
        {
        }

        public Polynom(Polynom<T> x)
        {
            this.mem0 = new T[x.mem0.Length];
            for (int i = 0; (i < x.mem0.Length); i = (i + 1))
            {
                this.mem0[i] = x.mem0[i];
            }
            this.mem0 = ifZeroCut(this.mem0);
        }

        public Polynom(T[] a0)
        {
            this.mem0 = new T[a0.Length];
            for (int i = 0; (i < a0.Length); i = (i + 1))
            {
                this.mem0[i] = a0[i];
            }
            this.mem0 = ifZeroCut(this.mem0);
        }


        /// <summary>
        /// The string constructor for the Polynom T
        /// </summary>
        /// <param name="str">the str parameter should be like this example: 2x + x^2 + -3x^5</param>
        public Polynom(string str)
        {
            string temp = "";
            int member = 0;
            for (int i = 0; (i < str.Length); i = (i + 1))
            {
                if ((str[i] != '[' && str[i] != ']'))
                {
                    temp += str[i];
                }
                if (((str[i] == ']')
                            || (i == str.Length - 1)))
                {
                    if ((member == 0))
                    {
                        int maxsize = 0;
                        Dictionary<int, T> index_value = generateStringToAttribute(temp, ref maxsize);
                        // enables dynamic x^2 + x^6 ...
                        this.mem0 = new T[maxsize +1];
                        //initialize with Zero
                        for (int l = 0; l < maxsize + 1; l++) this.mem0[l] = getZero();
                            foreach (KeyValuePair<int, T> entry in index_value)
                            {
                                this.mem0[entry.Key] = entry.Value;
                            }

                    }
                    temp = "";
                    member++;
                }
            }
            this.mem0 = ifZeroCut(this.mem0);
        }

       /// <summary>
       /// The + operator for two Polynoms T
       /// </summary>
        /// <param name="a">The first summand.</param>
        /// <param name="b">The sec summand.</param>
       /// <returns>returns the sum of two polynoms over some field</returns>
        public static Polynom<T> operator +(Polynom<T> a, Polynom<T> b)
        {
            Polynom<T> ret = new Polynom<T>();
            if ((a.mem0.Length >= b.mem0.Length))
            {
                ret.mem0 = new T[a.mem0.Length];
                for (int i = 0; (i < a.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] = a.mem0[i];
                }
                for (int i = 0; (i < b.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] += (dynamic)b.mem0[i];
                }
            }
            else
            {
                ret.mem0 = new T[b.mem0.Length];
                for (int i = 0; (i < b.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] = (b.mem0[i]);
                }
                for (int i = 0; (i < a.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] += (dynamic)a.mem0[i];
                }
            }
            ret.mem0 = ifZeroCut(ret.mem0);
            return ret;
        }


        //TODO : implement better template for faster multiplication
        /// <summary>
        /// The * operator for two Polynoms T
        /// </summary>
        /// <param name="a">The first multiplicand.</param>
        /// <param name="b">The sec multiplicand.</param>
        /// <returns>returns the multiplication result of two polynoms over some field based on convolution</returns>
        public static Polynom<T> operator *(Polynom<T> a, Polynom<T> b)
        {
            Polynom<T> ret = new Polynom<T>();
            ret.mem0 = new T[a.mem0.Length + b.mem0.Length];
            for (int i = 0; i < a.mem0.Length + b.mem0.Length; i++) ret.mem0[i] = getZero();

                for (int j = 0; (j < a.mem0.Length); j = (j + 1))
                {
                    for (int i = 0; (i < b.mem0.Length); i = (i + 1))
                    {
                        ret.mem0[i + j] += (dynamic)a.mem0[j] * (dynamic)b.mem0[i];
                    }
                }
            ret.mem0 = ifZeroCut(ret.mem0);
            return ret;
        }

        // necessary operator : -
        public static Polynom<T> operator -(Polynom<T> a, Polynom<T> b)
        {
            Polynom<T> ret = new Polynom<T>();
            if ((a.mem0.Length >= b.mem0.Length))
            {
                ret.mem0 = new T[a.mem0.Length];
                for (int i = 0; (i < a.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] = a.mem0[i];
                }
                for (int i = 0; (i < b.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] -= (dynamic)b.mem0[i];
                }
            }
            else
            {
                ret.mem0 = new T[b.mem0.Length];
                for (int i = 0; (i < b.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] = (-((dynamic)b.mem0[i]));
                }
                for (int i = 0; (i < a.mem0.Length); i = (i + 1))
                {
                    ret.mem0[i] += (dynamic)a.mem0[i];
                }
            }
            ret.mem0 = ifZeroCut(ret.mem0);
            return ret;
        }


        /// <summary>
        /// Division with rest for two Polynoms
        /// </summary>
        /// <param name="a">the divident</param>
        /// <param name="b">the divisor</param>
        /// <returns>returns the remainder</returns>
        public static Polynom<T> operator /(Polynom<T> a, Polynom<T> b)
        {
            Polynom<T> qout = new Polynom<T>();
            Polynom<T> rest = new Polynom<T>();
            Polynom<T> divisor = new Polynom<T>(a);
            Polynom<T> divident = new Polynom<T>(b);
            int restDimension = a.mem0.Length - b.mem0.Length + 1;
            rest.mem0 = new T[b.mem0.Length];
            qout.mem0 = new T[restDimension];
            for (int j = restDimension - 1; (j >= 0); j = (j - 1))
            {
                qout.mem0[j] = (dynamic)divisor.mem0[j + (dynamic)divident.mem0.Length - 1] / (dynamic)divident.mem0[divident.mem0.Length - 1];
                for (int i = divident.mem0.Length - 1; (i >= 0); i = (i - 1))
                {
                    divisor.mem0[j + i] = (dynamic)divisor.mem0[j + i] - (dynamic)divident.mem0[i] * (dynamic)qout.mem0[j];
                }
            }

            for (int j = b.mem0.Length - 1; j >= 0; j--)
            {
                rest.mem0[j] = divisor.mem0[j];
            }

            qout.mem0 = ifZeroCut(qout.mem0);
            return qout;
        }

        /// <summary>
        /// Division with rest for two Polynoms
        /// </summary>
        /// <param name="a">the divident</param>
        /// <param name="b">the divisor</param>
        /// <returns>returns the rest</returns>
        public static Polynom<T> operator %(Polynom<T> a, Polynom<T> b)
        {
            Polynom<T> qout = new Polynom<T>();
            Polynom<T> rest = new Polynom<T>();
            Polynom<T> divisor = new Polynom<T>(a);
            Polynom<T> divident = new Polynom<T>(b);
            int restDimension = a.mem0.Length - b.mem0.Length + 1;
            rest.mem0 = new T[b.mem0.Length];
            qout.mem0 = new T[restDimension];
            for (int j = restDimension - 1; (j >= 0); j = (j - 1))
            {
                qout.mem0[j] = (dynamic)divisor.mem0[j + (dynamic)divident.mem0.Length - 1] / (dynamic)divident.mem0[divident.mem0.Length - 1];
                for (int i = divident.mem0.Length - 1; (i >= 0); i = (i - 1))
                {
                    divisor.mem0[j + i] = (dynamic)divisor.mem0[j + i] - (dynamic)divident.mem0[i] * (dynamic)qout.mem0[j];
                }
            }

            for (int j = b.mem0.Length - 1; j >= 0; j--)
            {
                rest.mem0[j] = divisor.mem0[j];
            }

            rest.mem0 = ifZeroCut(rest.mem0);
            return rest;
        }

        /// <summary>
        /// > operator
        /// needed for shortFraction for fieldextension on euclidean rings
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns>returns true if a has a greater grad as b</returns>
        public static bool operator >(Polynom<T> a, Polynom<T> b)
        {
            bool ret = true; 
            if (a == b) return false;
            // the grad is the same
            if (a.mem0.Length == b.mem0.Length)
            {
                return false;
                /*
                for (int i = a.mem0.Length-1; (i >= 0); i = (i - 1))
                {
                    if ((dynamic)a.mem0[i] <= (dynamic)b.mem0[i]) return false;
                }
                 * */
            }
            else
            {
                if (a.mem0.Length > b.mem0.Length) return true;
                else return false;
            }


            return ret;
        }
        

        /// <summary>
        /// < operator
        /// needed for the shortFraction for fieldextension on euclidean rings
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns>returns true if a < b</returns>   
        public static bool operator <(Polynom<T> a, Polynom<T> b)
        {
            bool ret = true; 
            if (a == b) return false;
            if (a>b) return false;

            return ret;
        }
        
        /// <summary>
        /// simple minus operator for a Polynom T
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        public static Polynom<T> operator -(Polynom<T> a)
        {
            Polynom<T> b = new Polynom<T>(a);
            for (int i = 0; (i < a.mem0.Length); i = (i + 1))
            {
                b.mem0[i] = -(dynamic)a.mem0[i];
            }
            return b;
        }

        public  Polynom<T> divideConstant()
        {
            T b = this.mem0[0];
            Polynom<T> ret = new Polynom<T>(this.mem0);
            for (int i = 0; (i < this.mem0.Length); i = (i + 1))
            {
                ret.mem0[i]= ret.mem0[i] /(dynamic)b;
            }
            return ret;
        }

        /// <summary>
        /// the == operator for two Polynoms T
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static bool operator ==(Polynom<T> a, Polynom<T> b)
        {
            bool ret = true;
            ret = a.mem0.Length == b.mem0.Length;
            if (ret) for (int i = 0; i < a.mem0.Length; i++)
            { 
                ret = (dynamic)a.mem0[i] == (dynamic)b.mem0[i];
                if (!ret) return ret;
            }

            return ret;
        }


        /// <summary>
        /// the != operator for two Polynoms T
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        /// <returns></returns>
        public static bool operator !=(Polynom<T> a, Polynom<T> b)
        {
            return !(a == b);
        }

        // Yields the Zero Element:
        public static Polynom<T> Zero()
        {
            T[] x0 = new T[1];
            x0[0] = getZero();
            return new Polynom<T>(x0);
        }

        // Yields the One Element:
        public static Polynom<T> One()
        {
            T[] x0 = new T[1]; 
            x0[0] = getOne();
            return new Polynom<T>(x0);
        }

        /// <summary>
        /// Yields the one of the semiring. Either T needs to be of type Semiring (an euclidean ring) or double for this to work.
        /// 
        /// </summary>
        /// <returns>The one of the given semiring.</returns>
        public static T getOne()
        {
            if ((typeof(T) == typeof(int)) || (typeof(T) == typeof(uint)) || (typeof(T) == typeof(double))) return (dynamic)1;
            try
            {
                Type t = typeof(T);

                return (dynamic)t.GetMethod("One").Invoke(null, null);
            }
            catch (Exception)
            {
                throw (new Exception("One for type " + typeof(T).ToString() + " not defined! Define function One() in " + typeof(T).ToString() + " that yields the one of said semiring."));
            };
        }

        /// <summary>
        /// Yields the zero of the semiring.  Either T needs to be of type Semiring (an euclidean ring) or double for this to work.
        /// </summary>
        /// <returns>The zero of the given semiring.</returns>
        public static T getZero()
        {
            if ((typeof(T) == typeof(int)) || (typeof(T) == typeof(uint)) || (typeof(T) == typeof(double))) return (dynamic)0;
            try
            {
                Type t = typeof(T);

                return (dynamic)t.GetMethod("Zero").Invoke(null, null);
            }
            catch (Exception e)
            {
                throw (new Exception("Zero for type " + typeof(T).ToString() + " not defined! Define function Zero() in " + typeof(T).ToString() + " that yields the zero of said semiring."));
            };

        }

        /// <summary>
        /// The ToString method for the Polynom
        /// </summary>
        /// <returns>returns the Polynom in the well known schema: x^2 + 3x^4 ...</returns>
        public override string ToString()
        {
            string array0 = "";
            for (int i = 0; (i < mem0.Length); i = (i + 1))
            {
                if(typeof(T)== typeof(double)) {
                    if (i != mem0.Length - 1 && (dynamic)mem0[i]!=0) array0 += mem0[i].ToString().Replace(',', '.') + "x^" + i + " + ";
                    else if ((dynamic)mem0[i] != 0) array0 += mem0[i].ToString().Replace(',', '.') + "x^" + i;
                }else{
                    if (i != mem0.Length - 1 && (dynamic)mem0[i] != getZero()) array0 += mem0[i].ToString() + "x^" + i + " + ";
                    else if ((dynamic)mem0[i] != getZero()) array0 += mem0[i].ToString() + "x^" + i;
                }
      
            }
            return (array0);
        }






        /// <summary>
        /// This method converts the string input by the user to the corresponding type of the attribut.
        /// Polynom string input example for type Double: 1x^2 + 5x^4
        /// cooresponds to array:  new double[5];
        /// array[2]=1, array[4]=5
        /// schema of input: _x^2 + _x^4 and _ corresponding string depending on semiring
        /// </summary>
        /// <param name="temp"></param>
        /// <param name="maxsize"></param>
        /// <returns></returns>
        public static System.Collections.Generic.Dictionary<int, T> generateStringToAttribute(string temp, ref int maxsize)
        {
            Dictionary<int, T> index_value = new System.Collections.Generic.Dictionary<int, T>();
            string array = "";

            string[] split = temp.Split('+');
            for (int j = 0; (j < split.Length); j = (j + 1))
            {
                for (int k = 0; (k < split[j].Length); k = (k + 1))
                {
                    if ((split[j][k] != 'x'))
                    {
                        array += split[j][k];// containing the value _
                    }
                    if (split[j][k] == 'x' || k == split[j].Length-1)// for the case x^0
                    {
                        int index = 0;
                        if (split[j][k] == 'x')
                        {
                            int l = split[j].Length;
                            if (l - 1 > k)
                            {
                                index = Convert.ToInt32(split[j].Substring(k + 2,l-k-2));
                            }
                            else index = 1;// for x^1 = x
                        }
                        if ((maxsize < index))
                        {
                            maxsize = index;
                        }
                        Type t = typeof(T);
                       // ConstructorInfo ctor = t.GetConstructor(new[] { typeof(String) });
                   
                        if (typeof(T) == typeof(double))
                        {
                            if (array == "") array = "1";
                            index_value.Add(index, (dynamic)Convert.ToDouble(array));
                        }
                        else
                        {
                            if (array == "") array = t.GetMethod("One").Invoke(null, null).ToString().Replace("/",",");
                            // not using the constructor directly because of constrictions of the generic  new T()
                            try
                            {

                                ConstructorInfo ctor = t.GetConstructor(new[] { typeof(String) });
                                object instance = ctor.Invoke(new object[] { array });
                                index_value.Add(index, (dynamic)instance);
                            }
                            catch (Exception)
                            {
                                throw (new Exception("Some Problems occurs while instance initialization."));
                            };


                        }
                        k = split[j].Length;
                    }
                }
                array = "";
            }
            return index_value;
        }



        /// <summary>
        /// This method verifies the string input.
        /// Based on the stringToAttribute Method for string constructor
        /// </summary>
        /// <param name="str"></param>
        /// <returns>returns if the string is ok</returns>
        public static bool allowedStrings(string str)
        {
            bool ret = true;
            string temp = "";
            for (int i = 0; (i < str.Length); i = (i + 1))
            {
                if ((str[i] != '[') && str[i] == '[')
                {
                    temp += str[i];
                }
                if (((str[i] == ']')
                            || (i == str.Length - 1)))
                {

                        try
                        {
                            int max = 0;
                            generateStringToAttribute(temp, ref max);
                        }
                        catch (System.Exception e0)
                        {
                            return false;
                        }

                        temp = "";
                 }
              
             
            }
            return ret;
        }



        /// <summary>
        /// If an array has only fields with Zero elements at the end of the array,
        /// all those fields are cut of. Enables a unique Zero or One represantation
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        public static T[] ifZeroCut(T[] a)
        {
            if (a.Length == 1) return a;
            if (((dynamic)a[a.Length - 1] != getZero()))
            {
                return a;
            }
            else
            {
                T[] ret = new T[a.Length - 1];
                for (int i = 0; (i < a.Length - 1); i = (i + 1))
                {
                    ret[i] = a[i];
                }
                return ifZeroCut(ret);
            }
        }
    }
}
