﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;

namespace Analysis_Part
{
    [Serializable]
    public class PreorderModel : IModel
    {
        [field: NonSerialized]
        public event ModelHandler0<PreorderModel> load_obj;
        [field: NonSerialized]
        public event ModelHandler1<PreorderModel> changed;

        private static PreorderModel instance;

        public List<String> names;
        public Dictionary<string, Preorder> preorders;


        public PreorderModel()
        {
            instance = this;
            preorders = new Dictionary<string, Preorder>();
            names = new List<String>();

        }

        public static void setInstance(PreorderModel mod)
        {
            instance = mod;
        }

        public static PreorderModel getInstance
        {
            get
            {
                return instance;
            }
        }

        public List<string> getNames(int matrix_form)
        {
            return names;
        }

        public void attach(IModelObserver imo)
        {
            load_obj += new ModelHandler0<PreorderModel>(imo.ShowSelectedMatrix);
            changed += new ModelHandler1<PreorderModel>(imo.ViewListChanged);
        }



        public void updateViewList()
        {
            changed.Invoke(this, new ModelEventArgs1("FiniteLattice"));
        }

        // to create the preorder: t: name, input the user input, int r = irred. elements
        public void create(string t, string[] input, string[] notused, string name, int r, int c, bool viz, int matrix_form)
        {
            if (!names.Contains(name))
            {
                Preorder pre = new Preorder(r);
                string element = "";
                int index = r - 1;
                int tempIndex = index;
                for (int i = 0; i < input.Count(); i++)
                {
                    string temp = input[i];
                    for (int j = 0; j < temp.Length; j++)
                    {
                        if (temp[j] != '>') element += temp[j];

                        if (temp[j] == '>' || j == temp.Length - 1)
                        {
                            j++;
                            temp = temp.Substring(j, temp.Length - j);

                            if (!pre.names.Contains(element))
                            {
                                pre.setAlias(index, element);
                                tempIndex = index;
                                index--;
                            }
                            else
                            {
                                for (int a = 0; a < pre.predecessors.Count(); a++)
                                {
                                    if (pre.names[a] == element) tempIndex = a;
                                }
                            }



                            string[] down = temp.Split(',');
                            for (int l = 0; l < down.Count(); l++)
                            {
                                if (down[l] != "")
                                {
                                    if (!pre.names.Contains(down[l]))
                                    {
                                        pre.setAlias(index, down[l]);
                                        pre.addOrderPair(index, tempIndex);
                                        index--;
                                    }
                                    else
                                    {
                                        for (int a = 0; a < pre.predecessors.Count(); a++)
                                        {
                                            if (pre.names[a] == down[l])
                                            {
                                                pre.addOrderPair(a, tempIndex);
                                            }
                                        }
                                    }

                                }


                            }
                            element = "";
                            j = input[i].Length;

                        }
                    }
                }


                if (index <= 0)
                {
                    names.Add(name);
                    preorders.Add(name, pre);
                    names.Add("FiniteLattice");
                    FiniteLattice.preorder = pre;

                    // update gui
                    updateViewList();
                    // update ASR-File
                    string sourcePath = Directory.GetCurrentDirectory();
                    string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString()).ToString() + "\\Semiring_Generator\\bin\\Debug\\";
                    string structure = System.IO.File.ReadAllText("config_ASR.txt");
                    index = structure.LastIndexOf("#" + name + "-" + "FiniteLattice" + "#");
                    if (index < 0)
                    {
                        structure += "\n" + "#" + name + "-" + "FiniteLattice" + "#";
                        File.WriteAllText("config_ASR.txt", structure);
                        string sourceFile = System.IO.Path.Combine(sourcePath, "config_ASR.txt");
                        string destFile = System.IO.Path.Combine(targetPath, "config_ASR.txt");
                        System.IO.File.Copy(sourceFile, destFile, true);
                    }
                    
                }
                else MessageBox.Show("Please try it again and follow the example.");

            }
            else MessageBox.Show("Name of structure already exists!");

        }

        // init the corresponding preorder as active one

        public void loadPreorder(string p)
        {
            FiniteLattice.preorder = preorders[p];
        }


        /// <summary>
        /// visualization of elements:
        /// </summary>
        public void LoadAndCompute(string p, string s,int matrix_form, bool c, bool opt, bool cts = false, bool upto = false, bool threshold = false, uint t = 0)
        {
            loadPreorder(p);
            List<string> resultList = new List<string>();

            for (int i = 0; i < FiniteLattice.preorder.n; i++)
            {

                string input = i.ToString();
                string downset = FiniteLattice.preorder.names[i].ToString();
                /*for (int j = 0; j < FiniteLattice.preorder.predecessors[i].Count; j++)
                {
                        downset +=","+ FiniteLattice.preorder.names[FiniteLattice.preorder.predecessors[i][j]] ;
                }*/
                if (FiniteLattice.preorder.predecessors[i].Count > 0) getDownSet(ref downset, i, 0);
                //resultList.Add(input);
                resultList.Add(downset);
            }

            load_obj.Invoke(this, new ModelEventArgs(resultList));
        }

        private void getDownSet(ref string downset, int i, int ele)
        {
            if (!downset.Contains(FiniteLattice.preorder.names[FiniteLattice.preorder.predecessors[i][ele]])) downset += "," + FiniteLattice.preorder.names[FiniteLattice.preorder.predecessors[i][ele]];
            if (0 < FiniteLattice.preorder.predecessors[FiniteLattice.preorder.predecessors[i][ele]].Count) getDownSet(ref downset, FiniteLattice.preorder.predecessors[i][ele], 0);
            if (ele + 1 < FiniteLattice.preorder.predecessors[i].Count) getDownSet(ref downset, i, ele + 1);
        }


        private void updateConfigFiles(string name)
        {
            string configSR = System.IO.File.ReadAllText("config_ASR.txt");
            configSR += "\n#" + name + "#";
            System.IO.StreamWriter file = new System.IO.StreamWriter("config_ASR.txt");
            file.WriteLine(configSR);

            file.Close();
        }


        /// <summary>
        /// old saving, before serialize
        /// </summary>
        /// <returns></returns>
        public string save_currentStateofData()
        {
            string t = "";
            for (int i = 0; i < names.Count() - 1; i++)
            {
                updateConfigFiles(names[i] + "-FiniteLattice");
                t += names[i] + "---" + names[i + 1] + ": \n";
                t += preorders[names[i]].predecessors.Count() + " \n";
                // save preorder
                for (int j = 0; j < preorders[names[i]].predecessors.Count(); j++)
                {
                    t += preorders[names[i]].names[j] + ">";
                    // list of downset:
                    for (int d = 0; d < preorders[names[i]].predecessors[j].Count(); d++)
                    {

                        t += preorders[names[i]].names[preorders[names[i]].predecessors[j][d]];
                        if (d < preorders[names[i]].predecessors[j].Count() - 1) t += ",";
                    }
                    if (j != preorders[names[i]].predecessors.Count() - 1 && preorders[names[i]].predecessors[j].Count() > 0) t += ";";
                    if (preorders[names[i]].predecessors[j].Count() == 0) t += ";";
                }
                t += "\n";
                i++;

            }

            return t;
        }
    }
}
