using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Globalization;
using System.Threading;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.IO;


namespace Analysis_Part
{
    static class Program
    {


        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung: \Analysis_Part\Analysis_Part\Program.cs
        /// </summary>
        [STAThread]
        static void Main()
        {
            // english language for dialog boxes
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("en-US");
            // for "2.5" = 2.5 in Double
            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");

           
            Dictionary<String, IModel> models = new Dictionary<String, IModel>();
            Form2 view = new Form2();
            
            // prepare model information
            models.Add("FiniteLattice", new DataManager<FiniteLattice>());
            models.Add("LatticeBasedOnBDD", new DataManager<LatticeBasedOnBDD>());
            PreorderModel preorders = new PreorderModel();
            models.Add("PreorderModel", preorders);
            BDDModel bdds = new BDDModel();
            models.Add("BDDModel", bdds);


            // this files should always be consistent with saved info. 
            // this would not be the case, if you delete or add using visual-studio:
            // in this case a manual update must be done! 
            updateSrList( ref  models, "config_main.txt");
            updateSrList(ref  models, "config_ASR.txt");
            updateSrList(ref  models, "config_DP.txt");
            updateSrList(ref  models, "config_field.txt");
            updateSrList(ref  models, "config_stringConstr.txt");

            IFormatter formatter = new BinaryFormatter();

            try
            {
                Stream fromstream = new FileStream("autosave" + ".bin",
                              FileMode.Open,
                              FileAccess.Read,
                              FileShare.Read);
                try
                {
                    Dictionary<String, IModel> models_copy = new Dictionary<String, IModel>();
                    // models_copy allways contains the newest state for preorder-models
                    models_copy = models;
                 
                    // avoid conflicts with deleted semirings
                    models = (Dictionary<string, IModel>)formatter.Deserialize(fromstream);
          
                    if (models.Count > 0)
                    {
                        PreorderModel.setInstance((PreorderModel)models["PreorderModel"]);
                        BDDModel.setInstance((BDDModel)models["BDDModel"]);

                        if (models_copy["PreorderModel"].getNames(-1).Count > models["PreorderModel"].getNames(-1).Count)
                        {
                            models.Remove("PreorderModel");
                            models.Add("PreorderModel", models_copy["PreorderModel"]);
                            PreorderModel.setInstance((PreorderModel)models["PreorderModel"]);

                        }
                        if (models_copy.Count > models.Count)
                        {
                            //search for inconsistent to config-files:
                            foreach (KeyValuePair<string, IModel> k in models_copy)
                            {
                                if (!models.Keys.Contains(k.Key))
                                {
                                    models.Add(k.Key, k.Value);
                                }
                            }
                        }
                    }
                    else
                    {
                        models = models_copy;
                    }

         
                }
                catch (Exception e1) 
                   // Maybe you try to load deleted semiring -types       
                {
                 
                }

                fromstream.Close();
            }
            catch (FileNotFoundException fnf) { }
       
            IController cnt = new IncController(view,models);
            view.updateAfterFileloaded();
            Application.Run(view);
   
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="models"></param>
        /// <param name="filename"></param>
        public static void updateSrList( ref Dictionary<String, IModel>  models,string filename)
        {

            string[] configSR = System.IO.File.ReadAllLines(filename);
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {
                    if (!models.Keys.Contains(line.Substring(1, line.Length - 2)))
                    {
                        string sr = line.Substring(1, line.Length - 2);
                        if (!sr.Contains("FiniteLattice"))
                        {
                            try
                            {
                                Type generic = typeof(DataManager<>);
                                Type[] typeArgs = { Type.GetType("Analysis_Part." + sr) };
                                Type constructed = generic.MakeGenericType(typeArgs);
                                Object obj = Activator.CreateInstance(constructed, null);
                                models.Add(sr, (IModel)obj);
                            }
                            catch {
                                // update config file
                                // maybe semiring was deleted manuel:
                                string structure = System.IO.File.ReadAllText(filename);
                                int index = structure.LastIndexOf("#" + sr + "#");
                                if (index < 0 )
                                {
                                    structure += "\n" + "#" + sr + "#";
                                    File.WriteAllText(filename, structure);
       
                                }
                            
                            }

                        }

                    }

                }

            }
        }//end configfile method
  
    }



}
