﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    /// <summary>
    /// A base class for semirings providing the functions that can be defined independently of the concrete semiring.
    /// </summary>
    [Serializable]
    public class Semiring:Parser
    {

        
        /// <summary>
        /// Swaps two elements in an array of (generic) Typ T.
        /// </summary>
        /// <typeparam name="T">The type of the array.</typeparam>
        /// <param name="a">The array in which the elements should be swapped.</param>
        /// <param name="i">The index of the first of the elements that should be swapped.</param>
        /// <param name="j">The index of the second of the elements that should be swapped.</param>
        public static void swapArrayIndex<T>(T[] a, int i, int j)
        {
            T x = a[i];
            a[i] = a[j];
            a[j] = x;
        }
        /// <summary>
        /// Swaps two elements in a Semiring-Array
        /// </summary>
        /// <param name="a">The array in which the elements should be swapped.</param>
        /// <param name="i">The index of the first of the elements that should be swapped.</param>
        /// <param name="j">The index of the second of the elements that should be swapped.</param>
        public static void swapArrayIndex(Semiring[] a, int i, int j)
        {
            swapArrayIndex<Semiring>(a, i, j);
        }

        /// <summary>
        /// Swaps two elements in a list of (generic) Typ T.
        /// </summary>
        /// <typeparam name="T">The type of the list.</typeparam>
        /// <param name="l">The list in which the elements should be swapped.</param>
        /// <param name="i">The index of the first of the elements that should be swapped.</param>
        /// <param name="j">The index of the second of the elements that should be swapped.</param>
        public static void swapListIndex<T>(List<T> l, int i, int j)
        {
            T x = l[i];
            l[i] = l[j];
            l[j] = x;
        }
        /// <summary>
        /// Swaps two elements in a Semiring-list.
        /// </summary>
        /// <param name="l">The list in which the elements should be swapped.</param>
        /// <param name="i">The index of the first of the elements that should be swapped.</param>
        /// <param name="j">The index of the second of the elements that should be swapped.</param>
        public static void swapListIndex(List<Semiring> l, int i, int j)
        {
            swapListIndex<Semiring>(l, i, j);
        }
        /// <summary>
        /// In a list of arrays of (generic) type T, swap in each array the ith and the jth element. It is assumed and not checked, that in each array i and j are valid indexes.
        /// </summary>
        /// <typeparam name="T">The type of the arrays in the list.</typeparam>
        /// <param name="l">The list of arrays where the swapping should take place.</param>
        /// <param name="i">The index of the first of the elements that should be swapped. Make sure that this index is valid in all arrays of l!</param>
        /// <param name="j">The index of the second of the elements that should be swapped. Make sure that this index is valid in all arrays of l!</param>
        public static void swapArrayIndexInList<T>(List<T[]> l, int i, int j)
        {
            foreach (T[] a in l)
            {
                swapArrayIndex<T>(a, i, j);
            }
        }
        /// <summary>
        /// In a list of arrays of type Semiring, swap in each array the ith and the jth element. It is assumed and not checked, that in each array i and j are valid indexes.
        /// </summary>
        /// <param name="l">The list of arrays where the swapping should take place.</param>
        /// <param name="i">The index of the first of the elements that should be swapped. Make sure that this index is valid in all arrays of l!</param>
        /// <param name="j">The index of the second of the elements that should be swapped. Make sure that this index is valid in all arrays of l!</param>
        public static void swapArrayIndexInList(List<Semiring[]> l, int i, int j)
        {
            swapArrayIndexInList<Semiring>(l, i, j);
        }
        /// <summary>
        /// Computes the minimum of two integers.
        /// </summary>
        /// <param name="i">The first integer to be compared.</param>
        /// <param name="j">The second integer to be compared.</param>
        /// <returns>The minimum of i and j.</returns>
        public static int min(int i, int j)
        {
            if (i < j) return i; else return j;
        }

        /// <summary>
        /// Interprets a list of (generic) T-valued arrays as a matrix and transposes them.
        /// </summary>
        /// <typeparam name="T">The type of the array.</typeparam>
        /// <param name="set">The list of arrays to be transposed. It is assumed that each array in list has the same length.</param>
        /// <returns>A list of T-arrays where the ith element in the jth array is the same as the jth element in the ith array in set.</returns>
        protected static List<T[]> Transpose<T>(List<T[]> set)
        {
            List<T[]> result = new List<T[]>();
            if (set.Count() == 0) return result;
            for (int i = 0; i < set[0].Length; ++i)
            {
                T[] row = new T[set.Count()];
                for (int j = 0; j < set.Count(); ++j) row[j] = set[j][i];
                result.Add(row);
            }
            return result;
        }
        

    }
}
