﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    [Serializable]
    class TropicalSemiring
    {
        public float value;
        public TropicalSemiring(float value) { this.value = value; }
        public TropicalSemiring(TropicalSemiring value) { this.value = value.value; }
        public TropicalSemiring(String value) { float x; if (Single.TryParse(value, out x)) this.value = x; else this.value = System.Single.MaxValue; }

        public static TropicalSemiring Zero()
        {
            return new TropicalSemiring(System.Single.MaxValue);
        }
        public static TropicalSemiring One()
        {
            return new TropicalSemiring(0);
        }
        public static TropicalSemiring operator +(TropicalSemiring x, TropicalSemiring y)
        {
            if (x.value < y.value) return x;
            return y;
        }
        public static TropicalSemiring operator *(TropicalSemiring x, TropicalSemiring y)
        {
            double z = x.value + y.value;
            if (z > System.Single.MaxValue) return One();
            return new TropicalSemiring(x.value + y.value);
        }
        public static TropicalSemiring operator /(TropicalSemiring x, TropicalSemiring y)
        {
            if (x.value - y.value < 0) return new TropicalSemiring(0);
            return new TropicalSemiring(x.value - y.value);
        }
        public static bool operator >=(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value <= c2.value;
        }
        public static bool operator >(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value < c2.value;
        }
        public static bool operator <=(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value >= c2.value;
        }
        public static bool operator <(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value > c2.value;
        }

        public static bool operator ==(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value == c2.value;
        }
        public static bool operator !=(TropicalSemiring c1, TropicalSemiring c2)
        {
            return c1.value != c2.value;
        }
        public static TropicalSemiring infimum(TropicalSemiring c1, TropicalSemiring c2)
        {
            return new TropicalSemiring(c1 >= c2 ? c2 : c1);
        }

        public TropicalSemiring infimum(TropicalSemiring c2)
        {
            return new TropicalSemiring(this >= c2 ? c2 : this);
        }
        public override string ToString()
        {
            return value.ToString();
        }


        /// <summary>
        /// Check whether a linear combination exists, using the the algorithm from Zhou / Li: Largest Solution of Linear Equation over the complete heyting algebra.
        /// </summary>
        /// <param name="set">The set of rows the vector shall be combined from.</param>
        /// <param name="vec">The vector (represented as an array) that may be combined linearly.</param>
        /// <returns>A LinearCombination object that indicates in the field CombinationExists, whether a linear combination was found, and, provided a combination was found, in its field Multiplicands the multiplicands that form the linear combination.</returns>
        public static LinearCombination<TropicalSemiring> LinearCombination(List<TropicalSemiring[]> set, TropicalSemiring[] vec)
        {
            LinearCombination<TropicalSemiring> lc;
            lc.CombinationExists = true;
            lc.Multiplicands = new TropicalSemiring[set.Count()];
            for (int j = 0; j < lc.Multiplicands.Length; ++j)
            {
                lc.Multiplicands[j] = One();
                for (int i = 0; i < vec.Length; ++i)
                {
                    //i\in M_j(A,b)?
                    if (!(set[j][i] <= vec[i]))
                        lc.Multiplicands[j] = infimum(lc.Multiplicands[j], vec[i] / set[j][i]);
                }
            }
            //Check, if linear combination was found:
            for (int i = 0; i < vec.Length; ++i)
            {
                if (vec[i] != Matrix<TropicalSemiring>.multiplyVectors(Matrix<TropicalSemiring>.getColumn(set, i), lc.Multiplicands)) lc.CombinationExists = false;
            }
            return lc;
        }
        /// <summary>
        /// Checks whether two arrays of type TropicalSemiringInt are equal
        /// </summary>
        /// <param name="a">The first array to be compared.</param>
        /// <param name="b">The second array to be compared.</param>
        /// <returns>true if both arrays are the same length and in each position the have the same l-monoid element.</returns>
        private static bool equals(TropicalSemiring[] a, TropicalSemiring[] b)
        {
            if (b.Length != a.Length) return false;
            for (int i = 0; i < a.Length; ++i) if (a[i] != b[i]) return false;
            return true;
        }

        public static string inputSchema()
        {
            return "float next next...\nPress Enter for next row";
        }

    }
}
