﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    [Serializable]
    class TropicalSemiringInt
    {
        public uint value;
        public TropicalSemiringInt() { value = System.UInt32.MaxValue; }
        public TropicalSemiringInt(String value) {  uint x; if (UInt32.TryParse(value, out x)) this.value = x; else this.value = System.UInt32.MaxValue; }
        public TropicalSemiringInt(uint value) { this.value = value; }
        public TropicalSemiringInt(TropicalSemiringInt value) { this.value = value.value; }
        public static TropicalSemiringInt Zero()
        {
            return new TropicalSemiringInt(System.UInt32.MaxValue);
        }
        public static TropicalSemiringInt One()
        {
            return new TropicalSemiringInt(0);
        }
        public static TropicalSemiringInt operator +(TropicalSemiringInt x, TropicalSemiringInt y)
        {
            if (x.value < y.value) return x;
            return y;
        }
        public static TropicalSemiringInt operator *(TropicalSemiringInt x, TropicalSemiringInt y)
        {
            uint z = x.value+y.value;
            if ((z < x.value) || (z < y.value)) return Zero();
            return new TropicalSemiringInt(z);
        }
        public static TropicalSemiringInt operator /(TropicalSemiringInt x, TropicalSemiringInt y)
        {
            if (y.value > x.value) return new TropicalSemiringInt(0);
            return new TropicalSemiringInt(x.value - y.value);
        }
        public static bool operator >=(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value <= c2.value;
        }
        public static bool operator >(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value < c2.value;
        }
        public static bool operator <=(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value >= c2.value;
        }
        public static bool operator <(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value > c2.value;
        }

        public static bool operator ==(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value == c2.value;
        }
        public static bool operator !=(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return c1.value != c2.value;
        }
        public static TropicalSemiringInt infimum(TropicalSemiringInt c1, TropicalSemiringInt c2)
        {
            return new TropicalSemiringInt(c1 >= c2 ? c2 : c1);
        }

        public TropicalSemiringInt infimum(TropicalSemiringInt c2)
        {
            return new TropicalSemiringInt(this >= c2 ? c2 : this);
        }
        public override string ToString()
        {
            if (value == System.UInt32.MaxValue) return ("infty");
            return value.ToString();
        }

        public static bool allowedStrings(string s)
        {
            return true;
        }
        public static string inputSchema()
        {
            return "uint for finite value, \"infty\" for infinity...\nPress Enter for next row";
        }


        /// <summary>
        /// Check whether a linear combination exists, using the the algorithm from Zhou / Li: Largest Solution of Linear Equation over the complete heyting algebra.
        /// </summary>
        /// <param name="set">The set of rows the vector shall be combined from.</param>
        /// <param name="vec">The vector (represented as an array) that may be combined linearly.</param>
        /// <returns>A LinearCombination object that indicates in the field CombinationExists, whether a linear combination was found, and, provided a combination was found, in its field Multiplicands the multiplicands that form the linear combination.</returns>
        public static LinearCombination<TropicalSemiringInt> LinearCombination(List<TropicalSemiringInt[]> set, TropicalSemiringInt[] vec)
        {
            LinearCombination<TropicalSemiringInt> lc;
            lc.CombinationExists = true;
            lc.Multiplicands = new TropicalSemiringInt[set.Count()];
            for (int j = 0; j < lc.Multiplicands.Length; ++j)
            {
                lc.Multiplicands[j] = One();
                for (int i = 0; i < vec.Length; ++i)
                {
                    //i\in M_j(A,b)?
                    if (!(set[j][i] <= vec[i]))
                        lc.Multiplicands[j] = infimum(lc.Multiplicands[j], vec[i] / set[j][i]);
                }
            }
            //Check, if linear combination was found:
            for (int i = 0; i < vec.Length; ++i)
            {
                if (vec[i] != Matrix<TropicalSemiringInt>.multiplyVectors(Matrix<TropicalSemiringInt>.getColumn(set, i), lc.Multiplicands)) lc.CombinationExists = false;
            }
            return lc;
        }
        /// <summary>
        /// Checks whether two arrays of type TropicalSemiringInt are equal
        /// </summary>
        /// <param name="a">The first array to be compared.</param>
        /// <param name="b">The second array to be compared.</param>
        /// <returns>true if both arrays are the same length and in each position the have the same l-monoid element.</returns>
        private static bool equals(TropicalSemiringInt[] a, TropicalSemiringInt[] b)
        {
            if (b.Length != a.Length) return false;
            for (int i = 0; i < a.Length; ++i) if (a[i] != b[i]) return false;
            return true;
        }

    }


}
