﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Analysis_Part
{
    public class RuleSystem<T> where T : class
    {
        private List<Rule<T>> rules;
        public RuleSystem()
        {
            rules = new List<Rule<T>>();
        }
        public void Add(Rule<T> rule)
        {
            rules.Add(rule);
        }
        public Rule<T> Get(int i)
        {
            return rules[i];
        }
        public void Delete(int i)
        {
            rules.RemoveAt(i);
        }
        public int n()
        {
            return rules.Count();
        }

        public bool Contains(Rule<T> r)
        {
            for (int i = 0; i < rules.Count; ++i) if (rules[i] == r) return true;
            return false;
        }
        /// <summary>
        /// Yields a RuleSystem that contains all paris of (different) unit vectors. Used as initialisation for the partition refinement algorithm.
        /// </summary>
        /// <param name="n">Number of states in the system under consideration (and therefore dimension of Rules in the RuleSystem)</param>
        /// <returns>A RuleSystem that contains all paris of (different) unit vectors.</returns>
        public static RuleSystem<T> SimulationInitialisation(int n)//n: Dimension
        {
            RuleSystem<T> result = new RuleSystem<T>();
            for (int i = 0; i < n; ++i)
                for (int j = 0; j < n; ++j)
                    result.Add(new Rule<T>(new Vectoroflmelements<T>(n,i,Matrix<T>.One()), new Vectoroflmelements<T>(n,j,Matrix<T>.One())));
            return result;
        }

        public static RuleSystem<T> Random(int n, int m)//n: Dimension, m: Anzahl der Regeln
        {
            RuleSystem<T> result = new RuleSystem<T>();
            for (int i = 0; i < m; ++i)
            {
                result.Add(Rule<T>.Random(n));
            }
            return result;
        }

        private T mini(Rule<T> rule1, Rule<T> rule2, T mult)
        {
            T result = (dynamic)typeof(T).GetMethod("One").Invoke(null, null);
            for (int i = 0; i < rule1.n(); ++i)
                if ((dynamic)(((dynamic)rule1.Right.get(i) * mult) / rule2.Left.get(i)) < result) result = ((dynamic)rule1.Right.get(i) * mult) / rule2.Left.get(i);
            return result;
        }

        private T maxi(Rule<T> rule1, Rule<T> rule2, int k)
        {
            T result = (dynamic)typeof(T).GetMethod("Zero").Invoke(null, null);
            for (int j = 0; j < rule1.n(); ++j)
                if (((dynamic)rule1.Right.get(k) * rule2.Left.get(j) / ((dynamic)rule1.Right.get(j) * rule2.Right.get(k)) > result) && ((dynamic)rule1.Right.get(k) * rule2.Left.get(j) / ((dynamic)rule1.Right.get(j) * rule2.Right.get(k)) != (dynamic)typeof(T).GetMethod("One").Invoke(null, null))) result = (dynamic)rule1.Right.get(k) * rule2.Left.get(j) / ((dynamic)rule1.Right.get(j) * rule2.Right.get(k));
            return result;
        }

        public void printSuspectedMultiplicands()
        {
            for (int i = 0; i < rules.Count(); ++i)
                for (int j = 0; j < rules.Count(); ++j)
                    for (int k = 0; k < rules[0].n(); ++k)
                    {
                        for (int jott = 0; jott < rules[0].n(); ++jott)
                            Console.WriteLine(((dynamic)rules[i].Right.get(k) * rules[j].Left.get(jott) / ((dynamic)rules[i].Right.get(jott) * rules[j].Right.get(k))).ToString());
                        Console.WriteLine("Maxi:" + maxi(rules[i], rules[j], k));

                        //Console.WriteLine((((dynamic)rules[i].Right.get(k) / mini(rules[i], rules[j], rules[j].Right.get(k))).ToString()));
                        //Console.WriteLine(rules[i].Right.get(k).ToString() + "/" + mini(rules[i], rules[j]).ToString() + "*" + rules[j].Right.get(k).ToString());
                    }
        }

        public Vectoroflmelements<T> ApplyAtOnceOneRound(Vectoroflmelements<T> v)
        {
            Vectoroflmelements<T> result = v + v; //quick & dirty copying, using the fact that a+a=a
            for (int i = 0; i < n(); ++i)
            {
                result += rules[i].apply(v);
            }
            return result;
        }
        public Vectoroflmelements<T> ApplyOneAfterAnotherOneRound(Vectoroflmelements<T> v)
        {
            Vectoroflmelements<T> result = v + v; //quick & dirty copying, using the fact that a+a=a
            for (int i = 0; i < n(); ++i)
            {
                result += rules[i].apply(result);
                //if (i < n() - 1) Console.Write("; ");
            }
            //Console.WriteLine("test");
            return result;
        }
        public Vectoroflmelements<T> ApplyAtOnce(Vectoroflmelements<T> v)
        {
            //Console.WriteLine("Applying " + ToString() + " to vector " + v.ToString());
            int rounds = 0;
            Vectoroflmelements<T> result = v + v; //quick & dirty copying, using the fact that a+a=a
            Vectoroflmelements<T> previousresult;
            do
            {
                previousresult = result + result;
                for (int i = 0; i < n(); ++i)
                {
                    result += rules[i].apply(previousresult);
                    //if (i < n() - 1) Console.Write("; ");
                }
                //Console.WriteLine("");
                Vectoroflmelements<T> stepsize = new Vectoroflmelements<T>();
                //for (int i = 0; i < result.n(); ++i) stepsize.Add((dynamic)previousresult.get(i) / (dynamic)result.get(i));
                //Console.WriteLine("Intermediate result after " + (++rounds).ToString() + " rounds: " + result.ToString());
                //Console.WriteLine("Corresponding multiplicands: " + stepsize.ToString());
                //++rounds;
            } while (result != previousresult);
            //Console.WriteLine("Rewriting result after " + (rounds).ToString() + " rounds: " + result.ToString());
            return result;
        }


        public int ApplyAtOnceCounting(Vectoroflmelements<T> v)
        {
            int rounds = 0;
            Vectoroflmelements<T> result = v + v; //quick & dirty copying, using the fact that a+a=a
            Vectoroflmelements<T> previousresult;
            do
            {
                previousresult = result + result;
                for (int i = 0; i < n(); ++i)
                {
                    result += rules[i].apply(previousresult);
                }
                Vectoroflmelements<T> stepsize = new Vectoroflmelements<T>();
                ++rounds;
            } while (result != previousresult);
            return rounds;
        }

        public Vectoroflmelements<T> ApplyOneAfterAnother(Vectoroflmelements<T> v)
        {
            //Console.WriteLine("Applying " + ToString() + " to vector " + v.ToString());
            //int rounds = 0;
            Vectoroflmelements<T> result = v + v; //quick & dirty copying, using the fact that a+a=a
            Vectoroflmelements<T> previousresult;
            do
            {
                previousresult = result + result;
                //Console.Write("Applying rules at the following rates: ");
                for (int i = 0; i < n(); ++i)
                {
                    result += rules[i].apply(result);
                    //if (i < n() - 1) Console.Write("; ");
                }
                //Console.WriteLine("");
                Vectoroflmelements<T> stepsize = new Vectoroflmelements<T>();
                for (int i = 0; i < result.n(); ++i) stepsize.Add((dynamic)previousresult.get(i) / (dynamic)result.get(i));
                //Console.WriteLine("Intermediate result after " + (++rounds).ToString() + " rounds: " + result.ToString());
                //Console.WriteLine("Corresponding multiplicands: " + stepsize.ToString());
            } while (result != previousresult); //Checking if rule is applicable is almost as expensive as applying rule, so just check once per round if any rule was applicable.
            //Console.WriteLine("Final rewriting result after " + (++rounds).ToString() + " rounds: " + result.ToString());
            return result;
        }
        public override string ToString()
        {
            if (n() == 0) return "[]";
            string result = "[";
            for (int i = 0; i < n() - 1; ++i) result += rules[i].ToString() + ", ";
            return result + rules[n() - 1].ToString() + "]";
        }
    }
    public class Rule<T> where T : class
    {
        public T lastfactor;
        public Rule()
        {
            lastfactor = (dynamic)typeof(T).GetMethod("Zero").Invoke(null, null);
        }

        public Rule(Vectoroflmelements<T> left, Vectoroflmelements<T> right)
        {
            this.Left = left;
            this.Right = right;
        }

        public static bool operator ==(Rule<T> r1, Rule<T> r2)
        {
            if (r1.n() != r2.n()) return false;
            for (int i = 0; i < r1.n(); ++i ) if (((dynamic)r1.Left.get(i) != r2.Left.get(i)) || ((dynamic)r1.Right.get(i) != r2.Right.get(i))) return false;
            return true;
        }

        public static bool operator !=(Rule<T> r1, Rule<T> r2) { return !(r1 == r2); }

        public int n()
        {
            return Left.n();
        }

        public static Rule<T> Random(int n)
        {
            Rule<T> result = new Rule<T>();
            result.Left = Vectoroflmelements<T>.Random(n);
            result.Right = Vectoroflmelements<T>.Random(n) + result.Left;
            return result;
        }

        public Vectoroflmelements<T> Left { get; set; }
        public Vectoroflmelements<T> Right { get; set; }
        public Vectoroflmelements<T> apply(Vectoroflmelements<T> v)
        {
            T m = v / Left;
            //Console.Write((dynamic) m.ToString());

            lastfactor = m;
            Vectoroflmelements<T> result = v + m * Right;
            //if ((result > v)&&(m==(dynamic)typeof(T).GetMethod("One").Invoke(null, null))) Console.WriteLine("Rule fully applied!");
            if (result > v) return result; else return v;
        }
        public override string ToString()
        {
            return Left.ToString() + "->" + Right.ToString();
        }
    };
    public class Vectoroflmelements<T> where T : class
    {
        public List<T> elements;
        public Vectoroflmelements()
        {
            elements = new List<T>();
        }
        public Vectoroflmelements(int n, int i, T value)
        {
            elements = new List<T>();
            for (int j = 0; j < i; ++j) elements.Add(Matrix<T>.Zero());
            elements.Add(value);
            for (int j = i + 1; j < n; ++j) elements.Add(Matrix<T>.Zero());
        }
        public Vectoroflmelements(T[] values)
        {
            elements = new List<T>(values);
        }
        public Vectoroflmelements(List<T> values)
        {
            elements = values;
        }
        public T get(int i)
        {
            return elements[i];
        }
        public static Vectoroflmelements<T> Random(int n)
        {
            Vectoroflmelements<T> result = new Vectoroflmelements<T>();
            for (int i = 0; i < n; ++i)
                result.Add((dynamic)typeof(T).GetMethod("Random").Invoke(null, null));
            return result;
        }
        public int Length()
        {
            return elements.Count();
        }
        public int Count()
        {
            return elements.Count();
        }
        public int n()
        {
            return elements.Count();
        }
        public void Add(T x)
        {
            elements.Add(x);
        }
        public static Vectoroflmelements<T> operator +(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            Vectoroflmelements<T> result = new Vectoroflmelements<T>();
            for (int i = 0; i < lmv1.n(); ++i)
                result.Add((dynamic)lmv1.get(i) + (dynamic)lmv2.get(i));
            return result;
        }
        public static T operator *(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            T result = Matrix<T>.Zero();
            for (int i = 0; i < lmv1.n(); ++i)
                result = (dynamic)result+((dynamic)lmv1.get(i) * (dynamic)lmv2.get(i));
            return result;
        }
        public static T operator *(Vectoroflmelements<T> lmv1, T[] lmv2)
        {
            T result = Matrix<T>.Zero();
            for (int i = 0; i < lmv1.n(); ++i)
                result = (dynamic)result + ((dynamic)lmv1.get(i) * (dynamic)lmv2[i]);
            return result;
        }
        public static T operator *(T[] lmv2, Vectoroflmelements<T> lmv1)
        {
            T result = Matrix<T>.Zero();
            for (int i = 0; i < lmv1.n(); ++i)
                result = (dynamic)result + ((dynamic)lmv1.get(i) * (dynamic)lmv2[i]);
            return result;
        }
        public static Vectoroflmelements<T> operator *(Vectoroflmelements<T> lmv, T m)
        {
            Vectoroflmelements<T> result = new Vectoroflmelements<T>();
            for (int i = 0; i < lmv.n(); ++i)
                result.Add((dynamic)lmv.get(i) * (dynamic)m);
            return result;
        }
        public static Vectoroflmelements<T> operator *(T m, Vectoroflmelements<T> lmv)
        {
            return lmv * m;
        }
        public static T operator /(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            T result = (dynamic)typeof(T).GetMethod("One").Invoke(null, null);
            if (lmv1.Length() != lmv2.Length()) Console.WriteLine("lmv1: " + lmv1.Length() + ", lmv2: " + lmv2.Length());
            for (int i = 0; i < lmv1.n(); ++i)
                if ((dynamic)((dynamic)lmv1.get(i) / (dynamic)lmv2.get(i)) < (dynamic)result) result = (dynamic)lmv1.get(i) / (dynamic)lmv2.get(i);
            return result;
        }
        public static bool operator >(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            bool result = false;
            for (int i = 0; i < lmv1.n(); ++i)
                if (!((dynamic)lmv1.get(i) >= (dynamic)lmv2.get(i))) return false;
                else
                    if (((dynamic)lmv1.get(i) > (dynamic)lmv2.get(i))) result = true;
            return result;
        }
        public static bool operator <(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            return lmv2 > lmv1;
        }
        public static bool operator <=(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            for (int i = 0; i < lmv1.n(); ++i)
                if (!((dynamic)lmv1.get(i) <= (dynamic)lmv2.get(i))) return false;
            return true;
        }
        public static bool operator >=(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            for (int i = 0; i < lmv1.n(); ++i)
                if (!((dynamic)lmv1.get(i) >= (dynamic)lmv2.get(i))) return false;
            return true;
        }

        public static bool operator !=(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            if (lmv1.n() != lmv2.n()) return true;
            for (int i = 0; i < lmv1.n(); ++i)
                if ((dynamic)lmv1.get(i) != lmv2.get(i)) return true;
            return false;
        }
        public static bool operator ==(Vectoroflmelements<T> lmv1, Vectoroflmelements<T> lmv2)
        {
            if (lmv1.n() != lmv2.n()) return false;
            for (int i = 0; i < lmv1.n(); ++i)
                if ((dynamic)lmv1.get(i) != lmv2.get(i)) return false;
            return true;
        }
        public override string ToString()
        {
            string result = "(";
            for (int i = 0; i < n(); ++i) result += (dynamic)get(i).ToString() + ((i < n() - 1) ? ", " : ")");
            return result;
        }
    }
}
