﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.CodeDom;

namespace Generator_2._0
{
    /// <summary>
    /// GUI-inpumask for arbitrary semirings
    /// </summary>
    class ArbitrarySRPanel : Panel, IView2
    {
        private Panel panelMain;
        private string parameterstring;

        // handle input if the user and call teh model-save methods with the corresponding classname
        protected String classname;
        protected RichTextBox sourcebox;
        protected System.Windows.Forms.Control active;
        protected TextBox functionInfo;


        // individual fields only used by the parent class itself:
        IController2 controller;
        public event ViewHandler2<IView2> saveData2;
        private bool modul;// between module and semiring
        private bool sr;// between module and semiring
        private bool field; // if sr is a field
        private bool euclid;// is sr is an euc. ring

        // View will set the associated controller, this is how view is linked to the controller.
        public void setController(IController2 cont)
        {
            controller = cont;
        }

        public ArbitrarySRPanel()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Constructor used by all child panels
        /// </summary>
        /// <param name="p"></param>
        /// <param name="box"></param>
        public ArbitrarySRPanel(string classname, RichTextBox box, TextBox info)
        {
            this.classname = classname;
            this.sourcebox = box;
            this.sourcebox.Enabled = false;
            this.functionInfo = info;
        }

        public ArbitrarySRPanel(List<string> attributes, Dictionary<string, string> methods, Dictionary<string, string> optional, Dictionary<string, List<string>> optionalinfo, String classname, RichTextBox box, bool m, bool sr, bool field, bool euclid, TextBox info)
        {
            this.attributes = attributes;
            this.methods = methods;
            this.optional = optional;
            this.optionalinfo = optionalinfo;
            this.classname = classname;
            this.sourcebox = box;
            InitializeComponent();
            this.modul = m;
            this.sr = sr;
            this.field = field;
            this.euclid = euclid;
            this.parameterstring = "";
            this.functionInfo = info;
            updateView();

            // init comboBox
            updateAttributes();
        }


        /// <summary>
        /// init the basetypes selection
        /// </summary>
        /// <param name="filename"></param>
        protected virtual void initSelections(string filename)
        {

            string[] configSR = System.IO.File.ReadAllLines(filename);
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {
                    if (!this.comboBox1.Items.Contains(line.Substring(1, line.Length - 2)))
                    {
                        this.comboBox1.Items.Add(line.Substring(1, line.Length - 2));
                        //string-constructor generation for FiniteLattice [] not tested
                        if (!line.Contains("FiniteLattice")) this.comboBox1.Items.Add(line.Substring(1, line.Length - 2) + "[]"); 
                    }

                }

            }
        }
        /// <summary>
        /// to update the basetype selection
        /// </summary>
        public virtual void updateAttributes()
        {
            this.comboBox1.Items.Clear();
            initSelections("config_modules.txt");
            initSelections("config_ASR.txt");
            initSelections("config_field.txt");
            initSelections("config_DP.txt");
            initSelections("config_euclid.txt");
            initSelections("config_main.txt");
        }

        /// <summary>
        /// GUI components, parts are generated by the form-designer of visual-studio
        /// parts for event-handling are adpated
        /// </summary>
        private void InitializeComponent()
        {
            this.panelMain = new System.Windows.Forms.Panel();
            this.panelMain.SuspendLayout();
            this.SuspendLayout();


            this.label1 = new System.Windows.Forms.Label();
            this.label2 = new System.Windows.Forms.Label();
            this.comboBox1 = new System.Windows.Forms.ComboBox();
            this.listView1 = new System.Windows.Forms.ListView();
            this.selected = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));

            this.checkedListBox1 = new System.Windows.Forms.CheckedListBox();
            this.constructorSelect = new System.Windows.Forms.ComboBox();
            this.constructor = new System.Windows.Forms.Button();
            this.constructorInput = new System.Windows.Forms.ComboBox();
            this.label3 = new System.Windows.Forms.Label();
            this.label4 = new System.Windows.Forms.Label();
            this.plus = new System.Windows.Forms.ComboBox();
            this.mul = new System.Windows.Forms.ComboBox();
            this.label5 = new System.Windows.Forms.Label();
            this.label6 = new System.Windows.Forms.Label();
            this.Zerolabel = new System.Windows.Forms.Label();
            this.zero = new System.Windows.Forms.ComboBox();
            this.label7 = new System.Windows.Forms.Label();
            this.one = new System.Windows.Forms.ComboBox();
            this.panel2 = new System.Windows.Forms.Panel();
            this.equal = new System.Windows.Forms.ComboBox();

            this.label8 = new System.Windows.Forms.Label();
            this.label9 = new System.Windows.Forms.Label();
            this.label10 = new System.Windows.Forms.Label();
            this.minus = new System.Windows.Forms.ComboBox();

            this.label11 = new System.Windows.Forms.Label();
            this.div = new System.Windows.Forms.ComboBox();

            this.label12 = new System.Windows.Forms.Label();
            this.linCombi = new System.Windows.Forms.ComboBox();

            this.panel3 = new System.Windows.Forms.Panel();
            this.label13 = new System.Windows.Forms.Label();
            this.label14 = new System.Windows.Forms.Label();
            this.label15 = new System.Windows.Forms.Label();
            this.label16 = new System.Windows.Forms.Label();
            this.parameters = new System.Windows.Forms.TextBox();
            this.ret = new System.Windows.Forms.TextBox();
            this.textBox1 = new System.Windows.Forms.TextBox();
            this.addPara = new System.Windows.Forms.Button();
            this.listView2 = new System.Windows.Forms.ListView();

            this.button2 = new System.Windows.Forms.Button();
            this.reset = new System.Windows.Forms.Button();

            this.comboBox2 = new System.Windows.Forms.ComboBox();
            this.label17 = new System.Windows.Forms.Label();

            this.panel2.SuspendLayout();
            this.panel3.SuspendLayout();
            this.SuspendLayout();

            // 
            // label1
            // 
            this.label1.AutoSize = true;
            this.label1.Font = new System.Drawing.Font("Courier New", 12F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label1.Location = new System.Drawing.Point(27, 11);
            this.label1.Name = "label1";
            this.label1.Size = new System.Drawing.Size(478, 18);
            this.label1.TabIndex = 0;
            this.label1.Text = "Attributes, Constructors, Operators and Methods";
            // 
            // label2
            // 
            this.label2.AutoSize = true;
            this.label2.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label2.Location = new System.Drawing.Point(29, 38);
            this.label2.Name = "label2";
            this.label2.Size = new System.Drawing.Size(113, 13);
            this.label2.TabIndex = 1;
            this.label2.Text = "Select basetype(s):";
            // 
            // comboBox1
            // 
            this.comboBox1.BackColor = System.Drawing.Color.LightCoral;
            this.comboBox1.ForeColor = System.Drawing.SystemColors.Info;
            this.comboBox1.FormattingEnabled = true;
            this.comboBox1.Items.AddRange(new object[] {
            "int",
            "uint",
            "double",
            "LatticeZ",
            "LMonoidMaxTimes"});
            this.comboBox1.Location = new System.Drawing.Point(144, 35);
            this.comboBox1.Name = "comboBox1";
            this.comboBox1.Size = new System.Drawing.Size(121, 21);
            this.comboBox1.TabIndex = 2;
            this.comboBox1.DropDownStyle = ComboBoxStyle.DropDownList;
            this.comboBox1.SelectedIndexChanged += new System.EventHandler(this.comboBox1_SelectedIndexChanged);
            comboBox1.MouseWheel += new MouseEventHandler(comboBox1_MouseWheel);
            // 
            // listView1
            // 
            this.name = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.code = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.listView1.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.name});
            this.listView1.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.code});
            this.listView1.Location = new System.Drawing.Point(32, 62);
            this.listView1.Name = "listView1";
            this.listView1.Size = new System.Drawing.Size(233, 130);
            this.listView1.TabIndex = 3;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.GridLines = true;
            this.listView1.HideSelection = false;
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = System.Windows.Forms.View.Details;

            this.name.Tag = "";
            this.name.Text = "Selected basetype(s):";
            this.name.Width = 120;
            this.code.Tag = "";
            this.code.Text = "Name:";
            this.code.Width = 110;

            this.listView1.KeyDown += new System.Windows.Forms.KeyEventHandler(this.listView1_KeyDown);

            this.listView1.ColumnWidthChanging += new System.Windows.Forms.ColumnWidthChangingEventHandler(this.listView1_ColumnWidthChanging);

            // 
            // checkedListBox1
            // 
            this.checkedListBox1.BackColor = System.Drawing.SystemColors.GradientInactiveCaption;
            this.checkedListBox1.ColumnWidth = 70;
            this.checkedListBox1.FormattingEnabled = true;
            this.checkedListBox1.Items.AddRange(new object[] {
            "Attribute",
            "Eucl. Ring",
            "Semiring",
            "Field"});
            this.checkedListBox1.Location = new System.Drawing.Point(270, 35);
            this.checkedListBox1.MultiColumn = true;
            this.checkedListBox1.CheckOnClick = true;
            this.checkedListBox1.Name = "checkedListBox1";
            this.checkedListBox1.Size = new System.Drawing.Size(290, 19);
            this.checkedListBox1.TabIndex = 4;

            this.checkedListBox1.Enabled = true;
            this.checkedListBox1.ItemCheck += new System.Windows.Forms.ItemCheckEventHandler(checkedListBox1_ItemCheck);
            this.checkedListBox1.SetItemChecked(0, true);

            // 
            // constructorSelect
            // 
            this.constructorSelect.FormattingEnabled = true;
            this.constructorSelect.Items.AddRange(new object[] {
            "Constructor(classname x)",
            "Constructor(selected attributes)",
            "Constructor(string str)"});
            this.constructorSelect.Location = new System.Drawing.Point(30, 227);
            this.constructorSelect.Name = "constructorSelect";
            this.constructorSelect.Size = new System.Drawing.Size(147, 21);
            this.constructorSelect.TabIndex = 5;
            constructorSelect.MouseWheel += new MouseEventHandler(comboBox1_MouseWheel);


            // 
            // constructorInput
            // 
            this.constructorInput.FormattingEnabled = true;
            this.constructorInput.Items.AddRange(new object[] {
            Constants.CM,
            "Write code"});
            this.constructorInput.Location = new System.Drawing.Point(30, 255);
            this.constructorInput.Name = "constructorInput";
            this.constructorInput.Size = new System.Drawing.Size(147, 21);
            this.constructorInput.TabIndex = 7;
            this.constructorInput.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);
            // 
            // label3
            // 
            this.label3.AutoSize = true;
            this.label3.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label3.Location = new System.Drawing.Point(30, 208);
            this.label3.Name = "label3";
            this.label3.Size = new System.Drawing.Size(133, 13);
            this.label3.TabIndex = 8;
            this.label3.Text = "Constructor (Default Componentwise) :";
            // 
            // label4
            // 
            this.label4.AutoSize = true;
            this.label4.Location = new System.Drawing.Point(-1, 2);
            this.label4.Name = "label4";
            this.label4.Size = new System.Drawing.Size(86, 13);
            this.label4.TabIndex = 9;
            this.label4.Text = "Necessary parts:";
            // 
            // plus
            // 
            this.plus.FormattingEnabled = true;
            this.plus.Items.AddRange(new object[] {
            Constants.SB,
            Constants.CM});
            this.plus.Location = new System.Drawing.Point(39, 25);
            this.plus.Name = Constants.ADD;
            this.plus.Size = new System.Drawing.Size(107, 21);
            this.plus.TabIndex = 10;
            this.plus.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);


            // 
            // mul
            // 
            this.mul.FormattingEnabled = true;
            this.mul.Items.AddRange(new object[] {
            Constants.SB,
            Constants.CM});
            this.mul.Location = new System.Drawing.Point(39, 54);
            this.mul.Name = Constants.MUL;
            this.mul.Size = new System.Drawing.Size(107, 21);
            this.mul.DropDownWidth = DropDownWidth(mul);
            this.mul.TabIndex = 12;
            this.mul.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);
            // 
            // label5
            // 
            this.label5.AutoSize = true;
            this.label5.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label5.Location = new System.Drawing.Point(6, 25);
            this.label5.Name = "label5";
            this.label5.Size = new System.Drawing.Size(22, 24);
            this.label5.TabIndex = 13;
            this.label5.Text = "+";
            // 
            // label6
            // 
            this.label6.AutoSize = true;
            this.label6.Font = new System.Drawing.Font("DejaVu Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label6.Location = new System.Drawing.Point(8, 57);
            this.label6.Name = "label6";
            this.label6.Size = new System.Drawing.Size(20, 22);
            this.label6.TabIndex = 14;
            this.label6.Text = "*";

            // 
            // Zerolabel
            // 
            this.Zerolabel.AutoSize = true;
            this.Zerolabel.Location = new System.Drawing.Point(4, 114);
            this.Zerolabel.Name = "Zerolabel";
            this.Zerolabel.Size = new System.Drawing.Size(29, 13);
            this.Zerolabel.TabIndex = 17;
            this.Zerolabel.Text = "Zero";
            // 
            // zero
            // 
            this.zero.FormattingEnabled = true;
            this.zero.Items.AddRange(new object[] {
            Constants.SB,
            Constants.CM});
            this.zero.Location = new System.Drawing.Point(39, 111);
            this.zero.Name = Constants.Z;
            this.zero.Size = new System.Drawing.Size(107, 21);
            this.zero.DropDownWidth = DropDownWidth(zero);
            this.zero.TabIndex = 18;
            this.zero.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);


            // 
            // label7
            // 
            this.label7.AutoSize = true;
            this.label7.Location = new System.Drawing.Point(6, 142);
            this.label7.Name = "label7";
            this.label7.Size = new System.Drawing.Size(27, 13);
            this.label7.TabIndex = 20;
            this.label7.Text = "One";
            // 
            // one
            // 
            this.one.FormattingEnabled = true;
            this.one.Items.AddRange(new object[] {
            Constants.SB,
            Constants.CM});
            this.one.Location = new System.Drawing.Point(39, 139);
            this.one.Name = Constants.O;
            this.one.Size = new System.Drawing.Size(107, 21);
            this.one.DropDownWidth = DropDownWidth(one);
            this.one.TabIndex = 21;
            this.one.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);

            // 
            // panel2
            // 
            this.panel2.BackColor = System.Drawing.Color.LightCoral;
            //this.panel2.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panel2.Controls.Add(this.label8);
            this.panel2.Controls.Add(this.equal);
            this.panel2.Controls.Add(this.plus);
            this.panel2.Controls.Add(this.one);
            this.panel2.Controls.Add(this.zero);
            this.panel2.Controls.Add(this.label4);
            this.panel2.Controls.Add(this.label5);
            this.panel2.Controls.Add(this.mul);
            this.panel2.Controls.Add(this.label7);
            this.panel2.Controls.Add(this.label6);
            this.panel2.Controls.Add(this.Zerolabel);
            this.panel2.Location = new System.Drawing.Point(29, 287);
            this.panel2.Name = "panel2";
            this.panel2.Size = new System.Drawing.Size(236, 174);
            this.panel2.TabIndex = 23;
            // 
            // equal
            // 
            this.equal.FormattingEnabled = true;
            this.equal.Items.AddRange(new object[] {
            Constants.SB,
            Constants.CM});
            this.equal.Location = new System.Drawing.Point(39, 83);
            this.equal.Name = Constants.EQUAL;
            this.equal.Size = new System.Drawing.Size(107, 21);
            this.equal.DropDownWidth = DropDownWidth(equal);
            this.equal.TabIndex = 23;
            this.equal.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);

            // 
            // label8
            // 
            this.label8.AutoSize = true;
            this.label8.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label8.Location = new System.Drawing.Point(3, 81);
            this.label8.Name = "label8";
            this.label8.Size = new System.Drawing.Size(34, 24);
            this.label8.TabIndex = 25;
            this.label8.Text = "==";
            // 
            // label9
            // 
            this.label9.AutoSize = true;
            this.label9.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label9.Location = new System.Drawing.Point(295, 80);
            this.label9.Name = "label9";
            this.label9.Size = new System.Drawing.Size(17, 24);
            this.label9.TabIndex = 24;
            this.label9.Text = "-";
            // 
            // label10
            // 
            this.label10.AutoSize = true;
            this.label10.Location = new System.Drawing.Point(292, 64);
            this.label10.Name = "label10";
            this.label10.Size = new System.Drawing.Size(75, 13);
            this.label10.TabIndex = 25;
            this.label10.Text = "Optional parts:";
            // 
            // minus
            // 
            this.minus.FormattingEnabled = true;
            this.minus.Items.AddRange(new object[] {
           Constants.SB,
            Constants.CM});
            this.minus.Location = new System.Drawing.Point(315, 85);
            this.minus.Name = Constants.SUB;
            this.minus.Size = new System.Drawing.Size(104, 21);
            this.minus.DropDownWidth = DropDownWidth(minus);
            this.minus.TabIndex = 26;
            this.minus.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);

            // 
            // label11
            // 
            this.label11.AutoSize = true;
            this.label11.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label11.Location = new System.Drawing.Point(295, 114);
            this.label11.Name = "label11";
            this.label11.Size = new System.Drawing.Size(16, 24);
            this.label11.TabIndex = 28;
            this.label11.Text = "/";
            // 
            // div
            // 
            this.div.FormattingEnabled = true;
            this.div.Items.AddRange(new object[] {
           Constants.SB});
            this.div.Location = new System.Drawing.Point(315, 117);
            this.div.Name = Constants.DIV;
            this.div.Size = new System.Drawing.Size(104, 21);
            this.div.DropDownWidth = DropDownWidth(div);
            this.div.TabIndex = 29;
            this.div.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);
            ToolTip toolTipDiv = new ToolTip();
            toolTipDiv.SetToolTip(this.div, Constants.CM + "not supported!");





            // 
            // comboBox2
            // 
            this.comboBox2.FormattingEnabled = true;
            this.comboBox2.Items.AddRange(new object[] {
           Constants.SB,
            Constants.CM});
            this.comboBox2.Location = new System.Drawing.Point(315, 149);
            this.comboBox2.Name = Constants.GREATER;
            this.comboBox2.Size = new System.Drawing.Size(104, 21);
            this.comboBox2.DropDownWidth = DropDownWidth(comboBox2);
            this.comboBox2.TabIndex = 36;
            this.comboBox2.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);
            // 
            // label17
            // 
            this.label17.AutoSize = true;
            this.label17.Font = new System.Drawing.Font("Microsoft Sans Serif", 14.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label17.Location = new System.Drawing.Point(295, 146);
            this.label17.Name = "label17";
            this.label17.Size = new System.Drawing.Size(22, 24);
            this.label17.TabIndex = 35;
            this.label17.Text = ">";

            // 
            // label12
            // 
            this.label12.AutoSize = true;
            this.label12.Location = new System.Drawing.Point(296, 175);
            this.label12.Name = "label12";
            this.label12.Size = new System.Drawing.Size(135, 13);
            this.label12.TabIndex = 31;
            this.label12.Text = "LinearCombination(set,vec)";
            // 
            // linCombi
            // 
            this.linCombi.FormattingEnabled = true;
            this.linCombi.Items.AddRange(new object[] {
           Constants.SB,
            "Largest-Solution",
            "Gauss-Elimination"});
            this.linCombi.Location = new System.Drawing.Point(299, 195);
            this.linCombi.Name = Constants.LINCOMBI;
            this.linCombi.Size = new System.Drawing.Size(116, 21);
            this.linCombi.DropDownWidth = DropDownWidth(linCombi);
            this.linCombi.TabIndex = 32;
            this.linCombi.SelectedIndexChanged += new System.EventHandler(this.comboBox_SelectedIndexChanged);




            // 
            // panel3
            // 
            this.panel3.BackColor = System.Drawing.SystemColors.ButtonFace;
            this.panel3.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panel3.Controls.Add(this.button2);

            this.panel3.Controls.Add(this.listView2);
            this.panel3.Controls.Add(this.addPara);
            this.panel3.Controls.Add(this.reset);
            this.panel3.Controls.Add(this.textBox1);
            this.panel3.Controls.Add(this.ret);
            this.panel3.Controls.Add(this.parameters);
            this.panel3.Controls.Add(this.label16);
            this.panel3.Controls.Add(this.label15);
            this.panel3.Controls.Add(this.label14);
            this.panel3.Controls.Add(this.label13);

            this.panel3.Location = new System.Drawing.Point(283, 227);
            this.panel3.Name = "panel3";
            this.panel3.Size = new System.Drawing.Size(243, 234);
            this.panel3.TabIndex = 34;
            // 
            // label13
            // 
            this.label13.AutoSize = true;
            this.label13.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.label13.Location = new System.Drawing.Point(9, 9);
            this.label13.Name = "label13";
            this.label13.Size = new System.Drawing.Size(109, 13);
            this.label13.TabIndex = 0;
            this.label13.Text = "Optional methods:";
            // 
            // label14
            // 
            this.label14.AutoSize = true;
            this.label14.Location = new System.Drawing.Point(15, 35);
            this.label14.Name = "label14";
            this.label14.Size = new System.Drawing.Size(66, 13);
            this.label14.TabIndex = 1;
            this.label14.Text = "Methdname:";
            // 
            // label15
            // 
            this.label15.AutoSize = true;
            this.label15.Location = new System.Drawing.Point(15, 63);
            this.label15.Name = "label15";
            this.label15.Size = new System.Drawing.Size(63, 13);
            this.label15.TabIndex = 2;
            this.label15.Text = "Parameters:";
            // 
            // label16
            // 
            this.label16.AutoSize = true;
            this.label16.Location = new System.Drawing.Point(15, 92);
            this.label16.Name = "label16";
            this.label16.Size = new System.Drawing.Size(62, 13);
            this.label16.TabIndex = 3;
            this.label16.Text = "Returntype:";
            // 
            // parameters
            // 
            /*
            this.parameters.FormattingEnabled = true;
            this.parameters.Items.AddRange(new object[] {
            "int",
            "uint",
            "double",
            "string",
            "List<string>",
            "List<double>",
            "List<int>",
            "List<NewClass>",
            "NewClass"});
            */
            this.parameters.Location = new System.Drawing.Point(85, 63);
            //this.parameters.DropDownWidth = DropDownWidth(parameters);//old code
            this.parameters.Name = "parameters";
            this.parameters.Size = new System.Drawing.Size(65, 21);

            this.parameters.TabIndex = 4;
            // 
            // ret
            // 
            /*
            this.ret.FormattingEnabled = true;
            this.ret.Items.AddRange(new object[] {
            "int",
            "uint",
            "double",
            "string",
            "List<string>",
            "List<double>",
            "List<int>",
            "List<NewClass>",
            "NewClass"});
            */
            this.ret.Location = new System.Drawing.Point(85, 91);
            this.ret.Name = "ret";
            this.ret.Size = new System.Drawing.Size(139, 21);
            this.ret.TabIndex = 5;
            // 
            // textBox1
            // 
            this.textBox1.Location = new System.Drawing.Point(88, 35);
            this.textBox1.Name = "textBox1";
            this.textBox1.Size = new System.Drawing.Size(133, 20);
            this.textBox1.TabIndex = 6;
            // 
            // addPara
            // 
            this.addPara.Location = new System.Drawing.Point(158, 62);
            this.addPara.Name = "addPara";
            this.addPara.Size = new System.Drawing.Size(34, 23);
            this.addPara.TabIndex = 7;
            this.addPara.Text = "add";
            this.addPara.UseVisualStyleBackColor = true;
            this.addPara.Click += new System.EventHandler(this.addPara_Click);
            ToolTip infopara = new ToolTip();
            infopara.SetToolTip(addPara, "Adds a parameter for your method, naming starts with a0 to an.");


            this.reset.Location = new System.Drawing.Point(198, 62);
            this.reset.Name = "res";
            this.reset.Size = new System.Drawing.Size(30, 23);
            this.reset.TabIndex = 50;
            this.reset.Text = "re";
            this.reset.UseVisualStyleBackColor = true;
            this.reset.Click += new System.EventHandler(this.reset_Click);
            ToolTip infore = new ToolTip();
            infore.SetToolTip(reset, "Resets all your parameter settings!");
            // 
            // listView2
            // 
            this.methodname = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.paras = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.rettype = ((System.Windows.Forms.ColumnHeader)(new System.Windows.Forms.ColumnHeader()));
            this.listView2.Location = new System.Drawing.Point(18, 118);
            this.listView2.Name = "listView2";
            this.listView2.Size = new System.Drawing.Size(206, 123);
            this.listView2.TabIndex = 8;
            this.listView2.UseCompatibleStateImageBehavior = false;
            this.listView2.UseCompatibleStateImageBehavior = false;
            this.listView2.GridLines = true;
            this.listView2.HideSelection = false;
            this.listView2.View = System.Windows.Forms.View.Details;
            this.listView2.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.methodname});
            this.listView2.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.paras});
            this.listView2.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.rettype});
            this.listView2.KeyDown += new System.Windows.Forms.KeyEventHandler(this.listView2_KeyDown);
            this.listView2.SelectedIndexChanged += new System.EventHandler(optionalView_SelectedIndexChanged);


            this.methodname.Tag = "";
            this.methodname.Text = "Methodname:";
            this.methodname.Width = 120;
            this.paras.Tag = "";
            this.paras.Text = "Parameters:";
            this.paras.Width = 110;
            this.rettype.Tag = "";
            this.rettype.Text = "Returntype:";
            this.rettype.Width = 110;
            // 
            // button1
            // 

            // 
            // button2
            // 
            this.button2.Location = new System.Drawing.Point(165, 6);
            this.button2.Name = "activ";
            this.button2.Size = new System.Drawing.Size(58, 24);
            this.button2.TabIndex = 10;
            this.button2.Text = "activate";
            this.button2.UseVisualStyleBackColor = true;
            this.button2.Click += new System.EventHandler(this.activate);
            ToolTip infoOpt = new ToolTip();
            infoOpt.SetToolTip(button2, "Activate the optional input panel.");

            this.Name = "Form1";
            this.Text = "Form1";
            this.panel2.ResumeLayout(false);
            this.panel2.PerformLayout();
            this.panel3.ResumeLayout(false);
            this.panel3.PerformLayout();
            this.ResumeLayout(false);



            this.panelMain.Controls.Add(this.panel3);
            this.panelMain.Controls.Add(this.panel2);
            this.panelMain.Controls.Add(this.label1);
            this.panelMain.Controls.Add(this.label2);
            this.panelMain.Controls.Add(this.label3);



            this.panelMain.Controls.Add(this.label9);
            this.panelMain.Controls.Add(this.label10);
            this.panelMain.Controls.Add(this.label11);
            this.panelMain.Controls.Add(this.label12);


            this.panelMain.Controls.Add(this.comboBox1);
            this.panelMain.Controls.Add(this.listView1);
            this.panelMain.Controls.Add(this.checkedListBox1);
            this.panelMain.Controls.Add(this.constructorSelect);
            this.panelMain.Controls.Add(this.constructorInput);


            this.panelMain.Controls.Add(this.minus);
            this.panelMain.Controls.Add(this.div);
            this.panelMain.Controls.Add(this.linCombi);
            this.panelMain.Controls.Add(this.comboBox2);
            this.panelMain.Controls.Add(this.label17);




            // panelMain

            this.panelMain.BackColor = System.Drawing.SystemColors.GradientInactiveCaption;
            this.panelMain.Location = new System.Drawing.Point(4, 6);
            this.panelMain.Name = "Free";
            //this.panelMain.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panelMain.TabIndex = 0;
            this.panelMain.Size = new System.Drawing.Size(530, 471);

            this.panelMain.Anchor = (AnchorStyles.Top | AnchorStyles.Right);
            this.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;

            this.Controls.Add(this.panelMain);

        }





        /*above the gui-viusalstudio generated code ends
         * here starts the input-event code stuff: 
         */


        private void optionalView_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (((ListView)sender).SelectedItems.Count > 0)
            {
                // show the code of the first item:
                disableOtherPanels(ret);

                // enable sourceBox
                sourcebox.Enabled = true;
                string box = ((ListView)sender).SelectedItems[0].SubItems[0].Text;
                // update corresponding components:
                textBox1.Text = box;
                parameterstring = optionalinfo[box][0];
                ret.Text = optionalinfo[box][1];
                setParameterInfo(ret);
                if (optional.Keys.Contains(box))
                {
                    sourcebox.Clear();
                    sourcebox.AppendText(optional[box]);
                }

            }
        }




        /// <summary>
        /// remove method from optional methods
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void listView2_KeyDown(object sender, KeyEventArgs e)
        {
            // Remove selected item
            if (e.KeyCode == Keys.Delete)
            {
                List<ListViewItem> remove = new List<ListViewItem>();
                for (int i = 0; i < listView2.SelectedItems.Count; i++)
                {
                    remove.Add(listView2.SelectedItems[i]);
                }
                foreach (ListViewItem entry in remove)
                {
                    listView2.Items.Remove(entry);
                    optional.Remove(entry.Text);
                    optionalinfo.Remove(entry.Text);
                }
                try
                {
                    saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                }
                catch { }
            }
            genMemberNames();
        }
        /// <summary>
        /// the reset button of the optional method panel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void reset_Click(object sender, EventArgs e)
        {
            parameterstring = "";
        }
        /// <summary>
        /// add parameter button for the optional code
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addPara_Click(object sender, EventArgs e)
        {
            if (parameters.Text!="")
            {
                if (parameters.Text == "NewClass") parameterstring += textBox1.Text + ",";
                else parameterstring += parameters.Text + ",";
            }
        }



        /// <summary>
        /// get the implemented code by the user for the selected method/operator/constructor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void activate(object sender, EventArgs e)
        {
            disableOtherPanels((Control)sender);
        }

        private string getoptionalCode(string name)
        {
            if (optional.Keys.Contains(name))
            {
                return (optional[name]);
            }
            return "";
        }


        /// <summary>
        /// disable other gui components of gui-mask
        /// </summary>
        /// <param name="sender"></param>
        public virtual void disableOtherPanels(Control sender)
        {
            foreach (Control x in this.panelMain.Controls)
            {
                if (x is ComboBox && x != sender)
                {
                    ((ComboBox)x).Enabled = false;
                }
            }
            foreach (Control x in this.panel2.Controls)
            {
                if (x is ComboBox && x != sender)
                {
                    ((ComboBox)x).Enabled = false;
                }
            }
            sourcebox.Enabled = true;
            active = ret; // constructor not captured by this

            setParameterInfo(active);

        }

        /// <summary>
        /// save the spezification of what kind the semiring is:
        /// euclidean
        /// just some module
        /// semiring ( means will be usable by Algor.B)
        /// field  ( means will be usable by Algor.B)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkedListBox1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (!String.IsNullOrEmpty(classname))
            {


                if (e.NewValue == CheckState.Checked)
                {
                    if (e.Index == 0) modul = true;
                    if (e.Index == 1) { sr = true; euclid = true; }
                    if (e.Index == 2) sr = true;
                    if (e.Index == 3)
                    {
                        sr = true;
                        field = true;
                    }
                }
                else
                {
                    if (e.Index == 0) modul = false;
                    if (e.Index == 1) euclid = false;
                    if (e.Index == 2) sr = false;
                    if (e.Index == 3) field = false;
                }
                try
                {
                    saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                }
                catch { }

            }


        }

        /// <summary>
        /// Selection between componentwise generation of corresponding operator/method/constructor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected virtual void comboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            if (!String.IsNullOrEmpty(classname))
            {
                if (((ComboBox)sender).Text == Constants.SB)
                {
                    // disbale other spezification boxes
                    disableOtherComponents((ComboBox)sender);

                    // enable sourceBox
                    sourcebox.Enabled = true;

                    string box = ((ComboBox)sender).Name;

                    if (this.constructorSelect.Text == "Constructor(classname x)") box = Constants.C1;
                    if (this.constructorSelect.Text == "Constructor(selected attributes)") box = Constants.C2;
                    if (this.constructorSelect.Text == "Constructor(string str)") box = Constants.C3;
                    // show if there already exists any code
                    if (methods.Keys.Contains(box))
                    {
                        sourcebox.Clear();
                        sourcebox.AppendText(methods[((ComboBox)sender).Name]);
                    }

                }
                else
                {
                    if (((ComboBox)sender).Text == "Largest-Solution")
                    {
                        if (!methods.Keys.Contains(Constants.DIV))
                        {

                            ((ComboBox)sender).SelectedText = "";
                            ((ComboBox)sender).SelectedText = "";
                            MessageBox.Show("You must first define the Division!", "Info", MessageBoxButtons.OK);
                        }
                        else this.writeMethod(((ComboBox)sender).Text, ((ComboBox)sender).Name, (ComboBox)sender);
                    }
                    else this.writeMethod(((ComboBox)sender).Text, ((ComboBox)sender).Name, (ComboBox)sender);


                }
            }
            else
            {
                ((ComboBox)sender).SelectedText = "";
                ((ComboBox)sender).SelectedText = "";

                MessageBox.Show("First define a classname!", "Info", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// In this method the specific implementation of the binary operator/ method named by the
        /// string box is saved corresponding to the class name of the semiring to generate.
        /// </summary>
        /// <param name="text">implementation information</param>
        /// <param name="box">name of method/operator/constructor</param>
        /// <param name="sender">GUI sender component</param>
        public virtual void writeMethod(string text, string box, System.Windows.Forms.Control sender)
        {
            // optional methods
            if (sender.Name == "ret")
            {
                if (classname != "" && attributes.Count > 0)
                {

                    if (textBox1.Text != "" && sourcebox.Text != "")
                    {
                        List<string> info = new List<string>();

                        info.Add(parameterstring);
                        if (!String.IsNullOrWhiteSpace(ret.Text)) info.Add(ret.Text);
                        else info.Add("");


                        if (!optional.Keys.Contains(textBox1.Text))
                        {
                            optional.Add(textBox1.Text, sourcebox.Text);

                            optionalinfo.Add(textBox1.Text, info);
                        }
                        else
                        {
                            // over-writting
                            Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.Interaction.MsgBox("Do you want to override the existing selection/implementation ? ", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                            if (response == Microsoft.VisualBasic.MsgBoxResult.Yes)
                            {
                                optional[textBox1.Text] = sourcebox.Text;
                                optionalinfo[textBox1.Text] = info;
                            }


                        }

                        //update listview:
                        updateOptionalView();
                        try
                        {
                            saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                        }
                        catch { }
                    }
                    else MessageBox.Show("No methodname or source code defined.", "Info", MessageBoxButtons.OK);

                }

                else
                {
                    MessageBox.Show("No classname or no basetypes selected! \nPlease define some classname or basetypes!", "Info", MessageBoxButtons.OK);
                }
            }

            else if (text != "" && attributes.Count > 0)
            {
                //Dictionary<string, string>
                if (methods.Keys.Contains(box))
                {
                    Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.Interaction.MsgBox("Do you want to override the existing selection/implementation ? ", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                    if (response == Microsoft.VisualBasic.MsgBoxResult.Yes) methods[box] = text;
                }
                else methods.Add(box, text);


                try
                {
                    saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                }
                catch { }
            }

            else
            {
                if (attributes.Count == 0) MessageBox.Show("First define the basetypes of the semiring-class!", "Info", MessageBoxButtons.OK);
                else MessageBox.Show("Press Help for information how to use the sourceBox!", "Info", MessageBoxButtons.OK);
            }
        }


        /// <summary>
        /// To disable other components, just allow to specify one operation/operator/constructor at time
        /// </summary>
        /// <param name="sender">the tempory active gui component</param>
        public virtual void disableOtherComponents(ComboBox sender)
        {
            foreach (Control x in this.panelMain.Controls)
            {
                if (x is ComboBox && x != sender)
                {
                    ((ComboBox)x).Enabled = false;
                }
            }
            foreach (Control x in this.panel2.Controls)
            {
                if (x is ComboBox && x != sender)
                {
                    ((ComboBox)x).Enabled = false;
                }
            }
            foreach (Control x in this.panel3.Controls)
            {
                if (x is ComboBox && x != sender)
                {
                    ((ComboBox)x).Enabled = false;
                }

            }
            textBox1.Enabled = false;
            // disable optional input gui components:

            active = sender; // constructor not captured by this

            if (sender.Name == "constructorInput")
            {
                if (constructorSelect.Text == "Constructor(classname x)") active.Name = Constants.C1;
                if (constructorSelect.Text == "Constructor(selected attributes)") active.Name = Constants.C2;
                if (constructorSelect.Text == "Constructor(string str)") active.Name = Constants.C3;
            }
            setParameterInfo(active);

        }


        /// <summary>
        /// set the function/constructor parameter information
        /// active.Name (classname a, classname b)
        /// active.Name ( set, vec)
        /// seperatly for constructors
        /// </summary>
        /// <param name="active"></param>
        protected void setParameterInfo(System.Windows.Forms.Control active)
        {
            switch (active.Name)
            {
                case Constants.C1:
                    this.functionInfo.Text = "Constructor (" + classname + " x)";
                    break;
                case Constants.C2:
                    string paras = "";
                    for (int i = 0; i < attributes.Count(); i++)
                    {
                        paras += " a" + i;
                    }
                    this.functionInfo.Text = "Constructor (" + classname + "[" + paras + ")";
                    break;
                case Constants.C3:
                    this.functionInfo.Text = "Constructor (string str)";
                    break;
                case Constants.LINCOMBI:
                    //        public static LinearCombination<LatticeZ> LinearCombination(List<LatticeZ[]> set, LatticeZ[] vec)
                    this.functionInfo.Text = "LinearCombination<" + classname + "> (List<" + classname + "[]> set, " + classname + "[] vec)";
                    break;
                case "ret":
                    string ret_type = "void";
                    if (!String.IsNullOrWhiteSpace(ret.Text))
                    {
                        ret_type = ret.Text;//TODO testing
                    }
                    // prepare parameterstring-info
                    string[] para = parameterstring.Split(',');
                    string par = "";
                    for (int i = 0; i < para.Length - 1; i++)
                    {
                        par += para[i] + " a" + i;
                        if (i < para.Length - 2) par += ",";
                    }
                    this.functionInfo.Text = ret_type + " " + textBox1.Text + " (" + par + ")";
                    break;
                default:
                    // binary operators:
                    this.functionInfo.Text = "Binaryoperator: " + active.Name + "  (" + classname + " a," + classname + " b)";
                    break;
            }
        }


        /// <summary>
        /// activates the gui-panel-components
        /// called by cancel or add button
        /// </summary>
        public virtual void activateAll()
        {
            foreach (Control x in panelMain.Controls)
            {
                if (x is ComboBox)
                {
                    ((ComboBox)x).Enabled = true;
                }
            }
            foreach (Control x in panel2.Controls)
            {
                if (x is ComboBox)
                {
                    ((ComboBox)x).Enabled = true;
                }
            }
            foreach (Control x in this.panel3.Controls)
            {
                if (x is ComboBox)
                {
                    ((ComboBox)x).Enabled = true;
                }

            }
            textBox1.Enabled = true;

            active = null;
            this.constructorInput.Name = "constructorInput";
            sourcebox.Enabled = false;
        }

        // to get the active comboBox
        public System.Windows.Forms.Control getActiveOne()
        {
            return this.active;
        }


        /// <summary>
        /// deleting stuff from attributes list
        /// and update listview
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView1_KeyDown(object sender, KeyEventArgs e)
        {
            if (!String.IsNullOrEmpty(classname))
            {

                if (e.KeyCode == Keys.Delete)
                {
                    List<ListViewItem> remove = new List<ListViewItem>();
                    for (int i = 0; i < listView1.SelectedItems.Count; i++)
                    {
                        remove.Add(listView1.SelectedItems[i]);
                    }
                    foreach (ListViewItem entry in remove)
                    {
                        listView1.Items.Remove(entry);
                        attributes.Remove(entry.Text);
                    }
                    try
                    {
                        saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                    }
                    catch { }
                }
                genMemberNames();
            }
            else MessageBox.Show("No classname! Please define some classname!", "Info", MessageBoxButtons.OK);
        }

        /// <summary>
        /// no selection using mousewheel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void comboBox1_MouseWheel(object sender, MouseEventArgs e)
        {
            ((HandledMouseEventArgs)e).Handled = true;
        }

        // update combobox with
        private void listView1_ColumnWidthChanging(object sender, ColumnWidthChangingEventArgs e)
        {
            e.Cancel = true;
            e.NewWidth = ((ListView)sender).Columns[e.ColumnIndex].Width;
        }

        /// <summary>
        /// after selection, update corresponding datamodel-entry or if not exist create
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (classname != "")
            {
                attributes.Add(comboBox1.Text);
                try
                {
                    saveData2.Invoke(this, new ViewEventUpdateDataFree("Free", classname, attributes, methods, optional, optionalinfo, modul, sr, field, euclid));
                    genMemberNames();
                }
                catch { MessageBox.Show("Some error occurs while saving your input!", "Error", MessageBoxButtons.OK); }

            }
            else MessageBox.Show("No classname!", "Info", MessageBoxButtons.OK);
        }

        /// <summary>
        /// generate the attributes name, to make them usable for coding by the user
        /// </summary>
        protected void genMemberNames()
        {
            listView1.Items.Clear();

            for (int i = 0; i < attributes.Count; i++)
            {
                ListViewItem itm = new ListViewItem(attributes[i]);
                itm.SubItems.Add("mem" + i);
                listView1.Items.Add(itm);
            }
        }


        /// <summary>
        /// update the gui-components of the optional methods
        /// </summary>
        private void updateOptionalView()
        {
            listView2.Items.Clear();

            foreach (KeyValuePair<string, string> entry in optional)
            {
                ListViewItem itm = new ListViewItem(entry.Key);
                itm.SubItems.Add(optionalinfo[entry.Key][0]);
                itm.SubItems.Add(optionalinfo[entry.Key][1]);
                listView2.Items.Add(itm);
            }
        }

        /// <summary>
        /// Depending on the state of the semiring-user input,
        /// first check if all necessary information is specified 
        /// by the user.
        /// If not, user will be imformed
        /// </summary>
        /// <returns></returns>
        public virtual bool checkallRequirements()
        {
            if (!methods.Keys.Contains(Constants.ADD)) return false;
            if (!methods.Keys.Contains(Constants.MUL)) return false;
            if (!methods.Keys.Contains(Constants.Z)) return false;
            if (!methods.Keys.Contains(Constants.O)) return false;
            if (!methods.Keys.Contains(Constants.EQUAL)) return false;
            if (!methods.Keys.Contains(Constants.LINCOMBI))
            {
                MessageBox.Show("You must define the LinearCombination method!");
                return false;
            }

            if (!methods.Keys.Contains(Constants.GREATER) && (field || euclid))
            {
                MessageBox.Show("You must define the > operation!");
                return false;
            }
            if (!methods.Keys.Contains(Constants.SUB) && field)
            {
                MessageBox.Show("You must define the - operation!");
                return false;
            }
            if (!methods.Keys.Contains(Constants.DIV) && field)
            {
                MessageBox.Show("You must define the / operation!");
                return false;
            }
            if (methods.Keys.Contains(Constants.LINCOMBI)  && !methods.Keys.Contains(Constants.DIV))
            {
                if(methods[Constants.LINCOMBI] != "LargestSolution" && methods[Constants.LINCOMBI] != "Gauss-Elimination")
                {

                }
                else
                {
                   MessageBox.Show("You must define the / operation, because of LargestSolution or Gauss-Elimination!");
                   return false;
                }
            }
            if (!methods.Keys.Contains(Constants.C1)) methods.Add(Constants.C1, Constants.CM);
            if (!methods.Keys.Contains(Constants.C2)) methods.Add(Constants.C2, Constants.CM);
            if (!methods.Keys.Contains(Constants.C3)) methods.Add(Constants.C3, Constants.CM);
            return true;
        }


        /// <summary>
        /// Always let the gui be consistent with the state in the datamodels
        /// </summary>
        private void updateView()
        {

            genMemberNames();

            // modul or semiring:

            if (modul) checkedListBox1.SetItemChecked(0, true);
            else checkedListBox1.SetItemChecked(0, false);
            if (field)
            {
                checkedListBox1.SetItemChecked(3, true);
                sr = true;
            }
            else checkedListBox1.SetItemChecked(3, false);
            if (sr) checkedListBox1.SetItemChecked(2, true);
            else checkedListBox1.SetItemChecked(2, false);
            if (euclid)
            {
                checkedListBox1.SetItemChecked(1, true);
                sr = true;
            }
            else checkedListBox1.SetItemChecked(1, false);

            // methods          Dictionary<string, string>
            foreach (KeyValuePair<string, string> entry in methods)
            {
                switch (entry.Key)
                {
                    case Constants.LINCOMBI:
                        if (entry.Value.Contains("Gauss")) linCombi.SelectedText = "Gauss-Elimination(Default fields)";
                        if (entry.Value.Contains("Largest-Solution")) linCombi.SelectedText = "Largest-Solution";
                        else linCombi.SelectedText = Constants.SB;
                        break;
                    case Constants.Z:
                        if (entry.Value.Contains(Constants.CM)) zero.SelectedText = entry.Value;
                        else zero.SelectedText = Constants.SB;
                        break;
                    case Constants.O:
                        if (entry.Value.Contains(Constants.CM)) one.SelectedText = entry.Value;
                        else one.SelectedText = Constants.SB;
                        break;
                    case Constants.ADD:
                        if (entry.Value.Contains(Constants.CM)) plus.SelectedText = entry.Value;
                        else plus.SelectedText = Constants.SB;
                        break;
                    case Constants.MUL:
                        if (entry.Value.Contains(Constants.CM)) mul.SelectedText = entry.Value;
                        else mul.SelectedText = Constants.SB;
                        break;
                    case Constants.SUB:
                        if (entry.Value.Contains(Constants.CM)) minus.SelectedText = entry.Value;
                        else minus.SelectedText = Constants.SB;
                        break;
                    case Constants.DIV:
                        div.SelectedText = Constants.SB;
                        break;
                    case Constants.GREATER:
                        comboBox2.SelectedText = Constants.SB;
                        break;
                    case Constants.EQUAL:
                        if (entry.Value.Contains(Constants.CM)) equal.SelectedText = entry.Value;
                        else equal.SelectedText = Constants.SB;
                        break;
                }
            }

            // optionalmethods
            updateOptionalView();

        }

        /// <summary>
        /// gui-dropdownlist adpats to text
        /// visual-stuff
        /// </summary>
        /// <param name="myCombo"></param>
        /// <returns></returns>
        int DropDownWidth(ComboBox myCombo)
        {
            int maxWidth = 0;
            int temp = 0;
            Label label1 = new Label();

            foreach (var obj in myCombo.Items)
            {
                label1.Text = obj.ToString();
                temp = label1.PreferredWidth;
                if (temp > maxWidth)
                {
                    maxWidth = temp;
                }
            }
            label1.Dispose();
            return maxWidth;
        }

        /// <summary>
        /// called before updating the config-files
        /// after generating a new semiring!
        /// without that, nothing will work right!
        /// </summary>
        /// <returns></returns>
        public bool getSR()
        {
            return this.sr;
        }

        public bool getModul()
        {
            return this.modul;
        }

        public bool getField()
        {
            return this.field;
        }

        internal bool getEuclid()
        {
            return this.euclid;
        }



        protected System.Windows.Forms.ComboBox constructorInput;
        protected System.Windows.Forms.Button constructor;
        protected System.Windows.Forms.ComboBox constructorSelect;
        protected System.Windows.Forms.ListView listView1;


        private System.Windows.Forms.Label label3;
        private System.Windows.Forms.CheckedListBox checkedListBox1;
        private System.Windows.Forms.ColumnHeader selected;
        private System.Windows.Forms.ComboBox comboBox1;
        private System.Windows.Forms.Label label2;
        private System.Windows.Forms.Label label1;

        private System.Windows.Forms.Label label6;
        private System.Windows.Forms.Label label5;
        private System.Windows.Forms.ComboBox mul;

        private System.Windows.Forms.ComboBox plus;
        private System.Windows.Forms.Label label4;

        private System.Windows.Forms.ComboBox one;
        private System.Windows.Forms.Label label7;

        private System.Windows.Forms.ComboBox zero;
        private System.Windows.Forms.Label Zerolabel;
        private System.Windows.Forms.Panel panel2;

        private System.Windows.Forms.ComboBox equal;
        private System.Windows.Forms.Label label8;

        private System.Windows.Forms.ComboBox div;
        private System.Windows.Forms.Label label11;

        private System.Windows.Forms.ComboBox minus;
        private System.Windows.Forms.Label label10;
        private System.Windows.Forms.Label label9;

        private System.Windows.Forms.ComboBox linCombi;
        private System.Windows.Forms.Label label12;
        private System.Windows.Forms.Panel panel3;
        private System.Windows.Forms.Label label13;
        private System.Windows.Forms.ListView listView2;
        private System.Windows.Forms.Button addPara;
        private System.Windows.Forms.TextBox textBox1;
        private System.Windows.Forms.TextBox ret;
        private System.Windows.Forms.TextBox parameters;
        private System.Windows.Forms.Label label16;
        private System.Windows.Forms.Label label15;
        private System.Windows.Forms.Label label14;
        private System.Windows.Forms.Button button2;
        private System.Windows.Forms.Button reset;





        protected List<string> attributes;
        protected Dictionary<string, string> methods;


        private Dictionary<string, string> optional;
        private Dictionary<string, List<string>> optionalinfo;
        public System.Windows.Forms.ColumnHeader name;
        public System.Windows.Forms.ColumnHeader code;

        private ColumnHeader methodname;
        private ColumnHeader paras;
        private ColumnHeader rettype;

        private System.Windows.Forms.ComboBox comboBox2;
        private System.Windows.Forms.Label label17;

    }
}
