﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.IO;

namespace Generator_2._0
{
    /// <summary>
    /// Controller classes of MVC-Design
    /// IController interface for main-gui
    /// 
    /// and two extra interfaces for the gui-masks
    /// </summary>
    public interface IController
    {
        bool checkClassname(string name, string type);
        bool checkOtherClassname(string name, string type);
        bool checkAllClassname(string name);
        void saveState(string filename);
        void setModels(Dictionary<string, DataModel> mod);
        void deletedata(string srname);
    }

    /// <summary>
    /// Interface for DirectProduct and lmonoids
    /// </summary>
    public interface IController1
    {
        void saveData(IView1 sender, ViewEventUpdateData e);
        bool checkClassname(string name, string type);
    }


    /// <summary>
    /// Interface for ArbitrarySemirings
    /// </summary>
    public interface IController2
    {
        void saveFreeData(IView2 sender, ViewEventUpdateDataFree e);
        bool checkClassname(string name, string type);
    }


    /// <summary>
    /// communication to get the data saved in the corresponding model
    /// </summary>

    public class Controller : IController
    {
        IView view;

        public Dictionary<string, DataModel> models;
        public Controller(Dictionary<string, DataModel> models, IView v)
        {
            this.view = v;
            this.models = models;
            // init classname_box for datamodel content
            view.setController(this);

            foreach (string s in models.Keys)
            {
                models[s].attach((IModelObserver)view);
            }

            view.getDataList += new ViewHandler0<IView>(this.getDataList);
            view.getDataDict += new ViewHandlerDict<IView>(this.getDataDict);
            view.getDataDictList += new ViewHandlerDictList<IView>(this.getDataDictList);
            view.getBool += new ViewHandlerBool<IView>(this.getBool);
        }

        /// <summary>
        /// get the value of the semiring-triggers
        /// needed for config-file management
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private bool getBool(IView sender, ViewEventArgs e)
        {
            return ((IModel3)getModel(e.semi)).getBool(e.semi, e.name);
        }

        /// <summary>
        /// to get the optional-data saved for aribtrary semirings
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private Dictionary<string, List<string>> getDataDictList(IView sender, ViewEventArgs e)
        {
            return ((IModel3)getModel(e.semi)).getDataDictList(e.semi, e.name);
        }

        /// <summary>
        /// to geht the methods-specification
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        private Dictionary<string, string> getDataDict(IView sender, ViewEventArgs e)
        {
            if (e.semi == Constants.ASR)
            {
                return ((DataModelExd2)getModel(Constants.ASR)).getDataDict(((ViewEventUpdateDataFree)e).method_info, e.name);
            }
            else return ((IModel2)getModel(e.semi)).getDataDict(e.semi, e.name);
        }

        /// <summary>
        /// get the attributes
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <returns></returns>
        public List<string> getDataList(IView sender, ViewEventArgs e)
        {
            return getModel(e.semi).getDataList(e.semi, e.name);
        }

        /// <summary>
        /// get right model
        /// </summary>
        /// <param name="typename"></param>
        /// <returns></returns>
        public DataModel getModel(string typename)
        {
            return models[typename];
        }

        /// <summary>
        /// get the model by classname
        /// used for panel update
        /// </summary>
        /// <param name="classname"></param>
        /// <returns></returns>
        public string getModelbyClassname(string classname)
        {
            string model = "";
            bool ret = true;
            foreach (KeyValuePair<string, DataModel> entry in models)
            {
                ret = entry.Value.checkClassname(classname);
                if (!ret) model = entry.Key;
            }
            return model;
        }

        /// <summary>
        /// chech if the current model is using the current classname
        /// </summary>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public bool checkClassname(string name, string type)
        {
            bool ret = models[type].checkClassname(name);
            return ret;
        }


        /// <summary>
        /// Method for checking, if classname is used by any datamodel
        /// </summary>
        /// <param name="name"></param>
        /// <returns>returns true, if you can use it</returns>
        public bool checkAllClassname(string name)
        {
            bool ret = true;
            foreach (KeyValuePair<string, DataModel> entry in models)
            {
                ret = entry.Value.checkClassname(name);
                if (!ret) return ret;
            }
            return ret;
        }

        /// <summary>
        /// delete the data for a classname
        /// </summary>
        /// <param name="name"></param>
        public void deletedata(string name)
        {
            foreach (KeyValuePair<string, DataModel> entry in models)
            {
                if (!entry.Value.checkClassname(name))
                {
                    models[entry.Key].deleteData(name);
                }

            }

        }

        /// <summary>
        /// this method is checking, if any other model contains a
        /// class with the current input of the user for classname.
        /// </summary>
        /// <param name="name">current input for classname</param>
        /// <param name="type">model-type</param>
        /// <returns></returns>
        public bool checkOtherClassname(string name, string type)
        {
            bool ret = true;
            foreach (KeyValuePair<string, DataModel> entry in models)
            {
                if (entry.Key != type)
                {
                    ret = entry.Value.checkClassname(name);
                    if (!ret) return ret;
                }
            }
            return ret;
        }

        /// <summary>
        ///Serialization of the datamodels
        /// </summary>
        /// <param name="filename"></param>
        public void saveState(string filename)
        {
            if (filename.Contains(".bin")) filename = filename.Substring(0, filename.Length - 4);
            IFormatter formatter = new BinaryFormatter();
            Stream stream = new FileStream(filename + ".bin",
                                     FileMode.Create,
                                     FileAccess.Write, FileShare.None);

            try
            {
                formatter.Serialize(stream, models);
            }
            catch { }
            stream.Close();
        }

        public void setModels(Dictionary<string, DataModel> models)
        {
            this.models = models;
            foreach (string s in models.Keys)
            {
                models[s].attach((IModelObserver)view);
            }
        }


    }



    /*
     * seperate controllers for saving any changes
     * corresponding to the view of the semiring type
     * 
     * MVC- events and methods
     * similar methods like above!
     */

    public class Controller1 : IController1
    {
        IView1 view;
        DataModel model;
        IController main_control;

        public Controller1(DataModel model)
        {
            this.model = model;
        }


        public void saveData(IView1 sender, ViewEventUpdateData e)
        {
            model.saveData(e.name, e.semi, e.member, e.methods);
        }

        // update to view panel
        public void setView(IView1 v)
        {
            this.view = v;
            view.setController(this);
            view.saveData1 += new ViewHandler1<IView1>(this.saveData);
        }
        public void setController(IController main)
        {
            this.main_control = main;
        }

        public void setModel(DataModel model)
        {
            this.model = model;
        }

        public bool checkClassname(string name, string type)
        {
            bool ret = main_control.checkOtherClassname(name, type);
            return ret;
        }

    }

    public class Controller2 : IController2
    {
        IView2 view;
        DataModelExd2 model;
        IController main_control;

        public Controller2(DataModelExd2 model)
        {
            this.model = model;
        }

        public void saveFreeData(IView2 sender, ViewEventUpdateDataFree e)
        {
            model.saveData(e.semi, e.name, e.attributes, e.freemethods, e.optional, e.optionalinfo, e.modul, e.sr, e.field, e.euclid);
        }

        public void setController(IController main)
        {
            this.main_control = main;
        }

        public void setView(IView2 v)
        {
            this.view = v;
            view.setController(this);
            view.saveData2 += new ViewHandler2<IView2>(this.saveFreeData);
        }

        public void setModel(DataModelExd2 model)
        {
            this.model = model;
        }

        public bool checkClassname(string name, string type)
        {
            bool ret = main_control.checkOtherClassname(name, type);
            return ret;
        }

    }
}
