﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualBasic;
using System.Windows.Forms;
using System.IO;

namespace Generator_2._0
{
    /*
     * The data managment class for semiring information
     * saving all user input for the corresponding classname
     * data model is partitioned into 4 types:
     * directproduct
     * fields from fieldextension
     * lmonoids
     * arbitrary semirings
     * 
     */



    public delegate void ModelHandler<IModel>(IModel sender, ViewEventArgs e);


    /// <suods mmary>
    /// the model Interface containing all necessary methods for supporting datamanagment for all 4 types
    /// 3 different interfaces, depending on datastructures (depending on information, which should be stored)
    /// </summary>

    public interface IModel
    {
        void attach(IModelObserver imo);
        List<String> getDataList(string type, string name);

        void saveData(string type, string name, List<String> list1, Dictionary<string, string> list2);

        void deleteData(string name);

    }

    public interface IModel2
    {

        Dictionary<string, string> getDataDict(string type, string name);
    }

    public interface IModel3
    {

        void saveData(string type, string name, List<String> member, Dictionary<string, string> methodcode, Dictionary<string, string> optional, Dictionary<string, List<string>> info, bool m, bool sr, bool field, bool euclid);
        bool getBool(string type, string name);
        Dictionary<string, List<String>> getDataDictList(string type, string name);

    }



    public interface IModelObserver
    {
        void viewListchanged(IModel model, ViewEventArgs e);
    }



    //base model class, used directly by the directProduct
    [Serializable]
    public class DataModel : IModel
    {
        //MVC part
        [field: NonSerialized]
        public event ModelHandler<DataModel> changed;

        // Constructors and necessary operators and methods
        // name--> code as string or automatic keyword
        protected Dictionary<string, List<string>> attributes;
        public List<string> names;

        public DataModel()
        {
            attributes = new Dictionary<string, List<string>>();
            names = new List<string>();
        }


        /// <summary>
        /// delete an entry
        /// </summary>
        /// <param name="name"></param>
        public virtual void deleteData(string name)
        {
            if (attributes.Keys.Contains(name))
            {
                attributes.Remove(name);
                names.Remove(name);
            }

        }

        /// <summary>
        /// saves the current input of the user:
        /// </summary>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <param name="list1"></param>
        /// <param name="list2"></param>
        public virtual void saveData(string name, string type, List<String> list1, Dictionary<string, string> list2)
        {
            if (attributes.Keys.Contains(name))
            {
                attributes[name] = list1;
            }
            else
            {
                attributes.Add(name, list1);
                names.Add(name);
                updateViewList(name);
            }
        }
        /// <summary>
        /// corresponding attribute information for any SR
        /// </summary>
        /// <param name="type"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public List<string> getDataList(string type, string name)
        {
            if (attributes.Keys.Contains(name)) return attributes[name];
            return new List<string>();
        }


        /// <summary>
        /// if a name is already in use, the method returns false
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public virtual bool checkClassname(string name)
        {
            if (this.attributes.Keys.Contains(name)) return false;
            else return true;
        }

        /// <summary>
        /// to invoke corresponding method in view
        /// to update gui-state
        /// </summary>
        /// <param name="name"></param>
        public void updateViewList(string name)
        {
            changed.Invoke(this, new ViewEventArgs("", name));
        }

        // attach event based stuff
        public void attach(IModelObserver imo)
        {
            this.changed += new ModelHandler<DataModel>(imo.viewListchanged);
        }

    }

    [Serializable]
    // model used by fieldOFFraction and lmonoids
    public class DataModelExd : DataModel, IModel2
    {
        protected Hashtable trigger;

        // methods semirings:
        protected Dictionary<string, Dictionary<string, string>> methods;// classname--> methodname--> code as string

        public DataModelExd()
            : base()
        {
            trigger = new Hashtable();
            methods = new Dictionary<string, Dictionary<string, string>>();
        }

        /// <summary>
        /// Returns the corresponding method-implementation-info for given classname
        /// only for optional & FreeModules Methods
        /// </summary>
        /// <param name="type"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public virtual Dictionary<string, string> getDataDict(string type, string name)
        {

            if (methods.Keys.Contains(name)) return methods[name];
            else return new Dictionary<string, string>();
        }

        /// <summary>
        /// delete entry if exists
        /// </summary>
        /// <param name="name"></param>
        public override void deleteData(string name)
        {
            if (attributes.Keys.Contains(name))
            {
                attributes.Remove(name);
                methods.Remove(name);
                names.Remove(name);
                updateViewList(name);
            }

        }

        /// <summary>
        /// to update the current datamodel entry or create one
        /// </summary>
        /// <param name="name"></param>
        /// <param name="type"></param>
        /// <param name="list1"></param>
        /// <param name="list2"></param>
        public override void saveData(string name, string type, List<String> list1, Dictionary<string, string> list2)
        {

            if (attributes.Keys.Contains(name))
            {

                attributes[name] = list1;
                methods[name] = list2;

            }
            else
            {
                attributes.Add(name, list1);
                methods.Add(name, list2);
                names.Add(name);
                updateViewList(name);
            }


        }


    }

    /// <summary>
    /// Model class for arbitrary semirings
    /// the same like the other, but handling also optional methods
    /// </summary>
    [Serializable]
    public class DataModelExd2 : DataModelExd, IModel3
    {

        // name--> code as string
        Dictionary<string, Dictionary<string, string>> optional;// classname --> methodname--> code as string
        Dictionary<string, Dictionary<string, List<string>>> optionalinfo;// classname --> methodname --> parameter & returntype


        public DataModelExd2()
            : base()
        {

            optional = new Dictionary<string, Dictionary<string, string>>();
            optionalinfo = new Dictionary<string, Dictionary<string, List<string>>>();
        }
        public Dictionary<string, List<String>> getDataDictList(string type, string name)
        {
            if (optionalinfo.Keys.Contains(name)) return optionalinfo[name];
            else return new Dictionary<string, List<String>>();
        }

        public bool getBool(string type, string name)
        {
            if (trigger.Contains(name)) return (bool)trigger[name];
            if (!name.Contains("SR") || (!name.Contains("Field"))) return true; //default modul is set as true
            else return false;
        }
        public override Dictionary<string, string> getDataDict(string type, string name)
        {
            if (type == "methods")
            {
                if (methods.Keys.Contains(name)) return methods[name];
            }
            if (type == "optional")
            {
                if (optional.Keys.Contains(name)) return optional[name];
            }

            return new Dictionary<string, string>();
        }
        public override void deleteData(string name)
        {
            if (attributes.Keys.Contains(name))
            {
                attributes.Remove(name);
                methods.Remove(name);
                names.Remove(name);
                this.optional.Remove(name);
                this.optionalinfo.Remove(name);
                trigger.Remove(name);
                trigger.Remove(name + "SR");
                trigger.Remove(name + "Field");
                trigger.Remove(name + "Euclid");
                updateViewList(name);
            }

        }
        public void saveData(string type, string name, List<String> member, Dictionary<string, string> methodcode, Dictionary<string, string> optional, Dictionary<string, List<string>> info, bool m, bool sr, bool field, bool euclid)
        {

            if (attributes.Keys.Contains(name))
            {

                attributes[name] = member;
                methods[name] = methodcode;
                this.optional[name] = optional;
                this.optionalinfo[name] = info;
                trigger[name] = m;
                trigger[name + "SR"] = sr;
                trigger[name + "Field"] = field;
                trigger[name + "Euclid"] = euclid;

            }
            else
            {
                attributes.Add(name, member);
                methods.Add(name, methodcode);
                this.optional.Add(name, optional);
                this.optionalinfo.Add(name, info);
                trigger.Add(name, m);
                trigger.Add(name + "SR", sr);
                trigger.Add(name + "Field", field);
                trigger.Add(name + "Euclid", euclid);
                names.Add(name);
                updateViewList(name);
            }
        }//end method
    }
}
