﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.CodeDom;
using System.Reflection;

namespace Generator_2._0
{
    class FieldOfFractionGenerator:Generator
    {

        /// <summary>
        /// this class is providing two templates for FieldOfFraction generation
        /// one based on euclidean rings
        /// second based on a galouis concept
        /// 
        /// </summary>
        /// <param name="classname"></param>
        /// <param name="t"></param>
        public FieldOfFractionGenerator(String classname, string t)
            : base(classname,t)
        {

        }

               /// <summary>
        /// Field template generating method:
        /// with selected attributes and methods definitions
        /// 
        /// this method is generating all nessary CodeMemberMethods for the this CompileUnit
        /// Addition
        /// Multiplication
        /// Subtraction
        /// Division
        /// ==
        /// !=
        /// >
        /// <
        /// shortFraction
        /// 
        /// </summary>
        /// <param name="attributes">selected attributes</param>
        /// <param name="methods">including method information or non generic sourcecode</param>
        /// <param name="fieldextension">trigger: fieldextension based on euclidean rings or galois theorie</param>
        public bool generateField(List<string> attributes, Dictionary<string, string> methods)
        {
            // prepare member info:
            Dictionary<int, string> finite_SR = finiteSRInfo(attributes);
            attributes.Add(attributes[0]);
            generateAttributes(attributes);

            if (basetypes.Count != attributes.Count) return false;

            // generate Constructors:
            // the default empty one
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            targetClass.Members.Add(constructor);
            // class, basetype, string
            targetClass.Members.Add(addConstructorNewClassFOF(attributes, targetClass.Name));
            targetClass.Members.Add(addConstructorByAttriFOF(attributes));
            targetClass.Members.Add(addStringConstructorFOF(attributes));


            //todo operators & CO
            CodeStatement[] st;
            foreach (KeyValuePair<string, string> entry in methods)
            {
                string returntype = targetClass.Name;
                string op = "";
                if (entry.Key == Constants.ADD) op = "+";
                if (entry.Key == Constants.EQUAL) { returntype = "System.Boolean"; op = "=="; }
                if (entry.Key == Constants.MUL) op = "*";
                if (entry.Key == Constants.SUB) op = "-";
                if (entry.Key == Constants.DIV) op = "/";


                if (entry.Value.Contains(Constants.CM)) st = generateMethodComponentwiseFieldextension(op, attributes);
                else st = st = extractMethodsStatements(entry.Value, op, attributes, finite_SR);
                    targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator :" + op, "operator " + op, returntype, st));

            }
            st = new CodeStatement[1];
            st[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("return !(a==b)"));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : !=", "operator !=", "System.Boolean", st));

            // Zero, One, ToString

            targetClass.Members.Add(minusTFOF(targetClass.Name, attributes));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the One Element:", "One", targetClass.Name, getMethodFOF(attributes, "One()")));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the Zero Element:", "Zero", targetClass.Name, getMethodFOF(attributes, "Zero()")));

            targetClass.Members.Add(getToString(attributes));

            //special getSplit-methods for the string constructor
            //targetClass.Members.Add(generateGetSplit(attributes));
            //targetClass.Members.Add(generateClosedBracketSearch());

            // add short fraction
            //Type t = getType(attributes[0]);
            Type t = basetypes[0];
            targetClass.Members.Add(addShortFraction(t, attributes[0]));
            
            // add FindLinear
            getGauss(attributes, targetClass);
            // add allowedStrings
            targetClass.Members.Add(addAllowedString(attributes,finite_SR));

            return true;

        }

        /// <summary>
        /// NewClass Constructor with his own type as single parameter for 
        /// field-of-fraction
        /// </summary>
        /// <param name="t"></param>
        /// <param name="targetClass"></param>
        /// <returns></returns>
        public static CodeConstructor addConstructorNewClassFOF(List<string> attributes, string targetClass)
        {
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            constructor.Parameters.Add(new CodeParameterDeclarationExpression(targetClass, "x"));
            for (int i = 0; i < attributes.Count(); i++)
            {

                CodeAssignStatement as1;
                if (attributes[i] == "System.Int32" || attributes[i] == "System.Double")
                {
                    as1 = new CodeAssignStatement(new CodeVariableReferenceExpression("this.mem" + i), new CodeVariableReferenceExpression("x.mem" + i));
                    constructor.Statements.Add(as1);
                }

                if (!attributes[i].Contains("Int") && !attributes[i].Contains("Double"))
                {
                    // deep copy:
                    Type t = getType(attributes[i]);
                    CodeExpression newExpr = new CodeObjectCreateExpression(new CodeTypeReference(t), new CodeVariableReferenceExpression("x.mem" + i));
                    as1 = new CodeAssignStatement(new CodeVariableReferenceExpression("this.mem" + i), newExpr);
                    constructor.Statements.Add(as1);
                }

                // fieldextension: no division by Zero!
                if (i == 1)
                {
                    as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), new CodePrimitiveExpression(1));
                    if (attributes[i] == "System.Double" || attributes[i] == "System.Int32") constructor.Statements.Add(addIF(as1, "this.mem1", "0", CodeBinaryOperatorType.IdentityEquality));
                    else
                    {
                        as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[i]), "One")));
                        constructor.Statements.Add(addIF(as1, new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem1"), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[i]), "Zero")), CodeBinaryOperatorType.IdentityEquality));
                    }
                }
            }
            return constructor;
        }


        /// <summary>
        /// Template for componentwise-Constructor with the attributes as parameters
        /// for field-of-fraction
        /// </summary>
        /// <param name="t"></param>
        /// <param name="targetClass"></param>
        /// <returns></returns>
        public static CodeConstructor addConstructorByAttriFOF(List<string> attributes)
        {
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            for (int i = 0; i < attributes.Count(); i++)
            {
                constructor.Parameters.Add(new CodeParameterDeclarationExpression(attributes[i], "a" + i));
                if (attributes[i] == "System.Int32" || attributes[i] == "System.Double")
                {
                    constructor.Statements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), new CodeVariableReferenceExpression("a" + i)));
                }

                if (!attributes[i].Contains("Int") && !attributes[i].Contains("Double"))
                {
                    // deep copy:
                    Type t = getType(attributes[i]);
                    CodeExpression newExpr = new CodeObjectCreateExpression(new CodeTypeReference(t), new CodeVariableReferenceExpression("a" + i));
                    constructor.Statements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), newExpr));
                }


                // fieldextension: no division by Zero!
                if (i == 1)
                {
                    CodeAssignStatement as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), new CodePrimitiveExpression(1));
                    if (attributes[i] == "System.Double" || attributes[i] == "System.Int32") constructor.Statements.Add(addIF(as1, "this.mem1", "0", CodeBinaryOperatorType.IdentityEquality));
                    else
                    {
                        as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem" + i), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[i]), "One")));
                        constructor.Statements.Add(addIF(as1, new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem1"), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[i]), "Zero")), CodeBinaryOperatorType.IdentityEquality));
                    }
                }
            }
            return constructor;
        }

           /// <summary>
        /// Field-of-Fraction StringConstructor -template
        /// </summary>
        /// <param name="member"></param>
        /// <param name="test"></param>
        /// <returns></returns>
        public  CodeConstructor addStringConstructorFOF(List<string> attributes)
        {
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            constructor.Attributes = MemberAttributes.Public;
            constructor.Parameters.Add(new CodeParameterDeclarationExpression(typeof(System.String), "str"));

            // generate typeinfo for finite semirings
            // get the basetype for input:
            // convert attributes into array of fieldType
            Type[] obj = new Type[attributes.Count()];
            for (int i = 0; i < attributes.Count(); i++)
            {
                obj[i] = getType(attributes[i]);
            }
            List<Type> result = new List<Type>();
            getBaseTypeOrder(obj, 0, ref result);
            createInputInfo(result);
            // position of arrays and other basetypes
            List<int> array_info = new List<int>();
            for (int i = 0; i < result.Count(); i++)
            {
                if (result[i].IsArray) array_info.Add(i);
            }
            CodeVariableReferenceExpression infor = new CodeVariableReferenceExpression("info");
            CodeArrayCreateExpression cai = new CodeArrayCreateExpression(typeof(int), array_info.Count());
            CodeVariableDeclarationStatement arrayinfo = new CodeVariableDeclarationStatement("System.Int32[]", "info", cai);
            constructor.Statements.Add(arrayinfo);
            for (int i = 0; i < array_info.Count(); i++)
            {
                CodeAssignStatement ass_st = new CodeAssignStatement(new CodeArrayIndexerExpression(infor, new CodePrimitiveExpression(i)), new CodePrimitiveExpression(array_info[i]));
                constructor.Statements.Add(ass_st);
            }
            constructor.Statements.Add(new CodeVariableDeclarationStatement(typeof(string[]), "split", new CodeMethodInvokeExpression(null, "getSplit", Constants.str, new CodeVariableReferenceExpression("info"), new CodePrimitiveExpression(result.Count))));
      
                int start = 0;
                for (int i = 0; i < attributes.Count(); i++)
                {
                    int field = 0;
                    //Type t = Type.GetType(attributes[i]);
                    //if (t == null) t = getType(attributes[i]);
                    Type t = basetypes[i];
                    FieldInfo[] info = t.GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
                    if (info.Count() > 0 && attributes[i] != "Complex" && attributes[i] != "System.Int32" && !attributes[i].Contains("Polynom") && !basetypes[i].IsArray) field += getNumberofFields(info, field);
                    else field += 1;

                    CodeExpression ass = cunstructConcatenation(attributes[i], t, start, field, true);

                    if (t == typeof(uint)) ass = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression("Convert"), "ToUInt32", new CodeArrayIndexerExpression(Constants.split, new CodePrimitiveExpression(start)));
                    if (t == typeof(int)) ass = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression("Convert"), "ToInt32", new CodeArrayIndexerExpression(Constants.split, new CodePrimitiveExpression(start)));
                    if (t == typeof(double)) ass = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression("Convert"), "ToDouble", new CodeArrayIndexerExpression(Constants.split, new CodePrimitiveExpression(start)));
                    // should never be the case...
                    if (attributes[i] == "Complex") ass = new CodeObjectCreateExpression(new CodeTypeReference(attributes[i]), new CodeArrayIndexerExpression(Constants.split, new CodePrimitiveExpression(start)));
                    // no extra fo arrays.

                    CodeStatement st = new CodeAssignStatement(new CodeVariableReferenceExpression("mem" + i), ass);

                    start += field;
                    constructor.Statements.Add(st);

                }

            // if divider is zero by user input:
            CodeAssignStatement as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem1"), new CodePrimitiveExpression(1));
            if (attributes[1] == "System.Double" || attributes[1] == "System.Int32") constructor.Statements.Add(addIF(as1, "this.mem1", "0", CodeBinaryOperatorType.IdentityEquality));
            else
            {
                as1 = new CodeAssignStatement(new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem1"), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[1]), "One")));
                constructor.Statements.Add(addIF(as1, new CodeFieldReferenceExpression(new CodeThisReferenceExpression(), "mem1"), new CodeMethodInvokeExpression(new CodeMethodReferenceExpression(new CodeVariableReferenceExpression(attributes[1]), "Zero")), CodeBinaryOperatorType.IdentityEquality));
            }
            return constructor;
        }

        public CodeStatement[] generateMethodComponentwiseFieldextension(string op, List<string> member)
        {
            CodeStatement[] statement = new CodeStatement[member.Count];

            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression b = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");

            if (op == "+" || op == "-" || op == "==")
            {

                statement = new CodeStatement[3];

                // if both Arguments are Zero
                // inside one statement
                CodeStatement[] stinside = new CodeStatement[1];
                CodeMethodInvokeExpression invokeZero = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[0]), "Zero");
                CodeMethodInvokeExpression invokeOne = new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[0]), "One");
                if (op == "==") stinside[0] = new CodeMethodReturnStatement(new CodePrimitiveExpression(true));
                else
                {
                    statement = new CodeStatement[6];
                    if (member[0] == "System.Int32" || member[0] == "System.Double") stinside[0] = new CodeMethodReturnStatement(new CodeObjectCreateExpression(targetClass.Name, new CodeExpression[] { new CodePrimitiveExpression(0), new CodePrimitiveExpression(1) }));
                    else stinside[0] = new CodeMethodReturnStatement(new CodeObjectCreateExpression(targetClass.Name, new CodeExpression[] { invokeZero, invokeOne }));

                }


                if (member[0] == "System.Int32" || member[0] == "System.Double") statement[0] = addIF(stinside, a, new CodePrimitiveExpression(0), b, new CodePrimitiveExpression(0), CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.BooleanAnd);
                else statement[0] = addIF(stinside, a, invokeZero, b, invokeZero, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.BooleanAnd);

                //if one Argument are Zero
                stinside = new CodeStatement[1];
                if (op == "==") stinside[0] = new CodeMethodReturnStatement(new CodePrimitiveExpression(false));
                else
                {
                    string opSub = "b";
                    if (op == "-") opSub = "-b";
                    CodeExpression cpe = invokeZero;
                    if (member[0] == "System.Int32" || member[0] == "System.Double") cpe = new CodePrimitiveExpression(0);
                    stinside[0] = addIfElse(new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeVariableReferenceExpression(opSub) })), new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeVariableReferenceExpression("a") })), a, cpe, CodeBinaryOperatorType.IdentityEquality);
                }
                if (member[0] == "System.Int32" || member[0] == "System.Double") statement[1] = addIF(stinside, a, new CodePrimitiveExpression(0), b, new CodePrimitiveExpression(0), CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.BooleanOr);
                else statement[1] = addIF(stinside, a, invokeZero, b, invokeZero, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.BooleanOr);


                // 1 if else
                stinside = new CodeStatement[2];
                CodeStatement[] truest = new CodeStatement[1];
                if (op == "==")
                {
                    CodeMethodReturnStatement ret = new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.IdentityEquality, b));
                    truest[0] = new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.IdentityEquality, b));
                    CodeStatement[] falsest = new CodeStatement[3];
                    falsest[0] = new CodeVariableDeclarationStatement(basetypes[0], " ret1", new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Multiply, b1));
                    falsest[1] = new CodeVariableDeclarationStatement(basetypes[0], " ret2", new CodeBinaryOperatorExpression(a1, CodeBinaryOperatorType.Multiply, b));
                    falsest[2] = new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(new CodeVariableReferenceExpression("ret1"), CodeBinaryOperatorType.IdentityEquality, new CodeVariableReferenceExpression("ret2")));

                    statement[2] = addIfElse(truest, falsest, a1, b1, CodeBinaryOperatorType.IdentityEquality);
                }
                else
                {
                    CodeVariableReferenceExpression ret = new CodeVariableReferenceExpression("ret");
                    CodeBinaryOperatorType cbo = CodeBinaryOperatorType.Add;
                    if (op == "-") cbo = CodeBinaryOperatorType.Subtract;
                    statement[2] = new CodeVariableDeclarationStatement(new CodeTypeReference(targetClass.Name), "ret", new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name)));

                    truest[0] = new CodeAssignStatement(ret, new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeBinaryOperatorExpression(a, cbo, b), a1 }));
                    CodeStatement[] falsest = new CodeStatement[2];
                    falsest[0] = new CodeAssignStatement(new CodeFieldReferenceExpression(ret, "mem0"), new CodeBinaryOperatorExpression(new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Multiply, b1), cbo, new CodeBinaryOperatorExpression(b, CodeBinaryOperatorType.Multiply, a1)));
                    falsest[1] = new CodeAssignStatement(new CodeFieldReferenceExpression(ret, "mem1"), new CodeBinaryOperatorExpression(a1, CodeBinaryOperatorType.Multiply, b1));
                    statement[3] = addIfElse(truest, falsest, "a.mem1", "b.mem1", CodeBinaryOperatorType.IdentityEquality);

                    // check if Zero

                    if (member[0] == "System.Int32" || member[0] == "System.Double") statement[4] = addIF(new CodeAssignStatement(ret, new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodePrimitiveExpression(0), new CodePrimitiveExpression(1) })), new CodeFieldReferenceExpression(ret, "mem0"), new CodePrimitiveExpression(0), CodeBinaryOperatorType.IdentityEquality);
                    else statement[4] = addIF(new CodeAssignStatement(ret, new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { invokeZero, invokeOne })), new CodeFieldReferenceExpression(ret, "mem0"), invokeZero, CodeBinaryOperatorType.IdentityEquality);
                    // the ret statement
                    CodeMethodInvokeExpression ret_shortfraction = new CodeMethodInvokeExpression(null, "shortFraction", ret);
                    statement[5] = new CodeMethodReturnStatement(ret_shortfraction);
                }


                return statement;

            }

            if (op == "*")
            {

                statement = new CodeStatement[1];
                statement[0] = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Multiply, b), new CodeBinaryOperatorExpression(a1, CodeBinaryOperatorType.Multiply, b1) }));
                return statement;

            }
            if (op == "/")
            {
                statement = new CodeStatement[2];

                statement[0] = new CodeVariableDeclarationStatement(new CodeTypeReference(targetClass.Name), "inverse", new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { b1, b }));
                statement[1] = new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(new CodeVariableReferenceExpression("a"), CodeBinaryOperatorType.Multiply, new CodeVariableReferenceExpression("inverse")));
                return statement;

            }

            return statement;
        }

        protected static CodeMemberMethod minusTFOF(string name, List<string> attr)
        {

            CodeMemberMethod equals = new CodeMemberMethod();
            equals.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            equals.Name = "operator -";
            equals.ReturnType = new CodeTypeReference(name);
            equals.Parameters.Add(new CodeParameterDeclarationExpression(
                      new CodeTypeReference(name), "a"));
            // update to datatypes:
            equals.Statements.Add(new CodeVariableDeclarationStatement(new CodeTypeReference(name), "b", new CodeObjectCreateExpression(new CodeTypeReference(name), new CodeExpression[] { new CodeVariableReferenceExpression("a") })));
            for (int i = 0; i < attr.Count(); i++)
            {
                if (attr[i].Contains("[]")) equals.Statements.Add(addForLoop(new CodeFieldReferenceExpression(new CodeFieldReferenceExpression(Constants.a, "mem" + i), "Length"), "i", new CodeAssignStatement(new CodeArrayIndexerExpression(new CodeFieldReferenceExpression(Constants.b, "mem" + i), new CodeVariableReferenceExpression("i")), new CodeArrayIndexerExpression(new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("-a"), "mem" + i), new CodeVariableReferenceExpression("i")))));
                else equals.Statements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(Constants.b, "mem" + i), new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("-a"), "mem" + i)));
                i = 2;
            }


            equals.Statements.Add(new CodeMethodReturnStatement(Constants.b));
            return equals;
        }

        public CodeStatement[] getMethodFOF(List<string> member, string method)
        {
            CodeMethodReturnStatement ret = new CodeMethodReturnStatement();
            CodeStatement[] st = new CodeStatement[1];

            for (int i = 0; i < member.Count(); i++)
            {

                switch (member[i])
                {

                    case "System.Double":
                        if (method == "One()") ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodePrimitiveExpression(1), new CodePrimitiveExpression(1) }));
                        else ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodePrimitiveExpression(0), new CodePrimitiveExpression(1) }));

                        break;
                    case "System.Int32":
                        if (method == "One()") ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodePrimitiveExpression(1), new CodePrimitiveExpression(1) }));
                        else ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodePrimitiveExpression(0), new CodePrimitiveExpression(1) }));
                        break;
                    default:
                        if (method == "One()") ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[i]), "One"), new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[i]), "One") }));
                        else ret = new CodeMethodReturnStatement(new CodeObjectCreateExpression(new CodeTypeReference(targetClass.Name), new CodeExpression[] { new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[i]), "Zero"), new CodeMethodInvokeExpression(new CodeTypeReferenceExpression(member[i]), "One") }));
                        break;
                }
                i++;
            }

            st[0] = ret;

            return st;
        }

        /// <summary>
        /// Generates the corresponding shortFraction and ggt methods to avoid overflow
        /// for field-ext based on euclidean rings
        /// </summary>
        /// <returns></returns>
        public CodeMemberMethod addShortFraction(Type attr, string attr0)
        {
            string zero = "";
            switch (attr0)
            {
                case "System.Int32":
                    zero = "0";
                    break;
                case "System.Double":
                    zero = "0";
                    break;
                case "System.UInt32":
                    zero = "0";
                    break;
                default:
                    zero += attr0 + ".Zero()";
                    break;
            }
            CodeMemberMethod sf = new CodeMemberMethod();
            sf.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            sf.Name = "shortFraction";
            sf.ReturnType = new CodeTypeReference(targetClass.Name);
            sf.Parameters.Add(new CodeParameterDeclarationExpression(
                      new CodeTypeReference(targetClass.Name), "a"));


            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            // check if Zero or Equal, so 1
            sf.Statements.Add(addIF(new CodeExpressionStatement(new CodeVariableReferenceExpression("return Zero()")), "a.mem0", zero, CodeBinaryOperatorType.IdentityEquality));


            sf.Statements.Add(addIF(new CodeExpressionStatement(new CodeVariableReferenceExpression("return One()")), "a.mem0", "a.mem1", CodeBinaryOperatorType.IdentityEquality));
            // determine the greater one for ggt
            sf.Statements.Add(genDeclareInitExpression(attr, "ggt1", "a.mem1"));
            sf.Statements.Add(genDeclareInitExpression(attr, "ggt2", "a.mem0"));
            CodeStatement[] truest = new CodeStatement[2];
            truest[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("ggt1= a.mem0"));
            truest[1] = new CodeExpressionStatement(new CodeVariableReferenceExpression("ggt2= a.mem1"));
            sf.Statements.Add(addIF(truest, "a.mem0>a.mem1"));
            // get the ggt based on division with rest
            sf.Statements.Add(genDeclareInitExpression(attr, "r", "ggt1 % ggt2"));
            sf.Statements.Add(new CodeExpressionStatement(new CodeVariableReferenceExpression(attr0 + " rest")));

            /* no CodeDom Statement available, could be not generic in the context of languages
            while (r != attr.Zero())
            {
                rest = ggt2 / r;
                ggt2= r;
                r = rest;
            }
            */


            CodeSnippetStatement whileloop = new CodeSnippetStatement("                       while (r != " + zero + ")" +
                                                         "\n                        {\n" +
                                                         "                              rest = ggt2 % r;\n" +
                                                         "                              ggt2= r;\n" +
                                                         "                              r = rest;\n" +
                                                         "\n                        }\n");
            sf.Statements.Add(whileloop);
            sf.Statements.Add(new CodeVariableReferenceExpression("ggt2 = ggt2.divideConstant();"));// not generic in language generation
            // returns the remainder
            sf.Statements.Add(new CodeExpressionStatement(new CodeVariableReferenceExpression("a.mem0 = a.mem0 / ggt2")));
            sf.Statements.Add(new CodeExpressionStatement(new CodeVariableReferenceExpression("a.mem1 = a.mem1 / ggt2")));
            sf.Statements.Add(new CodeExpressionStatement(new CodeVariableReferenceExpression("return a")));

            return sf;
        }





    }// end class
}
