﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Windows.Forms;
using System.IO;
using Microsoft.CSharp;
using System.CodeDom.Compiler;
using System.Xml;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;

namespace Generator_2._0
{
    /// <summary>
    /// Main-GUI-class for code generation
    /// (sr means semiring)
    /// 
    /// At the end of the File: the ViewEventHandler classes!
    /// </summary>
    public partial class Form1 : Form, IView, IModelObserver
    {

        // userInput managment:
        private Panel x;
        private List<string> attributes = new List<string>();
        private Dictionary<string, string> methods = new Dictionary<string, string>();
        private String classname = "";

        // arbitrary semirings datamanagent:
        Dictionary<string, string> freemethods = new Dictionary<string, string>();
        Dictionary<string, string> optional = new Dictionary<string, string>();
        Dictionary<string, List<string>> optionalinfo = new Dictionary<string, List<string>>();

        // controller for seperate gui-panels
        public Controller1 cn1;
        public Controller2 cn2;
        // main controller
        IController controller;

        // event handling:
        public event ViewHandler0<IView> getDataList;
        public event ViewHandlerDict<IView> getDataDict;
        public event ViewHandlerDictList<IView> getDataDictList;
        public event ViewHandlerBool<IView> getBool;

        public Form1(Controller1 cn1, Controller2 cn2)
        {
            classname = "";
            InitializeComponent();
            this.cn1 = cn1;
            this.cn2 = cn2;

            //init-panel for code generation:
            x = new ArbitrarySRPanel(attributes, freemethods, optional, optionalinfo, classname, sourceBox, true, false, false, false, textBox1);
            x.Location = new System.Drawing.Point(0, 0);
            x.Parent = panel2;
            x.Size = x.Parent.Size;
            x.Visible = true;
            cn2.setView((IView2)x);

            this.panel2.Controls.Add(x);
            semiauto_sr.SetItemChecked(1, true);
            semiauto_sr.SelectedIndex = 1;


            // init comboboxes with semirings:
            deleteListViewUpdate("config_ASR.txt");
            deleteListViewUpdate("config_DP.txt");
            deleteListViewUpdate("config_field.txt");
            deleteListViewUpdate("config_euclid.txt");

            string[] configSR = System.IO.File.ReadAllLines("config_modules.txt");
            foreach (string line in configSR)
            {
                if (line != "#int#" && line != "#double#" && line != "#uint#" && line != "#Vector-double#" && line != "#Vector-int#")
                {
                    if (line.Length != 0)
                    {
                        if (!comboBox1.Items.Contains(line.Substring(1, line.Length - 2)))
                        {
                            comboBox1.Items.Add(line.Substring(1, line.Length - 2));

                        }

                    }

                }

            }
        }

        /// <summary>
        /// init-method: delete-semiring comboBox
        /// </summary>
        /// <param name="filename"></param>
        public void deleteListViewUpdate(string filename)
        {
            // only generated SR are allowed to remove from program
            // stored in  config files
            string[] configSR = System.IO.File.ReadAllLines(filename);
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {
                    if (!comboBox1.Items.Contains(line.Substring(1, line.Length - 2)) && !line.Contains("-FiniteLattice"))
                    {
                        if ("Complex" != line.Substring(1, line.Length - 2) && "Rational" != line.Substring(1, line.Length - 2)) comboBox1.Items.Add(line.Substring(1, line.Length - 2));

                    }
                }
            }
        }

        /// <summary>
        /// all names of constructed or already generated sr-classes are stored in
        /// an autosave data of the last session.
        /// </summary>
        /// <param name="model"></param>
        private void updateListfromAutosave(string model)
        {
            List<string> names = ((Controller)controller).getModel(model).names;
            if (names != null )
            {
                for (int i = 0; i < names.Count; i++)
                {
                   if(!classname_box.Items.Contains(names[i])) classname_box.Items.Add(names[i]);
                }
            }
        }


        // View will set the associated controller, this is how view is linked to the controller.
        public void setController(IController cont)
        {
            controller = cont;
            // update class list of last session or loaded session:
            classname_box.Items.Clear();
            updateListfromAutosave(Constants.ASR);
            updateListfromAutosave(Constants.FR);
            updateListfromAutosave(Constants.DP);
            updateListfromAutosave(Constants.LM);
        }


        /// <summary>
        /// this button calls the corresponding generator template
        /// depending on the active gui, the corresponding data-model will
        /// return the data, if the current active classname is of the current
        /// template type. Ohterwise the user will be informed to check his input 
        /// or to select the classname.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void code_generation_Click_1(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(classname))
            {
                MessageBox.Show("Please enter a name for the Semiring or select the corresponding template-panel", "Error", MessageBoxButtons.OK);
            }
            else
            {
                //update to lastsession to avoid conflicts:
                bool inuse = false;
                // check if semiring is used in last session
                // to avoid corruption of last autosave
                string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\bin\\Debug\\";
                string[] configSR = System.IO.File.ReadAllLines(targetPath + "config_lastsession_save.txt");
                foreach (string line in configSR)
                {
                    if (line.Length != 0 && !inuse)
                    {
                        if (line == classname)
                        {
                            inuse = true;
                            MessageBox.Show("You used this semiring in your last session.\nPlease delete all matrices in LEC and return to SRGenerator \nfor any updates on the semiring.", "Info", MessageBoxButtons.OK);
                        }
                    }
                }
                // call used-asbasetype
                if (!inuse)
                {
                    inuse = used_asBasetype(classname);
                    if (inuse) MessageBox.Show("This Semiring is used as basetype. Delete or changes not possible.\nFirst delete all semirings depending on that one!", "Info", MessageBoxButtons.OK);
                }
                if (classname == "Rational")
                {
                    MessageBox.Show("The Rational class is part of the SRGenerator. You can't delete or override it.", "Info", MessageBoxButtons.OK);
                    inuse = true;
                }

                if (!inuse)
                {

                Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.MsgBoxResult.Yes;

                // check if a class with the current name ist already compiled
                if (!classNotExists(classname))
                {
                    response = Microsoft.VisualBasic.Interaction.MsgBox("Do you want to override the inputs for " + classname_box.Text + ".cs ? ", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                    // if yes, than delete rom dll and recompile
                    if (response == Microsoft.VisualBasic.MsgBoxResult.Yes)
                    {
                        deleteSR(classname);      
                    }
                }

                if (response == Microsoft.VisualBasic.MsgBoxResult.Yes)
                {
                    // get the active GUI-Panel
                    string guipanel = getGuiPanel();
                    if (guipanel != "")
                    {
                        sourceBox.Clear();

                        if (guipanel == "Direct Product")
                        {
                            //get the data for the classname
                            try
                            {
                                attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.DP, classname));
                            }
                            catch
                            { }

                            // init the corresponding generator class
                            if (attributes.Count > 0)
                            {
                                GeneratorDirectProduct gen = new GeneratorDirectProduct(classname, "");
                                List<string> attr = new List<string>(attributes);
                                bool gen_ok=gen.generateDirectProduct(attr);
                                if (gen_ok)
                                {
                                    gen.GenerateCSharpCode();
                                    tryToCompile();
                                }
                   
                            }
                            else MessageBox.Show("No specification given! \nCheck your input or active classname", "Input-Info", MessageBoxButtons.OK);

                        }
                        if (guipanel == "Field-Extension")
                        {

                            //init the corresponding generator class ( sqrt based field-extension)
                            if (((FieldExtPanel)x).checkallRequirements())
                            {
                                GeneratorFieldSqrt gen = new GeneratorFieldSqrt(classname, "Semiring");
                                gen.generateSR(((FieldExtPanel)x).sqrtValue);
                                gen.GenerateCSharpCode();
                                tryToCompile();
                            }
                            else MessageBox.Show("No specification given! \nCheck your input or active classname", "Input-Info", MessageBoxButtons.OK);
                        }

                        if (guipanel == "Field-OfFraction")
                        {
                            // euclidean based construction of sr-classes:

                            //get the data for the classname
                            try
                            {
                                attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.FR, classname));
                                methods = getDataDict.Invoke(this, new ViewEventArgs(Constants.FR, classname));
                            }
                            catch
                            { }

                            if (attributes.Count > 0)
                            {
                                if (((FieldOfFractionPanel)x).checkallRequirements())
                                {
                                    FieldOfFractionGenerator gen = new FieldOfFractionGenerator(classname, "Semiring");
                                    List<string> attr = new List<string>(attributes);
                                    bool gen_ok=gen.generateField(attr, methods);
                                    if (gen_ok) {
                                        gen.GenerateCSharpCode();
                                        tryToCompile();
                                    }
                       
                                }
                                else MessageBox.Show("Define all necessary methods!", "Error", MessageBoxButtons.OK);
                            }

                            else MessageBox.Show("No specification given! \nCheck your input or active classname", "Input-Info", MessageBoxButtons.OK);

                        }

                        if (guipanel == "Arbitrary Semiring")
                        {
                            // get the data from the model
                            try
                            {
                                attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.ASR, classname));
                                freemethods = getDataDict.Invoke(this, new ViewEventUpdateDataFree(Constants.ASR, classname, "methods"));
                                optional = getDataDict.Invoke(this, new ViewEventUpdateDataFree(Constants.ASR, classname, "optional"));
                                optionalinfo = getDataDictList.Invoke(this, new ViewEventArgs(Constants.ASR, classname));
                            }
                            catch { }
                            if (attributes.Count > 0)
                            {
                                // check if all necessary members are defined
                                if (((ArbitrarySRPanel)x).checkallRequirements())
                                {
                                    // trigger at this point not needed
                                    string base_type = "";
                                    if (((ArbitrarySRPanel)x).getField()) base_type = "Semiring";
                                    Generator gen = new Generator(classname, base_type);
                                    List<string> attr = new List<string>(attributes);
                                    bool gen_ok=gen.generateSR(attr, freemethods, optional, optionalinfo);
                                    if (gen_ok)
                                    {
                                        gen.GenerateCSharpCode();
                                        tryToCompile();
                                    }// else case catched in generator
                  

                                }
                                else MessageBox.Show("You need to define: +,*,==,Zero & One!", "Error", MessageBoxButtons.OK);

                            }
                            else MessageBox.Show("No specification given! \nCheck your input or active classname", "Input-Info", MessageBoxButtons.OK);

                        }

                        if (guipanel == "Lattice-based")
                        {

                            try
                            {
                                attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.LM, classname));
                                methods = getDataDict.Invoke(this, new ViewEventArgs(Constants.LM, classname));
                            }
                            catch { }
                            if (attributes.Count > 0)
                            {
                                // check if all necessary members are defined
                                if (((LMonoidPanel)x).checkallRequirements())
                                {
                                    // trigger at this point not needed
                                    GeneratorLMonoid gen = new GeneratorLMonoid(classname, "");
                                    List<string> attr = new List<string>(attributes);
                                   bool gen_ok=gen.generateLMono(attr, methods);
                                   if (gen_ok)
                                   {
                                       gen.GenerateCSharpCode();
                                       tryToCompile();
                                   }
          

                                }
                                else MessageBox.Show("You need to define: +,*,==,>,Zero & One!", "Error", MessageBoxButtons.OK);

                            }
                            else MessageBox.Show("No specification given! \nCheck your input or active classname", "Input-Info", MessageBoxButtons.OK);

                        }

                    }

                }
              }

            }   //else       

        }


        /// <summary>
        /// Checking if any other datamodel is using this name
        /// </summary>
        /// <param name="classname"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        private bool classnameNotExists(string classname, string type)
        {
            bool ret = true;
            // Do not allow to override the generic Polynom-class
            if (classname == "Polynom") return false;
            ret = controller.checkOtherClassname(classname, type);
            return ret;
        }

        /// <summary>
        /// this method is checking if the current classname
        /// input is already generated and compiled as class.
        /// </summary>
        /// <param name="classname"></param>
        /// <returns></returns>
        private bool classNotExists(string classname)
        {
            bool ret = true;
            // Do not allow to override the generic Polynom-class
            if (classname == "Polynom") return false;
            // if already implemented:
            ret = checkSrFiles(classname, "config_main.txt");
            if (!ret) return ret;
            ret = checkSrFiles(classname, "config_modules.txt");
            if (!ret) return ret;
            ret = checkSrFiles(classname, "config_ASR.txt");
            if (!ret) return ret;
            ret = checkSrFiles(classname, "config_DP.txt");
            if (!ret) return ret;
            ret = checkSrFiles(classname, "config_field.txt");
            if (!ret) return ret;
            ret = checkSrFiles(classname, "config_stringConstr.txt");
            return ret;
        }

        /// <summary>
        /// is searching for the classname in any config file
        /// </summary>
        /// <param name="sr"></param>
        /// <param name="filename"></param>
        /// <returns></returns>
        public bool checkSrFiles(string sr, string filename)
        {
            try
            {
                string[] configSR = System.IO.File.ReadAllLines(filename);
                foreach (string line in configSR)
                {
                    if (line.Length != 0)
                    {
                        if (line.Substring(1, line.Length - 2) == sr) return false;

                    }

                }


            }
            catch (FileNotFoundException fe)
            {
                MessageBox.Show("Config file not found.", "Info", MessageBoxButtons.OK);
            }


            return true;
        }

        /// <summary>
        ///  process a test compiling : error handling befor App1 update
        ///  copy into App1 project folder
        ///  include into csproj
        ///  
        ///  update all config files
        ///  BUILDS the AlgorithmB application to update it to the new class!
        /// </summary>
        private void tryToCompile()
        {
            // get tha current path and target path of AlgorithmB APP
            string sourcePath = Directory.GetCurrentDirectory();
            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\";

            try
            {
                // get the generated code 
                sourceBox.Clear();
                sourceBox.AppendText(System.IO.File.ReadAllText(Directory.GetCurrentDirectory() + "\\" + classname + ".cs"));
            }
            catch (Exception error)
            {
                sourceBox.Clear();
                richTextBox1.AppendText("Exception caught. " + error.ToString());
            }

            if (sourceBox.Text.Length > 0 && (semiauto_sr.CheckedItems.Count > 0 || fullauto_sr.CheckedItems.Count > 0))
            {
                // if the user changes  anything!! old, should be alwasy done by the gut
                // and not via editor, to avoid inconsistence with the saved data in the model
                //File.WriteAllText(classname + ".cs", sourceBox.Text);

                // the .net CodeDOM generator-class
                CSharpCodeProvider provider = new CSharpCodeProvider();

                // Build the parameters for source compilation.
                CompilerParameters cp = new CompilerParameters();

                // Add the assembly references
                // TODO for future: change this as user via gui
                cp.ReferencedAssemblies.Add("System.dll");
                cp.ReferencedAssemblies.Add("System.Linq.dll");
                cp.ReferencedAssemblies.Add("System.Windows.Forms.dll");
                cp.ReferencedAssemblies.Add("Microsoft.VisualBasic.dll");
                cp.ReferencedAssemblies.Add("System.Drawing.dll");
                cp.ReferencedAssemblies.Add(targetPath + "bin\\debug\\Analysis_Part.exe");

                // Generate an executable instead of
                // a class library.
                cp.GenerateExecutable = false;
                // Save the assembly as a physical file.
                cp.GenerateInMemory = false;

                // Invoke compilation.
                // this could be also done directly after generation form the CodeDOM-graph
                // seperated here for first implemenatation: Current design GUI and Generator-classes are not connected
                CompilerResults cr = provider.CompileAssemblyFromFile(cp, classname + ".cs");
                richTextBox1.Clear();
                // Display compilation errors.
                if (cr.Errors.Count > 0)
                {
                    int pos = richTextBox1.Text.Length;

                    richTextBox1.AppendText("Errors building {0} into {1}" +
                    classname + ".cs" + cr.PathToAssembly);
                    foreach (CompilerError ce in cr.Errors)
                    {
                        richTextBox1.AppendText("  {0}" + ce.ToString() + "\n");

                    }
                    richTextBox1.Select(pos, richTextBox1.Text.Length - pos);
                    richTextBox1.SelectionColor = Color.Red;
                    richTextBox1.Select();
                    cp.ReferencedAssemblies.Remove(targetPath + "bin\\debug\\Analysis_Part.exe");
                }
                else// no compilation errors!! ;)
                {
                    richTextBox1.AppendText("Source {0} built into {1} successfully." +
                        classname + ".cs" + cr.PathToAssembly);

                    string fileName = classname + ".cs";

                    // Use Path class to manipulate file and directory paths.
                    string sourceFile = System.IO.Path.Combine(sourcePath, fileName);
                    string destFile = System.IO.Path.Combine(targetPath, fileName);

                    // To copy a file to another location and 
                    // overwrite the destination file if it already exists.
                    System.IO.File.Copy(sourceFile, destFile, true);

                    // update the proj. file of visual-studio
                    string csproj = System.IO.File.ReadAllText(targetPath + "Analysis_Part.csproj");
                    int index = csproj.LastIndexOf("<Compile Include=\"" + classname + ".cs\">");

                    if (index < 0) IncludeFilesInProject(targetPath, classname + ".cs");

                    // update MVC:
                    /* error handling
                     * allow only one update for every new semiring of the proj file
                     * 
                     */

                    //check if is semiring for App1 or only used for building anoother semiring (as module)
                    bool sr = true;
                    bool modul = false;
                    bool field = false;
                    bool dp = false;
                    bool euclid = false;

                    if (semiauto_sr.CheckedItems.Count > 0)
                    {
                        if ((string)semiauto_sr.CheckedItems[0] == Constants.ASR)
                        {
                            sr = ((ArbitrarySRPanel)x).getSR();
                            modul = ((ArbitrarySRPanel)x).getModul();
                            field = ((ArbitrarySRPanel)x).getField();
                            euclid = ((ArbitrarySRPanel)x).getEuclid();
                        }
                    }
                    else
                    {
                        if ((string)fullauto_sr.CheckedItems[0] == Constants.FE)
                        {
                            field = true;
                        }
                        if ((string)fullauto_sr.CheckedItems[0] == Constants.FR)
                        {
                            field = true;
                        }
                        if ((string)fullauto_sr.CheckedItems[0] == Constants.DP)
                        {
                            dp = true;
                        }
                    }



                    string structure;


                    //if string constructor is defined by the user, do not allow using for generation
                    //TODO new config file
                    if (methods.Keys.Contains(Constants.C3))
                    {
                        if (methods[Constants.C3] == Constants.SB)
                        {
                            structure = System.IO.File.ReadAllText("config_stringConstr.txt");
                            index = structure.LastIndexOf("#" + classname + "#");
                            if (index < 0 )
                            {
                                // init available modules:
                                structure += "\n" + "#" + classname + "#";
                                File.WriteAllText("config_stringConstr.txt", structure);
                                sourceFile = System.IO.Path.Combine(sourcePath, "config_stringConstr.txt");
                                destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_stringConstr.txt");
                                System.IO.File.Copy(sourceFile, destFile, true);

                            }

                        }
                    }
       
                    else
                    {
                        structure = System.IO.File.ReadAllText("config_modules.txt");
                        index = structure.LastIndexOf("#" + classname + "#");
                        if (index < 0 && modul)
                        {
                            // init available modules:
                            structure += "\n" + "#" + classname + "#";
                            File.WriteAllText("config_modules.txt", structure);

                        }

                        structure = System.IO.File.ReadAllText("config_field.txt");
                        index = structure.LastIndexOf("#" + classname + "#");
                        if (index < 0 && field)
                        {
                            // init available modules:
                            structure += "\n" + "#" + classname + "#";
                            File.WriteAllText("config_field.txt", structure);
                            sourceFile = System.IO.Path.Combine(sourcePath, "config_field.txt");
                            destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_field.txt");
                            System.IO.File.Copy(sourceFile, destFile, true);

                        }

                        // if direct product
                        structure = System.IO.File.ReadAllText("config_DP.txt");
                        index = structure.LastIndexOf("#" + classname + "#");
                        if (index < 0 && dp)
                        {
                            // init available modules:
                            structure += "\n" + "#" + classname + "#";
                            File.WriteAllText("config_DP.txt", structure);
                            sourceFile = System.IO.Path.Combine(sourcePath, "config_DP.txt");
                            destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_DP.txt");
                            System.IO.File.Copy(sourceFile, destFile, true);

                        }

                        // if euclidean ring
                        structure = System.IO.File.ReadAllText("config_euclid.txt");
                        index = structure.LastIndexOf("#" + classname + "#");
                        if (index < 0 && euclid)
                        {
                            // init available modules:
                            structure += "\n" + "#" + classname + "#";
                            File.WriteAllText("config_DP.txt", structure);
                            sourceFile = System.IO.Path.Combine(sourcePath, "config_euclid.txt");
                            destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_euclid.txt");
                            System.IO.File.Copy(sourceFile, destFile, true);

                        }


                        // update structure List :
                        structure = System.IO.File.ReadAllText(targetPath + "bin\\Debug\\config_ASR.txt");
                        index = structure.LastIndexOf("#" + classname + "#");
                        if (index < 0 && (sr || field) && !dp)
                        {
                            structure += "\n" + "#" + classname + "#";
                            File.WriteAllText("config_ASR.txt", structure);
                            sourceFile = System.IO.Path.Combine(sourcePath, "config_ASR.txt");
                            destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_ASR.txt");
                            System.IO.File.Copy(sourceFile, destFile, true);

                        }

                    }


                    // delete in current directory:
                    System.IO.File.Delete(sourcePath + "\\" + classname + ".cs");

                    //build updated App1
                    cp.ReferencedAssemblies.Remove(targetPath + "bin\\Debug\\Analysis_Part.exe");
                    richTextBox1.AppendText(Directory.GetParent(Directory.GetParent(targetPath.ToString()).ToString()).ToString());
                    try
                    {
                        Compiler.Build(Directory.GetParent(Directory.GetParent(targetPath.ToString()).ToString()).ToString() + "\\PAWS.sln");
                        // update gui-selection
                        updateComboboxes(classname);
                    }

                    catch (Exception e1){
                        MessageBox.Show("Something is wrong with your Application-files.", "Error-Info", MessageBoxButtons.OK);
                    }

                }//end else
            }//end if 
        }

     /// <summary>
     /// This method is called, when the user decides to swicht back to AlgorithmB
     /// </summary>
     /// <param name="sender"></param>
     /// <param name="e"></param>
        private void compile_build_click(object sender, EventArgs e)
        {

            // autosave of current state of data
            controller.saveState("autosave");

            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part";


            ProcessStartInfo startInfo = new ProcessStartInfo();
            startInfo.CreateNoWindow = false;
            startInfo.UseShellExecute = false;
            startInfo.FileName = targetPath + "\\bin\\Debug\\Analysis_Part.exe";
            startInfo.WindowStyle = ProcessWindowStyle.Hidden;
            Directory.SetCurrentDirectory(targetPath + "\\bin\\Debug\\");
            startInfo.WorkingDirectory = Environment.CurrentDirectory;

            try
            {
                // start AlgorithmB Application
                System.Diagnostics.Process.Start(startInfo);
                // close Generator
                Application.Exit();
            }
            catch (Exception ex)
            {
                Trace.TraceError(ex.Message);
            }

        }

        /// <summary>
        /// update the csproj file inlucding the new classname
        /// </summary>
        /// <param name="targetPath">the path of the csproj-file</param>
        /// <param name="tagContent">the new classname</param>
        private void IncludeFilesInProject(string targetPath, string tagContent)
        {
            string csprojPath = targetPath + "Analysis_Part.csproj";
            XmlDocument myXmlDocument = new XmlDocument();

            myXmlDocument.Load(csprojPath);

            XmlTextReader reader = new XmlTextReader(csprojPath);
            XmlDocument doc = new XmlDocument();

            doc.Load(reader);
            reader.Close();
            XmlNode currNode;



            XmlDocumentFragment docFrag = doc.CreateDocumentFragment();
            docFrag.InnerXml = "<ItemGroup>" +
                                "<Compile Include=\"" + tagContent + "\">" + "</Compile>" + "</ItemGroup>";

            // insert the availability node into the document

            currNode = doc.DocumentElement;
            currNode.InsertAfter(docFrag, currNode.LastChild);

            //save the output to a file
            doc.Save(csprojPath);

            //Overwrite the existing file with new file content.
            string projFile = File.ReadAllText(csprojPath);
            string newProjFile = projFile.Replace(" xmlns=\"\"", "");
            File.WriteAllText(csprojPath, newProjFile);
        }

        /// <summary>
        /// remove an existing classname from projectfile
        /// </summary>
        /// <param name="targetPath"></param>
        /// <param name="tagContent"></param>
        private void RemoveFilesInProject(string targetPath, string tagContent)
        {
            string csprojPath = targetPath + "Analysis_Part.csproj";
            XmlDocument myXmlDocument = new XmlDocument();

            myXmlDocument.Load(csprojPath);

            XmlTextReader reader = new XmlTextReader(csprojPath);
            XmlDocument doc = new XmlDocument();

            doc.Load(reader);
            reader.Close();
            XmlNode currNode;

            currNode = doc.DocumentElement;

            // this line should be maybe updated if visual studio is changing the proj file contents.
            XmlDocumentFragment docFrag = doc.CreateDocumentFragment();
            docFrag.InnerXml =
                                "<Compile Include=\"" + tagContent + "\" xmlns=\"http://schemas.microsoft.com/developer/msbuild/2003\"></Compile>";

            //currNode.InsertAfter(docFrag, currNode.LastChild);
            XmlNodeList nodes = currNode.ChildNodes;
            for (int i = 0; i < nodes.Count; i++)
            {

                if (docFrag.InnerXml == nodes[i].InnerXml)
                {
                    currNode.RemoveChild(nodes[i]);
                    doc.Save(csprojPath);
                }

            }
            //save the output to a file
            doc.Save(csprojPath);

            //Overwrite the existing file with new file content.
            string projFile = File.ReadAllText(csprojPath);
            string newProjFile = projFile.Replace(" xmlns=\"\"", "");
            File.WriteAllText(csprojPath, newProjFile);
        }

        /// <summary>
        /// allow only one selected gui-semiring-panel
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void semiauto_sr_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            CheckedListBox other = semiauto_sr;
            if (((CheckedListBox)sender) == other) other = fullauto_sr;
            if (e.NewValue == CheckState.Checked && ((CheckedListBox)sender).CheckedItems.Count > 0)
            {
                ((CheckedListBox)sender).ItemCheck -= semiauto_sr_ItemCheck;
                ((CheckedListBox)sender).SetItemChecked(((CheckedListBox)sender).CheckedIndices[0], false);
                ((CheckedListBox)sender).ItemCheck += semiauto_sr_ItemCheck;
            }
            if (e.NewValue == CheckState.Unchecked && other.CheckedItems.Count == 0) x.Visible = false;
            if (e.NewValue == CheckState.Checked) x.Visible = true;
        }



        /// <summary>
        /// The reset button for all temporary user inputs. The data inside the model will be not deleted,
        /// only the panel-textboxes and comboboxes. The first input by the user using the panel, is updating the model-data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void reset_Click(object sender, EventArgs e)
        {
            sourceBox.Clear();

            if ((semiauto_sr.CheckedItems.Count > 0 || fullauto_sr.CheckedItems.Count > 0) && !String.IsNullOrEmpty(classname_box.Text))
            {

                panel2.Controls.Clear();

                if (semiauto_sr.CheckedItems.Count > 0)
                {

                    if (semiauto_sr.CheckedItems[0].ToString() == "Arbitrary Semiring")
                    {
                        attributes = new List<string>();
                        freemethods = new Dictionary<string, string>();
                        optional = new Dictionary<string, string>();
                        optionalinfo = new Dictionary<string, List<string>>();
                        x = new ArbitrarySRPanel(attributes, freemethods, optional, optionalinfo, classname, sourceBox, true, false, false, false, textBox1);
                        x.Location = new System.Drawing.Point(0, 0);
                        cn2.setView((IView2)x);

                    }

                    if (semiauto_sr.CheckedItems[0].ToString() == Constants.LM)
                    {
                        attributes = new List<string>();
                        methods = new Dictionary<string, string>();


                        x = new LMonoidPanel(attributes, methods, classname, sourceBox, textBox1);
                        cn1.setView((IView1)x);
                        cn1.setModel(((Controller)controller).getModel(Constants.LM));

                    }
                }
                else
                {
                    if (fullauto_sr.CheckedItems[0].ToString() == "Direct Product")
                    {

                        attributes = new List<string>();
                        x = new DirectProductPanel(attributes, classname);
                        x.Location = new System.Drawing.Point(0, 0);
                        cn1.setView((IView1)x);

                    }

                    if (fullauto_sr.CheckedItems[0].ToString() == "Field-Extension")
                    {
                        x = new FieldExtPanel(sourceBox, classname, textBox1);
                        x.Location = new System.Drawing.Point(0, 0);
                        cn1.setView((IView1)x);
                    }

                    if (fullauto_sr.CheckedItems[0].ToString() == Constants.FR)
                    {

                        attributes = new List<string>();
                        methods = new Dictionary<string, string>();
                        x = new FieldOfFractionPanel(attributes, methods, sourceBox, classname, textBox1);
                        x.Location = new System.Drawing.Point(0, 0);
                        cn1.setView((IView1)x);
                    }

                }

                x.Parent = panel2;
                x.Size = x.Parent.Size;
                x.TabIndex = 0;
                x.Anchor = (AnchorStyles.Top | AnchorStyles.Right);
                panel2.Controls.Clear();
                panel2.Controls.Add(x);
                panel2.ResumeLayout(false);
                panel2.PerformLayout();
                panel2.Visible = true;

            }
            else { MessageBox.Show("For code generation choose some predifined structure please \nor you just forget the classname?"); }
        }


        /// <summary>
        /// the help button event opens the corresponding help-html file
        /// TODO at least
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void help_Click(object sender, EventArgs e)
        {
            string gui = getGuiPanel();
            if(gui==Constants.ASR)System.Diagnostics.Process.Start("help\\help.html");
            if (gui == Constants.LM) System.Diagnostics.Process.Start("help\\help.html");
            if (gui == Constants.DP) System.Diagnostics.Process.Start("help\\help_dp.html");
            if (gui == Constants.FR) System.Diagnostics.Process.Start("help\\help_field.html");
            if (gui == Constants.FE) System.Diagnostics.Process.Start("help\\help_fieldext.html");
        }

        /// <summary>
        /// Realizing the selection between both checkboxes
        /// Updating the panel to the corresponding data
        /// Method for automatic-generated sr
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void fullauto_sr_SelectedIndexChanged(object sender, EventArgs e)
        {


            if (x.Visible && fullauto_sr.SelectedItems.Count > 0)
            {
                panel2.Controls.Clear();
                if (fullauto_sr.SelectedItem.ToString() == "Direct Product")
                {
                    //if class in work
                    if (!classnameNotExists(classname, Constants.DP))
                    {
                        classname = "";
                        current.Text = "";
                    }
                    try
                    {
                        attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.DP, classname));
                    }
                    catch { }


                    x = new DirectProductPanel(attributes, classname);
                    cn1.setView((IView1)x);
                    cn1.setModel(((Controller)controller).getModel(Constants.DP));


                }

                if (fullauto_sr.SelectedItem.ToString() == "Field-Extension")
                {
                    //if class in work
                    if (!classnameNotExists(classname, Constants.FE))
                    {
                        classname = "";
                        current.Text = "";
                    }
                    x = new FieldExtPanel(sourceBox, classname, textBox1);
                    cn1.setView((IView1)x);
                    //cn1.setModel(((Controller)controller).getModel(Constants.FE));
                }

                if (fullauto_sr.SelectedItem.ToString() == "Field-OfFraction")
                {
                    if (!classnameNotExists(classname, Constants.FR))
                    {
                        classname = "";
                        current.Text = "";
                    }
                    try
                    {
                        attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.FR, classname));
                        methods = getDataDict.Invoke(this, new ViewEventArgs(Constants.FR, classname));
                    }
                    catch { }


                    x = new FieldOfFractionPanel(attributes, methods, sourceBox, classname, textBox1);
                    cn1.setView((IView1)x);
                    cn1.setModel(((Controller)controller).getModel(Constants.FR));


                }

                x.Parent = panel2;
                x.Size = x.Parent.Size;
                x.TabIndex = 0;
                x.Anchor = (AnchorStyles.Top | AnchorStyles.Right);
                panel2.Controls.Add(x);
                panel2.ResumeLayout(false);
                panel2.PerformLayout();
            }

            if (semiauto_sr.CheckedItems.Count != 0 && fullauto_sr.SelectedItem != null)
            {
                //remove other selection
                try
                {
                    semiauto_sr.SetItemCheckState(semiauto_sr.SelectedIndex, CheckState.Unchecked);
                    semiauto_sr.SelectedItem = null;
                }
                catch { }
            }
        }

        /// <summary>
        /// Realizing the selection between both checkboxes
        /// Updating the panel to the corresponding data
        /// Method for semi-automatic-generated sr
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void semiauto_sr_SelectedIndexChanged(object sender, EventArgs e)
        {

            sourceBox.Clear();

            if (x.Visible && semiauto_sr.SelectedItems.Count > 0)
            {
                panel2.Controls.Clear();
                if (semiauto_sr.SelectedItem.ToString() == "Arbitrary Semiring")
                {
                    //if class in work
                    if (!classnameNotExists(classname, Constants.ASR))
                    {
                        classname = "";
                        current.Text = "";
                    }
                    try
                    {
                        attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text));
                        freemethods = getDataDict.Invoke(this, new ViewEventUpdateDataFree(Constants.ASR, classname_box.Text, "methods"));
                        optional = getDataDict.Invoke(this, new ViewEventUpdateDataFree(Constants.ASR, classname_box.Text, "optional"));
                        optionalinfo = getDataDictList.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text));
                    }
                    catch { }


                    x = new ArbitrarySRPanel(attributes, freemethods, optional, optionalinfo, classname, sourceBox, getBool.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text)), getBool.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text + "SR")), getBool.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text + "Field")), getBool.Invoke(this, new ViewEventArgs(Constants.ASR, classname_box.Text + "Euclid")), textBox1);
                    cn2.setView((IView2)x);
                    cn2.setModel((DataModelExd2)((Controller)controller).getModel(Constants.ASR));

                }


                if (semiauto_sr.SelectedItem.ToString() == "Lattice-based")
                {
                    if (!classnameNotExists(classname, Constants.LM))
                    {
                        classname = "";
                        current.Text = "";
                    }
                    try
                    {
                        attributes = getDataList.Invoke(this, new ViewEventArgs(Constants.LM, classname_box.Text));
                        methods = getDataDict.Invoke(this, new ViewEventArgs(Constants.LM, classname_box.Text));
                    }
                    catch { }


                    x = new LMonoidPanel(attributes, methods, classname, sourceBox, textBox1);
                    cn1.setView((IView1)x);
                    cn1.setModel(((Controller)controller).getModel(Constants.LM));

                }

                x.Parent = panel2;
                x.Size = x.Parent.Size;
                x.TabIndex = 0;
                x.Anchor = (AnchorStyles.Top | AnchorStyles.Right);

                panel2.Controls.Add(x);
                panel2.ResumeLayout(false);
                panel2.PerformLayout();

            }

            if (fullauto_sr.CheckedItems.Count != 0 && semiauto_sr.SelectedItem != null)
            {

                //remove other selection
                try
                {
                    fullauto_sr.SetItemCheckState(fullauto_sr.SelectedIndex, CheckState.Unchecked);
                    fullauto_sr.SelectedItem = null;
                }
                catch { }

            }

        }



        /// <summary>
        /// only allow the user to remove generated SR
        /// 
        /// All config files are updated and the csproj file too.
        /// After that: a new exe file for AlgoB App is build.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void deletesr_Click(object sender, EventArgs e)
        {
            if(comboBox1.SelectedItem !=null)
            {
                bool inuse = false;
                // check if semiring is used in last session
                // to avoid corruption of last autosave
                string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\bin\\Debug\\";
                string[] configSR = System.IO.File.ReadAllLines(targetPath + "config_lastsession_save.txt");
                foreach (string line in configSR)
                {
                    if (line.Length != 0 && !inuse)
                    {
                        if (line == comboBox1.SelectedItem.ToString())
                        {
                            inuse = true;
                            MessageBox.Show("You used this semiring in your last session.\nPlease delete all matrices in LEC and return to SRGenerator \nfor any updates on the semiring.", "Info", MessageBoxButtons.OK);
                        }
                    }
                }

                // call used-asbasetype
                if (!inuse)
                {
                    inuse = used_asBasetype(comboBox1.SelectedItem.ToString());
                    if (inuse) MessageBox.Show("This Semiring is used as basetype. Delete no possible.\nFirst delete all semirings depending on that one!", "Info", MessageBoxButtons.OK);
                }


                if (comboBox1.SelectedItem != null && !inuse)
                {
                    deleteSR(comboBox1.SelectedItem.ToString());
                    // update comboBox1:
                    comboBox1.Items.Remove(comboBox1.SelectedItem);
                    comboBox1.Text = "";

                    //check if in datamodel
                    if (!controller.checkAllClassname(classname_box.Text))
                    {
                        //delete in model
                        controller.deletedata(classname_box.Text);
                    }

                }

            }
            else MessageBox.Show("No Semiring selected!", "Info", MessageBoxButtons.OK);

        }

        /// <summary>
        /// this method avoid deleting semiring-classes, which are used as basetypes
        /// </summary>
        /// <param name="classname"></param>
        /// <returns></returns>
        public bool used_asBasetype(string classname)
        {
            bool inuse = false;
            inuse = updateSrList("config_ASR.txt", classname);
            if (inuse) return true;
            inuse = updateSrList("config_DP.txt", classname);
            if (inuse) return true;
            inuse = updateSrList("config_euclid.txt", classname);
            if (inuse) return true;
            inuse = updateSrList("config_field.txt", classname);
            if (inuse) return true;
            inuse = updateSrList("config_modules.txt", classname);
            if (inuse) return true;

            return false;
        }

        /// <summary>
        /// searching for a basetype with this classname
        /// </summary>
        /// <param name="filename"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool updateSrList(string filename, string name)
        {
            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\";
            byte[] readAllBytes = File.ReadAllBytes(targetPath + "bin\\debug\\Analysis_Part.exe");
            Assembly assembly = Assembly.Load(readAllBytes);

            string[] configSR = System.IO.File.ReadAllLines(filename);
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {                   
                        string sr = line.Substring(1, line.Length - 2);
                        if (!sr.Contains("FiniteLattice") && !sr.Contains("LatticeBasedOnBDD") && sr!= "int" && sr != "uint" && sr != "double")
                        {
                        // not for int, uint etc.
                            try
                            {
                       
                                    Type t = assembly.GetType("Analysis_Part." + sr);
                                    FieldInfo[] info = t.GetFields(BindingFlags.NonPublic | BindingFlags.Instance);
                                    // List for used basetypes as members:                                
                                    for (int i = 0; i < info.Length; i++)
                                    {
                                        string temp= info[i].FieldType.FullName.Replace("Analysis_Part.", "");
                                        // if used as member, do not delete!
                                        if (temp == name) return true;
                                        else if (name.Contains("Polynom") && temp.Contains(name)) return true;
                                    }                                                                                      
                            }
                            catch {
                            MessageBox.Show("Config-file-Inconsistence! Inform the developer.", "Info", MessageBoxButtons.OK);
                            return false;
                        }

                        }
                }

            }

            return false;
        }




        /// <summary>
        /// Method to delete a already generated Semiring
        /// AlgorithmB must be updated!!
        /// 
        /// But first check, if uses directly as basetype for some other Semiring-> no delete allowed!!
        /// </summary>
        /// <param name="name"></param>
        private void deleteSR(string name)
        {
            string sourcePath = Directory.GetCurrentDirectory();
            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\";
       

            // remove from csproj file
            string csprojCheck = System.IO.File.ReadAllText(targetPath + "Analysis_Part.csproj");
            int index = csprojCheck.LastIndexOf("<Compile Include=\"" + name + ".cs\">");
            if (index > 0)
            {
                RemoveFilesInProject(targetPath, name + ".cs");
                try
                {
                    // rebuild windows exe
                    Compiler.Build(Directory.GetParent(Directory.GetParent(targetPath.ToString()).ToString()).ToString() + "\\PAWS.sln");
                    // Compiler.Build(Directory.GetParent(Directory.GetParent(targetPath.ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part.csproj");
                    // update available exe
                    string sourceFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "Analysis_Part.exe");
                    string destFile = System.IO.Path.Combine(sourcePath, "Analysis_Part.exe");// old

      
                    // Update of config files
                    string config = System.IO.File.ReadAllText("config_ASR.txt");
                    index = config.LastIndexOf("#" + name + "#");
                    if (index < 0 || index != 1) index = config.LastIndexOf("\n#" + name + "#");
                    int length = ("#" + name + "#").Length;
                    if (index > 0)
                    {
                        if (index > 1) config = config.Remove(index, length + 1);
                        if (index == 1) config = config.Remove(index, length);
                        File.WriteAllText("config_ASR.txt", config);
                        // copy to program AlgoB
                        sourceFile = System.IO.Path.Combine(sourcePath, "config_ASR.txt");
                        destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_ASR.txt");
                        System.IO.File.Copy(sourceFile, destFile, true);
                    }

                    //maybe its a sr with manual imple. string constructor

                    config = System.IO.File.ReadAllText("config_stringConstr.txt");
                    index = config.LastIndexOf("#" + name + "#");
                    if (index < 0 || index != 1) index = config.LastIndexOf("\n#" + name + "#");
                    length = ("#" + name + "#").Length;
                    if (index > 0)
                    {
                       if(index>1) config = config.Remove(index, length + 1);
                       if (index == 1) config = config.Remove(index, length);
                        File.WriteAllText("config_stringConstr.txt", config);
                        // copy to program AlgoB
                        sourceFile = System.IO.Path.Combine(sourcePath, "config_stringConstr.txt");
                        destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_stringConstr.txt");
                        System.IO.File.Copy(sourceFile, destFile, true);
                    }


                    //maybe its a module

                    config = System.IO.File.ReadAllText("config_modules.txt");
                    index = config.LastIndexOf("#" + name + "#");
                    if (index < 0 || index != 1) index = config.LastIndexOf("\n#" + name + "#");
                    length = ("#" + name + "#").Length;
                    if (index > 0)
                    {
                        if (index > 1) config = config.Remove(index, length + 1);
                        if (index == 1) config = config.Remove(index, length);
                        File.WriteAllText("config_modules.txt", config);
                    }

                    //maybe its a field
                    config = System.IO.File.ReadAllText("config_field.txt");
                    index = config.LastIndexOf("#" + name + "#");
                    if (index < 0 || index !=1) index = config.LastIndexOf("\n#" + name + "#");
                    length = ("#" + name + "#").Length;
                    if (index > 0)
                    {
                        if (index > 1) config = config.Remove(index, length + 1);
                        if (index == 1) config = config.Remove(index, length);
                        File.WriteAllText("config_field.txt", config);
                        // copy to program AlgoB
                        sourceFile = System.IO.Path.Combine(sourcePath, "config_field.txt");
                        destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_field.txt");
                        System.IO.File.Copy(sourceFile, destFile, true);

                    }

                    //maybe its a DirectProduct
                    config = System.IO.File.ReadAllText("config_DP.txt");
                    index = config.LastIndexOf("#" + name + "#");
                    if (index < 0 || index != 1) index = config.LastIndexOf("\n#" + name + "#");
                    length = ("#" + name + "#").Length;
                    if (index > 0)
                    {
                        if (index > 1) config = config.Remove(index, length + 1);
                        if (index == 1) config = config.Remove(index, length);
                        File.WriteAllText("config_DP.txt", config);
                        // copy to program AlgoB
                        sourceFile = System.IO.Path.Combine(sourcePath, "config_DP.txt");
                        destFile = System.IO.Path.Combine(targetPath + "bin\\Debug\\", "config_DP.txt");
                        System.IO.File.Copy(sourceFile, destFile, true);

                    }

                    // update gui-selection
                    updateComboboxes(name);
                    // delete from folders
                    // Delete a file by using File class static method...
                    if (System.IO.File.Exists(targetPath + name + ".cs"))
                    {

                        try
                        {
                            System.IO.File.Delete(targetPath + name + ".cs");
                            // should not be the case:
                            System.IO.File.Delete(sourcePath + "\\" + name + ".cs");
                        }
                        catch (FileNotFoundException fnf)
                        {

                        }
                    }
                }
                catch (Exception e1)
                {
                    IncludeFilesInProject(targetPath, name + ".cs");
                    MessageBox.Show(e1.ToString(), "Error", MessageBoxButtons.OK);
                }
            }

        }

        /// <summary>
        /// deleting or adding an compiles sr:
        /// update the current comboboxes
        /// </summary>
        public void updateComboboxes(string classname)
        {
            //comboBox1
            if (getGuiPanel() == Constants.ASR) ((ArbitrarySRPanel)x).updateAttributes();
            if (getGuiPanel() == Constants.FR) ((FieldOfFractionPanel)x).updateAttributes();
            if (getGuiPanel() == Constants.LM) ((LMonoidPanel)x).updateAttributes();
            if (getGuiPanel() == Constants.DP) ((DirectProductPanel)x).updateAttributes();

            // main panel
            if (comboBox1.Items.Contains(classname))
            {
                comboBox1.Items.Remove(classname);
            }
            else { comboBox1.Items.Add(classname); }


        }


        /// <summary>
        /// Adds the code specified by the user for selected method/operator/constructor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void addUsercode_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(sourceBox.Text) && sourceBox.Enabled)
            {
                // get the selected method/operator/constructor
                System.Windows.Forms.Control active = ((ArbitrarySRPanel)x).getActiveOne();
                // add the code
                if (active != null) ((ArbitrarySRPanel)x).writeMethod(sourceBox.Text, active.Name, active);
                ((ArbitrarySRPanel)x).activateAll();
            }
            else
            {
               if(String.IsNullOrEmpty(sourceBox.Text)) MessageBox.Show("No sourcecode defined!", "Info", MessageBoxButtons.OK);
               else MessageBox.Show("First activate sourcebox!", "Info", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Activate all other elements, called by cancel button (right side gui)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cancel_Click(object sender, EventArgs e)
        {
            //delete sourceCode
            sourceBox.Clear();
            // call corresponding activate ALL
            string gui = getGuiPanel();
            switch (gui)
            {
                case Constants.ASR:
                    ((ArbitrarySRPanel)x).activateAll();
                    break;
                case Constants.LM:
                    ((LMonoidPanel)x).activateAll();
                    break;
                case Constants.FR:
                    ((FieldOfFractionPanel)x).activateAll();
                    break;
            }

        }


        /// <summary>
        /// Classnames of Algorithm B are forbidden!!!
        /// TODO update
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool checkConstantClassname(string name)
        {
            if (name == "Matrix") return false;
            if (name == "Polynom") return false;
            if (name == "LinearCombination") return false;
            if (name == "Preorder") return false;
            if (name == "PreorderModel") return false;
            if (name == "BDDModel") return false;
            if (name == "FiniteLattice") return false;
            if (name == "DataController") return false;
            if (name == "Rational") return false;
            if (name == "DataManager") return false;
            if (name == "Permutation") return false;
            if (name == "Semiring") return false;
            if (name == "Modulo") return false;
            if (name == "Form2") return false;
            if (name == "linenum_rtf") return false;
            if (name.Contains("Modulo-")) return false;
            if (name.Contains("-FiniteLattice")) return false;
            // check if semiring is used in last session
            // to avoid corruption of last autosave
            string targetPath = Directory.GetParent(Directory.GetParent(Directory.GetParent(Directory.GetCurrentDirectory()).ToString()).ToString()).ToString() + "\\Analysis_Part\\Analysis_Part\\bin\\Debug\\";
            string[] configSR = System.IO.File.ReadAllLines(targetPath+"config_lastsession_save.txt");
            foreach (string line in configSR)
            {
                if (line.Length != 0)
                {
                    if (line == name)
                    {
                        MessageBox.Show("You used this semiring in your last session.\nPlease delete all matrices in LEC and return to SRGenerator \nfor any updates on the semiring.", "Info", MessageBoxButtons.OK);
                        return false;
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// check if name is already used
        /// </summary>
        public void resetOrOverride()
        {
            if (!Char.IsUpper(classname_box.Text[0])) classname_box.Text = CultureInfo.CurrentCulture.TextInfo.ToTitleCase(classname_box.Text);
            // check if name is free

            bool inuse = false;
            // call used-asbasetype
            if (!inuse)
            {
                inuse = used_asBasetype(classname_box.Text);
                if (inuse) MessageBox.Show("This Semiring is used as basetype. Delete no possible.\nFirst delete all semirings depending on that one!", "Info", MessageBoxButtons.OK);
            }

            // no classnames of AlgorithmB Programm or saved in last session
            if (checkConstantClassname(classname_box.Text)&& !inuse)
            {
                if (!controller.checkAllClassname(classname_box.Text) || !classNotExists(classname_box.Text))
                {
                    if (!classNotExists(classname_box.Text))
                    {

                        Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.MsgBoxResult.Yes;
                        response = Microsoft.VisualBasic.Interaction.MsgBox("Do you want to override the inputs for " + classname_box.Text + ".cs ? ", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                        if (response == Microsoft.VisualBasic.MsgBoxResult.Yes)
                        {
                            // delete and rebuild method update
                            deleteSR(classname_box.Text);

                            //check if in datamodel
                            if (!controller.checkAllClassname(classname_box.Text))
                            {

                                //delete in model
                                controller.deletedata(classname_box.Text);
                                classname = classname_box.Text;
                                current.Text = classname;
                                if (!classname_box.Items.Contains(classname_box.Text))
                                {
                                    classname_box.Items.Add(classname_box.Text);
                                }
                                else { classname_box.Items.Remove(classname_box.Text); }

                            }
                            else
                            {
                                classname = classname_box.Text;
                                current.Text = classname;
                            }

                        }
                        else
                        {
                            classname_box.Text = "";
                        }


                    }// not a class yet
                    else
                    {
                        Microsoft.VisualBasic.MsgBoxResult response = Microsoft.VisualBasic.MsgBoxResult.Yes;
                        //check if any datamodel is using this name
                        if (!controller.checkAllClassname(classname_box.Text))
                        {
                            response = Microsoft.VisualBasic.Interaction.MsgBox("Do you want to override the inputs for " + classname_box.Text + "? ", Microsoft.VisualBasic.MsgBoxStyle.YesNo, "Override");
                            if (response == Microsoft.VisualBasic.MsgBoxResult.Yes)
                            {
                                //delete in model
                                controller.deletedata(classname_box.Text);
                                classname = classname_box.Text;
                                current.Text = classname;
                                //delete in gui
                                if (!classname_box.Items.Contains(classname_box.Text))
                                {
                                    classname_box.Items.Add(classname_box.Text);
                                }
                                else { classname_box.Items.Remove(classname_box.Text); }
                            }
                            else
                            {
                                classname_box.Text = "";
                            }
                        }

                    }


                }
                else// is free:
                {
                    classname = classname_box.Text;
                    current.Text = classname;
                }
                if (getGuiPanel() != "")
                {
                    resetPanel(getGuiPanel());
                }
            }// no constant names allowed and other dependencies

        }

        /// <summary>
        /// to reset the current panel
        /// </summary>
        /// <param name="panel"></param>

        public void resetPanel(string panel)
        {
            attributes = new List<string>();
            switch (panel)
            {
                case Constants.DP:

                    x = new DirectProductPanel(attributes, classname);
                    x.Location = new System.Drawing.Point(0, 0);
                    cn1.setView((IView1)x);
                    break;
                case Constants.FR:

                    methods = new Dictionary<string, string>();
                    x = new FieldOfFractionPanel(attributes, methods, sourceBox, classname, textBox1);
                    x.Location = new System.Drawing.Point(0, 0);
                    cn1.setView((IView1)x);
                    break;
                case Constants.FE:
                    x = new FieldExtPanel(sourceBox, classname, textBox1);
                    x.Location = new System.Drawing.Point(0, 0);
                    cn1.setView((IView1)x);
                    break;
                case Constants.ASR:
                    freemethods = new Dictionary<string, string>();
                    optional = new Dictionary<string, string>();
                    optionalinfo = new Dictionary<string, List<string>>();
                    x = new ArbitrarySRPanel(attributes, freemethods, optional, optionalinfo, classname, sourceBox, true, false, false, false, textBox1);
                    x.Location = new System.Drawing.Point(0, 0);
                    cn2.setView((IView2)x);
                    break;
                case Constants.LM:
                    methods = new Dictionary<string, string>();
                    x = new LMonoidPanel(attributes, methods, classname, sourceBox, textBox1);
                    cn1.setView((IView1)x);
                    cn1.setModel(((Controller)controller).getModel(Constants.LM));
                    break;
            }


            x.Parent = panel2;
            x.Size = x.Parent.Size;
            x.TabIndex = 0;
            x.Anchor = (AnchorStyles.Top | AnchorStyles.Right);
            panel2.Controls.Clear();
            panel2.Controls.Add(x);
            panel2.ResumeLayout(false);
            panel2.PerformLayout();
            panel2.Visible = true;

        }


        /// <summary>
        /// get the current active gui-inputmask
        /// </summary>
        /// <returns></returns>
        public string getGuiPanel()
        {
            if (semiauto_sr.CheckedItems.Count > 0) return semiauto_sr.SelectedItem.ToString();
            if (fullauto_sr.CheckedItems.Count > 0) return fullauto_sr.SelectedItem.ToString();
            return "";
        }


        /// <summary>
        /// this method is for setting a new classname,
        /// but before the user could start to specify the class-structure and behavoiur
        /// first this method is calling a classname-is in use- checking method
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void setName_Click(object sender, EventArgs e)
        {
            bool nosharp = true;
            nosharp = classname_box.Text.Contains("#");

            if (String.IsNullOrEmpty(classname_box.Text) || nosharp)
            {
                MessageBox.Show("Please enter a name (without #) for the Semiring or select the corresponding template-panel", "Error", MessageBoxButtons.OK);
            }
            else
            {
                resetOrOverride();              
            }

        }

        /// <summary>
        /// for selection of any semiring in work
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void classname_box_SelectedIndexChanged(object sender, EventArgs e)
        {
            // update to gui panel for corresponding classname
            classname = classname_box.SelectedText;
            classname = classname_box.Text;
            current.Text = classname;
            if (semiauto_sr.CheckedItems.Count != 0)
            {
                semiauto_sr.SetItemCheckState(semiauto_sr.SelectedIndex, CheckState.Unchecked);
                semiauto_sr.SelectedIndex = -1;
            }
            if (fullauto_sr.CheckedItems.Count != 0)
            {
                fullauto_sr.SetItemCheckState(fullauto_sr.SelectedIndex, CheckState.Unchecked);
                fullauto_sr.SelectedIndex = -1;
            }

            // select gui panel
            string guipanel = ((Controller)controller).getModelbyClassname(classname);
            if (guipanel == Constants.DP)
            {
                fullauto_sr.SetItemCheckState(0, CheckState.Checked);
                fullauto_sr.SelectedIndex = 0;
            }
            if (guipanel == Constants.FE)
            {
                fullauto_sr.SetItemCheckState(2, CheckState.Checked);
                fullauto_sr.SelectedIndex = 2;
            }

            if (guipanel == Constants.FR)
            {
                fullauto_sr.SetItemCheckState(1, CheckState.Checked);
                fullauto_sr.SelectedIndex = 1;
            }

            if (guipanel == Constants.ASR)
            {
                semiauto_sr.SetItemCheckState(1, CheckState.Checked);
                semiauto_sr.SelectedIndex = 1;
            }
            if (guipanel == Constants.LM)
            {
                semiauto_sr.SetItemCheckState(1, CheckState.Checked);
                semiauto_sr.SelectedIndex = 0;
            }
        }


        // msg from models
        public void viewListchanged(IModel model, ViewEventArgs e)
        {
            if (!classname_box.Items.Contains(e.name))
            {
                classname_box.Items.Add(e.name);
            }
            else { classname_box.Items.Remove(e.name); }
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                SaveFileDialog saveFileDialog1 = new SaveFileDialog();
                saveFileDialog1.Title = "Save your semirings.";
                saveFileDialog1.ShowDialog();
                if (saveFileDialog1.FileName != "")
                {
                    controller.saveState(saveFileDialog1.FileName);
                }
            }

            catch (Exception)
            {
                MessageBox.Show("Some error occurs.");
            }
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.InitialDirectory = "c:\\";
            openFileDialog1.FilterIndex = 2;
            openFileDialog1.RestoreDirectory = true;

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    if ((openFileDialog1.OpenFile()) != null)
                    {
                        string filename = openFileDialog1.FileName;
                        IFormatter formatter = new BinaryFormatter();

                        try
                        {
                            Stream fromstream = new FileStream(filename,
                                          FileMode.Open,
                                          FileAccess.Read,
                                          FileShare.Read);
                            try
                            {
                                controller.setModels((Dictionary<string, DataModel>)formatter.Deserialize(fromstream));
                                // update class list of last session:
                                classname_box.Items.Clear();
                                updateListfromAutosave(Constants.ASR);
                                updateListfromAutosave(Constants.FR);
                                updateListfromAutosave(Constants.DP);
                                updateListfromAutosave(Constants.LM);
                            }
                            catch (Exception e0) { MessageBox.Show("Error: Could not read file from disk. Original error: " + e0.Message); }

                            fromstream.Close();
                            //updateAfterFileloaded();
                        }
                        catch { }
                    }

                }

                catch (Exception ex)
                {
                    MessageBox.Show("Error: Could not read file from disk. Original error: " + ex.Message);
                }
            }
        }

 
    }//end class



    /// <summary>
    /// Event handling of the GUI- Form1
    /// </summary>
    /// <typeparam name="IView"></typeparam>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    public delegate List<string> ViewHandler0<IView>(IView sender, ViewEventArgs e);
    public delegate Dictionary<string, string> ViewHandlerDict<IView>(IView sender, ViewEventArgs e);
    public delegate Dictionary<string, List<String>> ViewHandlerDictList<IView>(IView sender, ViewEventArgs e);
    public delegate bool ViewHandlerBool<IView>(IView sender, ViewEventArgs e);

    public delegate void ViewHandler1<IView>(IView sender, ViewEventUpdateData e);
    public delegate void ViewHandler2<IView>(IView sender, ViewEventUpdateDataFree e);

    public class ViewEventArgs : EventArgs
    {
        public String name = " ";
        public String semi = " ";


        public ViewEventArgs()
        {
        }
        public ViewEventArgs(String s, String m)
        {
            this.name = m;
            this.semi = s;
        }

    }
    public class ViewEventUpdateData : ViewEventArgs
    {
        public List<string> member;
        public Dictionary<string, string> methods;

        public ViewEventUpdateData(string name, string type, List<string> list1, Dictionary<string, string> list2)
        {

            this.name = name;
            this.semi = type;
            this.member = list1;
            this.methods = list2;
        }

    }

    public class ViewEventUpdateDataFree : ViewEventArgs
    {

        public List<string> attributes;
        public Dictionary<string, string> freemethods;
        public Dictionary<string, string> optional;
        public Dictionary<string, List<string>> optionalinfo;
        public bool modul;
        public bool sr;
        public bool field;
        public bool euclid;
        public string method_info;

        public ViewEventUpdateDataFree(string type, string name, string info)
            : base(type, name)
        {
            this.method_info = info;
        }

        public ViewEventUpdateDataFree(string type, string name, List<string> attributes, Dictionary<string, string> freemethods, Dictionary<string, string> optional, Dictionary<string, List<string>> optionalinfo, bool modul, bool semiring, bool field, bool euclid)
        {
            this.semi = type;
            this.name = name;
            this.attributes = attributes;
            this.freemethods = freemethods;
            this.optional = optional;
            this.optionalinfo = optionalinfo;
            this.modul = modul;
            this.sr = semiring;
            this.field = field;
            this.euclid = euclid;

        }


    }

    public interface IView
    {

        event ViewHandler0<IView> getDataList;
        event ViewHandlerDict<IView> getDataDict;
        event ViewHandlerDictList<IView> getDataDictList;
        event ViewHandlerBool<IView> getBool;
        void setController(IController cont);
    }
    public interface IView1
    {
        event ViewHandler1<IView1> saveData1;

        void setController(IController1 cont);
    }

    public interface IView2
    {
        event ViewHandler2<IView2> saveData2;
        void setController(IController2 cont);
    }

}
