﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.CodeDom;


namespace Generator_2._0
{   
    /// <summary>
    ///  This class represents a template for DirectProduct Code-Generation
    ///  based on building a CodeDom -Diagram
    /// </summary>
    class GeneratorDirectProduct:Generator
    {   
        public GeneratorDirectProduct(String classname, string t)
            : base(classname, t)
        {
        }

        /// <summary>
        /// This method is generating the DirectProduct for the corresponding semirings,
        /// selected by the user via GUI.
        /// </summary>
        /// <param name="semirings"></param>
        public bool generateDirectProduct(List<string> semirings)
        {
            //1.Part Attribtutes:
            // prepare types
            Dictionary<int, string> finite_SR = finiteSRInfo(semirings);
            generateAttributes(semirings);
            if (basetypes.Count != semirings.Count) return false;
            // 2. Part generation of constructors

            // Type as parameters:
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            targetClass.Members.Add(constructor);
            CodeConstructor constructor1 = addConstructorByAttri(semirings);
            targetClass.Members.Add(constructor1);
            CodeConstructor constructor2 = addConstructorNewClass(semirings, targetClass.Name);
            targetClass.Members.Add(constructor2);
            CodeConstructor constructor3 = generateStringConstructor(semirings, finite_SR);
            targetClass.Members.Add(constructor3);


            /* 3. Part  Define the necessary operators:
             */
            // + & * operators
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operators +", "operator +", targetClass.Name, getDPMethodStatements("+", semirings.Count + 1 + finite_SR.Count, 0, semirings, finite_SR, "", "","","")));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operators *", "operator *", targetClass.Name, getDPMethodStatements("*", semirings.Count + 1 + finite_SR.Count, 0, semirings, finite_SR, "", "", "", "")));
            // == operator
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operators ==", "operator ==", "System.Boolean", getDPMethodStatements("==",semirings.Count + 1 + finite_SR.Count, 0, semirings, finite_SR, "return true","if(a.mem" , "!=b.mem" , ")return false")));
            // != operatpor
            CodeStatement[] st = new CodeStatement[1];
            CodeExpressionStatement statement1 = new CodeExpressionStatement(new CodeVariableReferenceExpression("return !(a==b)"));
            st[0] = statement1;
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operators !=", "operator !=", "System.Boolean", st));


            /* Methods
             * One & Zero
             * ToString()
             * */
            st = new CodeStatement[semirings.Count + 1 + finite_SR.Count];
            st = getDPMethodStatements("One", semirings.Count + 1 + finite_SR.Count, 0, semirings, finite_SR, "", "", "", "");
            targetClass.Members.Add(addNecessaryOperators("", "Yields the One elements", "One", targetClass.Name,st));
            st = getDPMethodStatements("Zero", semirings.Count + 1 + finite_SR.Count, 0, semirings, finite_SR, "", "", "", "");
            targetClass.Members.Add(addNecessaryOperators("", "Yields the Zero elements", "Zero", targetClass.Name, st));

            // ToString()
            st = getDPMethodStatements("ToString", semirings.Count + 1 + finite_SR.Count, 1, semirings, finite_SR, "", "", "", "");
            targetClass.Members.Add(addNecessaryOperators("System.String", "returns the string representation", "ToString", "System.String", st));
            
            // next to generate componentwise:
            //public static LinearCombination<...> LinearCombination(List<...[]> set, ...[] vec)

            CodeMemberMethod findlinear = new CodeMemberMethod();
            findlinear.Comments.Add(new CodeCommentStatement(
            "Check whether a linear combination exists, working componentwise"));
            findlinear.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            findlinear.Name = "LinearCombination";
            findlinear.ReturnType = new CodeTypeReference("LinearCombination<" + targetClass.Name + ">");
            findlinear.Parameters.Add(new CodeParameterDeclarationExpression("List<" + targetClass.Name + "[]> ", "set"));
            findlinear.Parameters.Add(new CodeParameterDeclarationExpression(targetClass.Name + "[]", "vec"));
            CodeVariableDeclarationStatement lc = new CodeVariableDeclarationStatement("LinearCombination<" + targetClass.Name + ">", "lc");
            findlinear.Statements.Add(lc);
            CodeVariableReferenceExpression variableRef1 = new CodeVariableReferenceExpression("lc.CombinationExists = true");
            CodeVariableReferenceExpression variableRef2 = new CodeVariableReferenceExpression("lc.Multiplicands = new " + targetClass.Name + "[set.Count]");
            findlinear.Statements.Add(variableRef1);
            findlinear.Statements.Add(variableRef2);
            // 1.Part vec and Lists componentwise

            for (int i = 0; i < semirings.Count; i++)
            {
                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression(semirings[i] + " [] vec" + i + " = new " + semirings[i] + "[vec.Length]"));
                findlinear.Statements.Add(st1);
                CodeExpressionStatement st2 = new CodeExpressionStatement(new CodeVariableReferenceExpression("List<" + semirings[i] + "[]> set" + i + " = new List<" + semirings[i] + "[]>()"));
                findlinear.Statements.Add(st2);
            }
            //t0[j]=vec[i].mem0;
            st = new CodeStatement[semirings.Count];
            for (int i = 0; i < semirings.Count; i++)
            {
                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression("vec" + i + " [i] = vec[i].mem" + i));
                st[i] = st1;
            }
            findlinear.Statements.Add(addForLoopStatements("vec.Length", "i", 0, st));

            // lists from list
            CodeStatement[] stAll = new CodeStatement[semirings.Count * 2 + 1];
            for (int i = 0; i < semirings.Count; i++)
            {
                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression(semirings[i] + " [] t" + i + " = new " + semirings[i] + "[set[0].Length]"));
                stAll[i] = st1;
            }
            st = new CodeStatement[semirings.Count];
            for (int i = 0; i < semirings.Count; i++)
            {
                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression("t" + i + " [j] = set[i][j].mem" + i));
                st[i] = st1;
            }
            CodeIterationStatement inner1 = addForLoopStatements("set[0].Length", "j", 0, st);
            stAll[semirings.Count] = inner1;
            //    set0.Add(t0);
            for (int i = semirings.Count + 1; i < semirings.Count * 2 + 1; i++)
            {
                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression("set" + (i - semirings.Count - 1) + ".Add(t" + (i - semirings.Count - 1) + ")"));
                stAll[i] = st1;
            }
            CodeIterationStatement outside = addForLoopStatements("set.Count", "i", 0, stAll);
            findlinear.Statements.Add(outside);



            string lcmulti = "lc.Multiplicands[i]= new " + targetClass.Name + "(";
            for (int i = 0; i < semirings.Count; i++)
            {
                //  if(!Type0.LinearCombination(set0,vec0)   lc.CombinationExists = false;
                if (finite_SR.Keys.Contains(i))
                {
                    string loadType = "";
                    if (semirings[i] == "Modulo") loadType = "Modulo.modulo =" + finite_SR[i];
                    if (semirings[i] == "FiniteLattice") loadType = "PreorderModel.getInstance.loadPreorder(\"" + finite_SR[i] + "\")";
                    if (semirings[i] == "LatticeBasedOnBDD") loadType = "BDDModel.getInstance.loadBDD(\"" + finite_SR[i] + "\")";
                    CodeExpressionStatement lt = new CodeExpressionStatement(new CodeVariableReferenceExpression(loadType));
                    findlinear.Statements.Add(lt);
                }



                CodeExpressionStatement st1 = new CodeExpressionStatement(new CodeVariableReferenceExpression("LinearCombination<" + semirings[i] + "> lc" + i + "= " + semirings[i] + ".LinearCombination(set" + (i) + ",vec" + (i) + ")"));
                CodeExpressionStatement st2 = new CodeExpressionStatement(new CodeVariableReferenceExpression("if(! lc" + (i) + ".CombinationExists) lc.CombinationExists = false"));
                findlinear.Statements.Add(st1);
                findlinear.Statements.Add(st2);

                // 1. part inner statements:  lc.Multiplicant[i]= new Class(lc0.Multiplicant[i],lc1...)
                if ((i) != semirings.Count - 1) lcmulti += "lc" + (i) + ".Multiplicands[i],";
                else lcmulti += "lc" + (i) + ".Multiplicands[i])";
            }

            //2.part for 
            st = new CodeStatement[1];
            st[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression(lcmulti));
            CodeIterationStatement lastfor = addForLoopStatements("lc.Multiplicands.Length", "i", 0, st);
            //3.part if 
            CodeExpression condition = new CodeVariableReferenceExpression("lc.CombinationExists");
            CodeConditionStatement lastIf = new CodeConditionStatement(condition, lastfor);
            findlinear.Statements.Add(lastIf);

            CodeExpressionStatement returnlc = new CodeExpressionStatement(new CodeVariableReferenceExpression("return lc"));
            findlinear.Statements.Add(returnlc);

            targetClass.Members.Add(findlinear);


            // allowedString
            targetClass.Members.Add(addAllowedString(semirings, finite_SR));

            return true;

        }


        /// <summary>
        /// For every semiring of the DirectProduct to create, the corresponding operation
        /// is created componentwise based on the single semiring operation.
        /// 
        /// Ofcourse for Modulo and FiniteLattice the static 'value' or 'Preorder' must be updated.
        /// </summary>
        /// <param name="stsize"></param>
        /// <param name="update"></param>
        /// <param name="attributes"></param>
        /// <param name="finite_SR"></param>
        /// <returns></returns>
        public CodeStatement[] getDPMethodStatements(string op,int stsize, int update, List<string> attributes, Dictionary<int, string> finite_SR, string expr1, string expr2a, string expr2b, string expr2c)
        {
            CodeStatement[] st = new CodeStatement[stsize];

            string returnexpression = "return new " + targetClass.Name + "(";
            if (op == "ToString")
            {
                returnexpression = "string ret =\"\"";
                st = new CodeStatement[attributes.Count + 2 + finite_SR.Count];
                st[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression(returnexpression));
                returnexpression = "return ret+=";
                for (int i = 1; i < attributes.Count + finite_SR.Count + 1; i++)
                {
                    if (finite_SR.Keys.Contains(i - update))
                    {
                        string loadType = "";
                        if (attributes[i - update] == "Modulo") loadType = "Modulo.modulo =" + finite_SR[i - update];
                        if (attributes[i - update] == "FiniteLattice") loadType = "PreorderModel.getInstance.loadPreorder(\"" + finite_SR[i - update] + "\")";
                        if (attributes[i - update] == "LatticeBasedOnBDD") loadType = "BDDModel.getInstance.loadBDD(\"" + finite_SR[i - update] + "\")";
                        CodeExpressionStatement lt = new CodeExpressionStatement(new CodeVariableReferenceExpression(loadType));
                        st[i] = lt;
                        i++;
                        update++;
                    }

                    st[i] = new CodeExpressionStatement(new CodeVariableReferenceExpression("string ret" + (i - update) + "= mem" + (i - update) + ".ToString()"));
                    if (i-update == 0) returnexpression += "ret" + (i - update) + "+\",\"";
                    else if (i < attributes.Count + finite_SR.Count) returnexpression += "+ret" + (i - update) + "+\",\"";
                    if (i == attributes.Count + finite_SR.Count) returnexpression += "+ret" + (i - update);
                }
                CodeExpressionStatement returnState = new CodeExpressionStatement(new CodeVariableReferenceExpression(returnexpression));
                st[attributes.Count + 1 + finite_SR.Count] = returnState;
            }
            else
            {
                for (int i = 0; i < attributes.Count + finite_SR.Count; i++)
                {
                    if (finite_SR.Keys.Contains(i - update))
                    {
                        string loadType = "";
                        if (attributes[i - update] == "Modulo") loadType = "Modulo.modulo =" + finite_SR[i - update];
                        if (attributes[i - update] == "FiniteLattice") loadType = "PreorderModel.getInstance.loadPreorder(\"" + finite_SR[i - update] + "\")";
                        if (attributes[i - update] == "LatticeBasedOnBDD") loadType = "BDDModel.getInstance.loadBDD(\"" + finite_SR[i - update] + "\")";
                        CodeExpressionStatement lt = new CodeExpressionStatement(new CodeVariableReferenceExpression(loadType));
                        st[i] = lt;
                        i++;
                        update++;
                    }
                    if (op == "==") st[i] = new CodeExpressionStatement(new CodeVariableReferenceExpression(expr2a + (i - update) + expr2b + (i - update) + expr2c));
                    if (op == "+" || op == "*")
                    {
                        st[i] = new CodeExpressionStatement(new CodeVariableReferenceExpression(attributes[i - update] + " s" + (i - update) + "= a.mem" + (i - update) + op + " b.mem" + (i - update)));
                        if (i - update != attributes.Count - 1) returnexpression += "s" + (i - update) + ",";
                        else returnexpression += "s" + (i - update) + ")";
                    }

                    if (op == "One" || op == "Zero")
                    {
                        st[i] = new CodeExpressionStatement(new CodeVariableReferenceExpression(attributes[(i - update)] + " s" + (i - update) + "= " + attributes[(i - update)] + "." + op + "()"));
                        if (i - update != attributes.Count - 1) returnexpression += "s" + (i - update) + ",";
                        else returnexpression += "s" + (i - update) + ")";
                    }

                }
                if (op == "==") st[attributes.Count + finite_SR.Count] = new CodeExpressionStatement(new CodeVariableReferenceExpression(expr1));
                if (op == "+" || op == "*" || op == "One" || op == "Zero") st[attributes.Count + finite_SR.Count] = new CodeExpressionStatement(new CodeVariableReferenceExpression(returnexpression));

            }

            return st;
        }


        /*
        /// <summary>
         *  Old solution:
        /// For the string constructor, the input for creating a Matrix<T> needs to be verified,
        /// for better overview of the CodeDom based code, this is seperated from the String-Constructor
        /// and it also should happen before creating the single values  for a Matrix<T>. This is more easy to handle 
        /// directly by the MVC.
        /// 
        /// Ofcourse for Modulo and FiniteLattice the static 'value' or 'Preorder' must be updated.
        /// </summary>
        /// <param name="t"></param>
        /// <returns></returns>
        public CodeMemberMethod addAllowedString(List<string> t, Dictionary<int, string> finite_SR)
        {
            CodeMemberMethod allowed = new CodeMemberMethod();
            allowed.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            allowed.Parameters.Add(new CodeParameterDeclarationExpression(typeof(System.String), "str"));
            allowed.Name = "allowedStrings";
            allowed.ReturnType = new CodeTypeReference("System.Boolean");

            CodeVariableReferenceExpression con1 = new CodeVariableReferenceExpression("string temp=\"\"");
            CodeVariableReferenceExpression con2 = new CodeVariableReferenceExpression("int l=str.Length");
            CodeVariableReferenceExpression con3 = new CodeVariableReferenceExpression("int member=0");
            CodeVariableReferenceExpression con4 = new CodeVariableReferenceExpression("bool ret=true");
            allowed.Statements.Add(con1); allowed.Statements.Add(con2); allowed.Statements.Add(con3); allowed.Statements.Add(con4);

            // 
            CodeStatement []insideloop = new CodeStatement[2];
            // first if:   temp+=str[i];
            insideloop[0] = addIF(new CodeExpressionStatement(new CodeVariableReferenceExpression("temp+=str[i]")), "str[i]", "','", CodeBinaryOperatorType.IdentityInequality);

            // sec if: 
            CodeStatement[] insideSecIf = new CodeStatement[2 + t.Count() + finite_SR.Count];
            int index=0;
            for (int i = 0; i < t.Count() + finite_SR.Count; i++)
            {
                if(finite_SR.ContainsKey(i-index))
                {
                    string loadType = "";
                    if (t[i - index] == "Modulo") loadType = "Modulo.modulo =" + finite_SR[i - index];
                    if (t[i - index] == "FiniteLattice") loadType = "PreorderModel.getInstance.loadPreorder(\"" + finite_SR[i - index] + "\")";
                    insideSecIf[i]= new CodeExpressionStatement(new CodeVariableReferenceExpression(loadType));
                    index++;
                    i++;
                }
                insideSecIf[i]=new CodeExpressionStatement(new CodeVariableReferenceExpression("  if(member==" + (i-index) + ") ret = " + t[i-index] + ".allowedStrings(temp)"));

            }
            insideSecIf[t.Count() + finite_SR.Count] = new CodeExpressionStatement(new CodeVariableReferenceExpression("temp=\"\""));
            insideSecIf[1 + t.Count() + finite_SR.Count] = new CodeExpressionStatement(new CodeVariableReferenceExpression("member++"));


            insideloop[1] = addIF(insideSecIf, "str[i]", "','", "i", "l-1", CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.IdentityEquality, CodeBinaryOperatorType.BooleanOr);
            allowed.Statements.Add(addForLoopStatements("l","i",0,insideloop));

            CodeExpressionStatement con5 = new CodeExpressionStatement(new CodeVariableReferenceExpression(" return ret"));
            allowed.Statements.Add(con5);
            return allowed;
        }
         * */
    }
}
