﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.CodeDom;
using System.CodeDom.Compiler;

namespace Generator_2._0
{
    /// <summary>
    /// this class generates field-extensions based on sqrt-2
    /// </summary>
    class GeneratorFieldSqrt:Generator
    {
        
        public GeneratorFieldSqrt(String classname, string t)
            : base(classname, t)
        {
        }

        public  void generateSR(int sqrt_base)
        {


            // attributes: c= a+b*sqrt(value)
            CodeMemberField a = new CodeMemberField();
            a.Attributes = MemberAttributes.Private;
            a.Name = "mem0";
            a.Type = new CodeTypeReference("Rational");
            targetClass.Members.Add(a);
            CodeMemberField b = new CodeMemberField();
            b.Attributes = MemberAttributes.Private;
            b.Name = "mem1";
            b.Type = new CodeTypeReference("Rational");
            targetClass.Members.Add(b);
            CodeMemberField c = new CodeMemberField();
            c.Attributes = MemberAttributes.Private | MemberAttributes.Static;
            c.Name = "mem2";
            c.Type = new CodeTypeReference("System.Double");
            c.InitExpression = new CodeMethodInvokeExpression(new CodeVariableReferenceExpression("Math"), "Sqrt", new CodePrimitiveExpression (sqrt_base));
            targetClass.Members.Add(c);
            basetypes = new List<Type>();
            basetypes.Add(getType("Rational"));
            basetypes.Add(getType("Rational"));
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            targetClass.Members.Add(constructor);
            List<string> attributes = new List<string>();
            attributes.Add("Rational");
            attributes.Add("Rational");
            //Constructor parameter NewSR x
            targetClass.Members.Add(addConstructorNewClass(attributes, targetClass.Name));

            //Constructor parameter Rat a and Rat b
            targetClass.Members.Add(addConstructorByAttri(attributes));

            //Constructor parameter String str
            targetClass.Members.Add(generateStringConstructor(attributes, new Dictionary<int, string>()));
            // necessary operators:
            // Addition
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : +", "operator +", targetClass.Name,generateAddition()));
            // Multiplication
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : *", "operator *", targetClass.Name, generateMul(sqrt_base)));
            // ==
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : ==", "operator ==", "System.Boolean", generateEqual()));

            // add the (!=) operator, based on negation of (==)
            CodeStatement[] st = new CodeStatement[1];
            st[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("return !(a==b)"));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : !=", "operator !=", "System.Boolean", st));        
            // Subtraction
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : -", "operator -", targetClass.Name, generateSubtraction()));
            // Division
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : /", "operator /", targetClass.Name, generateDiv(sqrt_base)));

            // One and Zero
            attributes.RemoveAt(1);
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the Zero Element:", "Zero", targetClass.Name, getMethod("Zero")));
            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the One Element:", "One", targetClass.Name, getMethod("One")));
            // ToString
            targetClass.Members.Add(getToString());
            // LinearCombination
            attributes.Add("Rational");
            getGauss(attributes, targetClass);
            targetClass.Members.Add(addAllowedString(attributes,new Dictionary<int,string>()));

        }

        public CodeStatement [] generateAddition()
        {
       
            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"),"mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");
            CodeStatement [] st = new CodeStatement[2];
            CodeObjectCreateExpression oce = new CodeObjectCreateExpression(targetClass.Name, new CodeExpression[] { new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Add, a1), new CodeBinaryOperatorExpression(b, CodeBinaryOperatorType.Add, b1) });
            st[0] = new CodeVariableDeclarationStatement(targetClass.Name, "ret",oce);
            st[1] = new CodeMethodReturnStatement(new CodeVariableReferenceExpression("ret"));

            return st;
        }

        public CodeStatement[] generateSubtraction()
        {

            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");
            CodeStatement[] st = new CodeStatement[2];
            CodeObjectCreateExpression oce = new CodeObjectCreateExpression(targetClass.Name, new CodeExpression[] { new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Subtract, a1), new CodeBinaryOperatorExpression(b, CodeBinaryOperatorType.Subtract, b1) });
            st[0] = new CodeVariableDeclarationStatement(targetClass.Name, "ret", oce);
            st[1] = new CodeMethodReturnStatement(new CodeVariableReferenceExpression("ret"));

            return st;
        }


        public CodeStatement[] generateMul(int value)
        {

            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");
            CodeStatement[] st = new CodeStatement[2];
            //mem0 = mem0*mem0' + mem1*mem1'*2 the rational part:
            CodeBinaryOperatorExpression mem_a1 = new CodeBinaryOperatorExpression(a,CodeBinaryOperatorType.Multiply,b);
            CodeBinaryOperatorExpression mem_a2 = new CodeBinaryOperatorExpression(new CodeBinaryOperatorExpression(a1, CodeBinaryOperatorType.Multiply, b1),CodeBinaryOperatorType.Multiply,new CodePrimitiveExpression(value));// *value
            //mem1 = mem0*mem1' + mem1*mem0'
            CodeBinaryOperatorExpression mem_b1 = new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.Multiply, b1);
            CodeBinaryOperatorExpression mem_b2 = new CodeBinaryOperatorExpression(a1, CodeBinaryOperatorType.Multiply, b);

            CodeObjectCreateExpression oce = new CodeObjectCreateExpression(targetClass.Name, new CodeExpression[] { new CodeBinaryOperatorExpression(mem_a1, CodeBinaryOperatorType.Add, mem_a2), new CodeBinaryOperatorExpression(mem_b1, CodeBinaryOperatorType.Add, mem_b2) });
            st[0] = new CodeVariableDeclarationStatement(targetClass.Name, "ret", oce);
            st[1] = new CodeMethodReturnStatement(new CodeVariableReferenceExpression("ret"));

            return st;
        }

        public CodeStatement[] generateDiv(int value)
        {
            CodeTypeReference t = new CodeTypeReference("Rational");
            CodeTypeReference t1 = new CodeTypeReference(targetClass.Name);
            CodeFieldReferenceExpression a = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");
            CodeStatement[] st = new CodeStatement[4];
            // get the Inverse of b and multiply!
            // a^2 -2b^2
            CodeBinaryOperatorExpression first = new CodeBinaryOperatorExpression(b,CodeBinaryOperatorType.Multiply,b);
            CodeBinaryOperatorExpression sec = new CodeBinaryOperatorExpression(new CodeBinaryOperatorExpression(b1, CodeBinaryOperatorType.Multiply, b1), CodeBinaryOperatorType.Multiply, new CodePrimitiveExpression(value));
            // -b
            CodeFieldReferenceExpression bminus = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("-b"), "mem0");
            CodeVariableDeclarationStatement rat_a = new CodeVariableDeclarationStatement(t,"rat_a",new CodeBinaryOperatorExpression(b, CodeBinaryOperatorType.Divide,new CodeBinaryOperatorExpression(first,CodeBinaryOperatorType.Subtract,sec)));
            CodeVariableDeclarationStatement rat_b = new CodeVariableDeclarationStatement(t,"rat_b", new CodeBinaryOperatorExpression(bminus, CodeBinaryOperatorType.Divide, new CodeBinaryOperatorExpression(first, CodeBinaryOperatorType.Subtract, sec))); 
            st[0] = rat_a;
            st[1] = rat_b;
            st[2] = new CodeVariableDeclarationStatement(t1,"inverse",new CodeObjectCreateExpression(t1, new CodeExpression[] { new CodeVariableReferenceExpression("rat_a"), new CodeVariableReferenceExpression("rat_b") }));
            st[3] = new CodeMethodReturnStatement(new CodeObjectCreateExpression(t1,new CodeBinaryOperatorExpression(new CodeVariableReferenceExpression("a"),CodeBinaryOperatorType.Multiply,new CodeVariableReferenceExpression("inverse"))));
            return st;
        }

        public CodeStatement[]  generateEqual()
        {
            CodeStatement[] st = new CodeStatement[1];
            // (a==a') && (b==b')
          
            CodeFieldReferenceExpression a1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem0");
            CodeFieldReferenceExpression a2 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem0");
            CodeBinaryOperatorExpression a = new CodeBinaryOperatorExpression(a1,CodeBinaryOperatorType.IdentityEquality,a2);
            CodeFieldReferenceExpression b1 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("a"), "mem1");
            CodeFieldReferenceExpression b2 = new CodeFieldReferenceExpression(new CodeVariableReferenceExpression("b"), "mem1");
            CodeBinaryOperatorExpression b = new CodeBinaryOperatorExpression(b1, CodeBinaryOperatorType.IdentityEquality, b2);

            st[0] = new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(a, CodeBinaryOperatorType.BooleanAnd, b));
            return st;
        }


        public CodeStatement[] getMethod(string name)
        {
            CodeTypeReference t = new CodeTypeReference(targetClass.Name);
            CodeVariableReferenceExpression staticRef = new CodeVariableReferenceExpression("Rational");
            CodeStatement[] st = new CodeStatement[2];
            if (name == "Zero") st[0] = new CodeVariableDeclarationStatement(new CodeTypeReference(targetClass.Name), "ret", new CodeObjectCreateExpression(t, new CodeExpression[] { new CodeMethodInvokeExpression(staticRef, "Zero"), new CodeMethodInvokeExpression(staticRef, "Zero") }));
            else st[0] = new CodeVariableDeclarationStatement(new CodeTypeReference(targetClass.Name), "ret", new CodeObjectCreateExpression(t, new CodeExpression[] { new CodeMethodInvokeExpression(staticRef, "One"), new CodeMethodInvokeExpression(staticRef, "Zero") }));
            st[1] = new CodeMethodReturnStatement(new CodeVariableReferenceExpression("ret"));
            return st;
        }

        public CodeMemberMethod getToString()
        {
            CodeMemberMethod method = new CodeMemberMethod();
            method.Comments.Add(new CodeCommentStatement(
            "ToString()"));
            method.Attributes =
                MemberAttributes.Public | MemberAttributes.Override;
            method.Name = "ToString";
            method.ReturnType =
                new CodeTypeReference(typeof(System.String));

            // return mem0.ToString() + (mem1* sqrt(value)).ToString()
            //  return (mem0.ToString()+" + " + mem1+"*"+"sqrt("+Math.Pow(mem2,2)+")");
  
            //CodeMethodInvokeExpression mem0 = new CodeMethodInvokeExpression(new CodeVariableReferenceExpression("mem0"),"ToString");
            //CodeMethodInvokeExpression mem1 = new CodeMethodInvokeExpression(new CodeBinaryOperatorExpression(new CodeVariableReferenceExpression("mem1"), CodeBinaryOperatorType.Multiply, new CodeVariableReferenceExpression("mem2")), "ToString");
            method.Statements.Add(new CodeMethodReturnStatement(new CodeVariableReferenceExpression("mem0.ToString()+ \"+\" + mem1.ToString()+ \"*\"+\"sqrt(\" +Math.Pow(mem2,2)"+"+\")\"")));
            return method;

        }


    }
}
