﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.CodeDom;

namespace Generator_2._0
{
    class GeneratorLMonoid: Generator
    {

        public GeneratorLMonoid(String classname, string t)
            : base(classname, t)
        {
        }

        public bool generateLMono(List<string> attributes, Dictionary<string, string> methodcode)
        {
            Dictionary<int, string> finite_SR = new Dictionary<int, string>();
            for (int i = 0; i < attributes.Count; i++)
            {
                if (attributes[i].Contains("-"))
                {
                    string[] split = attributes[i].Split('-');
                    if (split[0] == "Modulo")
                    {
                        attributes[i] = split[0];
                        finite_SR.Add(i, split[1]);
                    }
                    else
                    {
                        attributes[i] = split[1];
                        finite_SR.Add(i, split[0]);
                    }
                }
            }
            generateAttributes(attributes);

            if (basetypes.Count != attributes.Count) return false;

            // empty constructor, is used by Analysis_Part while creating a Matrix<T> by the corresponding data model
            // Object obj = Activator.CreateInstance(typeof(T), null) to enable checking the user input before creating the Matrix<T>
            // (Creates an instance of the specified type using the constructor that best matches the specified parameters.)
            CodeConstructor constructor = new CodeConstructor();
            constructor.Attributes = MemberAttributes.Public;
            targetClass.Members.Add(constructor);

            // necessary methods and constructors of the new class:
            foreach (KeyValuePair<string, string> entry in methodcode)
            {
                switch (entry.Key)
                {
                    case Constants.LINCOMBI:
                        if (entry.Value.Contains("Largest")) targetClass.Members.Add(getLargestSolution(attributes, targetClass));
                        else
                        {
                            CodeStatement[] lincom = extractMethodsStatements(entry.Value, "LinearCombination", attributes, finite_SR);
                            targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "find linearcombination", "LinearCombination", "LinearCombination<" + targetClass.Name + ">", lincom));
                        }
                        break;
                    case Constants.C1:
                        if (entry.Value.Contains("Componentwise")) targetClass.Members.Add(addConstructorNewClass(attributes, targetClass.Name));
                        else targetClass.Members.Add(addConstructor(targetClass.Name, entry.Key, extractCode(entry.Value), attributes));
                        break;
                    case Constants.C2:
                        if (entry.Value.Contains("Componentwise")) targetClass.Members.Add(addConstructorByAttri(attributes));
                        else targetClass.Members.Add(addConstructor(targetClass.Name, entry.Key, extractCode(entry.Value), attributes));
                        break;
                    case Constants.C3:
                        if (entry.Value.Contains("Componentwise")) targetClass.Members.Add(generateStringConstructor(attributes, finite_SR));
                        else targetClass.Members.Add(addConstructor(targetClass.Name, entry.Key, extractCode(entry.Value), attributes));
                        break;
                    case Constants.Z:
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the Zero Element:", "Zero", targetClass.Name, getMethod(attributes, "Zero()",finite_SR)));
                        break;
                    case Constants.O:
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "Yields the One Element:", "One", targetClass.Name, getMethod(attributes, "One()", finite_SR)));
                        break;
                    case Constants.ADD:
                        CodeStatement[] plus = extractMethodsStatements(entry.Value, "+", attributes, finite_SR);
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : +", "operator +", targetClass.Name, plus));
                        break;
                    case Constants.MUL:
                        CodeStatement[] mul = extractMethodsStatements(entry.Value, "*", attributes, finite_SR);
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : *", "operator *", targetClass.Name, mul));
                        break;
                    case Constants.DIV:
                        CodeStatement[] div = extractMethodsStatements(entry.Value, "/", attributes, finite_SR);
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : /", "operator /", targetClass.Name, div));
                        break;
                    case Constants.EQUAL:
                        CodeStatement[] eq = extractMethodsStatements(entry.Value, "==", attributes, finite_SR);
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : ==", "operator ==", "System.Boolean", eq));
                        break;
                    case Constants.GREATER:
                        CodeStatement[] greater = extractMethodsStatements(entry.Value, ">", attributes, finite_SR);
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : >", "operator >", "System.Boolean", greater));
                        CodeStatement[] lassThan = new CodeStatement[1];
                        //new CodeMethodReturnStatement(new CodeBinaryOperatorExpression(new CodeBinaryOperatorExpression(new CodeVariableReferenceExpression("a"),CodeBinaryOperatorType.LessThan ,new CodeVariableReferenceExpression("b")),CodeBinaryOperatorType.BooleanAnd,new CodeBinaryOperatorExpression()));
                        lassThan[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("return ((b>a) && (a!=b))"));
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : <", "operator <", "System.Boolean", lassThan));
                        lassThan[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("return ((b>a) || (a==b))"));
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : <=", "operator <=", "System.Boolean", lassThan));
                        lassThan[0] = new CodeExpressionStatement(new CodeVariableReferenceExpression("return ((a>b) || (a==b))"));
                        targetClass.Members.Add(addNecessaryOperators(targetClass.Name, "necessary operator : >=", "operator >=", "System.Boolean", lassThan));
                        break;
                }

            }

            return true;
        }




        /*
         * Template algorithm from Zhou / Li: Largest Solution of Linear Equation over the complete heyting algebra:
         * 
            LinearCombination<LatticeZ> lc;
            lc.CombinationExists = true;
            lc.Multiplicands = new LatticeZ[set.Count()];
            for (int j = 0; j < lc.Multiplicands.Length; ++j)
            {
                lc.Multiplicands[j] = One();
                for (int i = 0; i < vec.Length; ++i)
                {
                    //i\in M_j(A,b)?
                    if (!(set[j][i] <= vec[i]))
                        lc.Multiplicands[j] *= vec[i] / set[j][i];
                }
            }
            //Check, if linear combination was found:
            for (int i = 0; i < vec.Length; ++i)
            {
                if (vec[i] != Matrix<LatticeZ>.multiplyVectors(Matrix<LatticeZ>.getColumn(set, i), lc.Multiplicands)) lc.CombinationExists = false;
            }
            return lc;
         * */

        public static CodeMemberMethod getLargestSolution(List<string> attributes, CodeTypeDeclaration targetClass)
        {
            //needed types:
            CodeTypeReference t1 = new CodeTypeReference("LinearCombination<" + targetClass.Name + ">");
            CodeVariableReferenceExpression t3 = new CodeVariableReferenceExpression("Matrix<" + targetClass.Name + ">");
            CodeTypeReference t2 = new CodeTypeReference(targetClass.Name);
            CodeVariableReferenceExpression lc = new CodeVariableReferenceExpression("lc");
            CodeVariableReferenceExpression set = new CodeVariableReferenceExpression("set");
            CodeVariableReferenceExpression vec = new CodeVariableReferenceExpression("vec");

            CodeArrayIndexerExpression setIJ = new CodeArrayIndexerExpression(new CodeArrayIndexerExpression(set, new CodeExpression[] { new CodeVariableReferenceExpression("i") }), new CodeExpression[] { new CodeVariableReferenceExpression("j")});
            CodeArrayIndexerExpression vecI = new CodeArrayIndexerExpression(vec, new CodeVariableReferenceExpression("i"));
            CodeFieldReferenceExpression lcmu = new CodeFieldReferenceExpression(lc, "Multiplicands");
            CodeArrayIndexerExpression lcm = new CodeArrayIndexerExpression(lcmu,new CodeVariableReferenceExpression("j"));

            CodeMemberMethod fl = new CodeMemberMethod();
            fl.Attributes = MemberAttributes.Public | MemberAttributes.Static;
            fl.Name = "LinearCombination";
            fl.ReturnType = t1;
            fl.Parameters.Add(new CodeParameterDeclarationExpression(
                      new CodeTypeReference("List<" + targetClass.Name + "[]>"), "set"));
            fl.Parameters.Add(new CodeParameterDeclarationExpression(
             new CodeTypeReference( targetClass.Name + "[]"), "vec"));

            fl.Statements.Add(new CodeVariableDeclarationStatement(t1,"lc"));
            fl.Statements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(lc, "CombinationExists"), new CodePrimitiveExpression(true)));
            fl.Statements.Add(new CodeAssignStatement(new CodeFieldReferenceExpression(lc, "Multiplicands"), new CodeArrayCreateExpression(t2,new CodeFieldReferenceExpression(set,"Count"))));
            // inside inner for loop
            // lc.Multiplicands[j] =  lc.Multiplicands[j]*(vec[i] / set[j][i]);
            CodeAssignStatement st =   new CodeAssignStatement( lcm,  new CodeBinaryOperatorExpression(lcm, CodeBinaryOperatorType.Multiply,new CodeBinaryOperatorExpression(vecI,CodeBinaryOperatorType.Divide,setIJ)));
            //        if (!(set[j][i] <= vec[i]))
            CodeBinaryOperatorExpression con = new CodeBinaryOperatorExpression(setIJ,CodeBinaryOperatorType.LessThanOrEqual,vecI);
            CodeConditionStatement inside = new CodeConditionStatement(con, new CodeStatement[] { new CodeCommentStatement("Do nothing, CodeDOM !-Expression not available or found") }, new CodeStatement[] { st });

            // inside for loop
            CodeStatement[] insideloop = new CodeStatement[2];
            insideloop[0] = new CodeAssignStatement(lcm, new CodeVariableReferenceExpression("One()"));
            insideloop[1] = addForLoop(new CodeFieldReferenceExpression(vec,"Length"),"i",inside);
            // outer loop
            fl.Statements.Add(addForLoopStatements(new CodeFieldReferenceExpression(lcmu, "Length"), "j", 0, insideloop));
            // checking statement
            CodeMethodInvokeExpression right = new CodeMethodInvokeExpression(t3, "multiplyVectors", new CodeExpression[] {new CodeMethodInvokeExpression(t3,"getColumn",new CodeExpression[] {set, new CodeVariableReferenceExpression("i")}), lcmu });
            CodeAssignStatement truest = new CodeAssignStatement(new CodeFieldReferenceExpression(lc, "CombinationExists"),new CodePrimitiveExpression(false));
            // update to for loop:
            fl.Statements.Add(addForLoop(new CodeFieldReferenceExpression(vec, "Length"), "i", addIF(truest, vecI, right, CodeBinaryOperatorType.IdentityInequality)));
            // the return statement
            fl.Statements.Add(new CodeMethodReturnStatement(lc));

            return fl;
        }
    }
}
